<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Nathan Roth <nroth@interworx.com>
 * @date    Feb 10, 2020
 */

/**
 * Tests the autossl cron.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class CLI_AutosslTest extends IWorx_Framework_TestCase
{
    /**
     * Final cleanup.
     */
    public static function tearDownAfterClass()
    {
        self::deleteSWAccountIfNecessaryStatic();
    }

    /**
     * Faking LE is actually installed. Since we have no live tests yet, this should be fine.
     */
    public function testInstallLE()
    {
        Ini::set(Ini::PLUGINS, 'lets-encrypt', 1);
        Ini::set(Ini::APACHE, 'autossl', 1);
    }
    /**
     * Set a blacklisted domain.
     */
    public function testBlacklistDomain()
    {
        $blacklist = Ini::get(Ini::IWORX_DIR, 'etc') . '/autossl-domain-blacklist.txt';
        //set to staging so we don't get banned.
        Ini::set(Ini::APACHE, 'le_mode', 'staging');
        Ini::write();

        $this->deleteSWAccountIfNecessary();
        $this->addSWAccountIfNecessary();

        IWorxFileSys::smartBarf($blacklist, 'addedbytest.com', false, false, 0600, false, false);

        $cmd = Ini::get(Ini::IWORX_CRON, 'iworx') . ' --run-one Autossl';
        IWorxExec::exec($cmd, $result, $retval);
        $this->assertEquals($retval, 0);

        $cmd = 'tail -n500 ' . Ini::get(Ini::LOG, 'cronfile');
        IWorxExec::exec($cmd, $result, $retval);

        $this->assertFalse(empty(preg_grep('/addedbytest.com is in autossl blacklist./', $result)));
        IWorxFileSys::smartBarf($blacklist, '', false, false, 0600, false, false);
    }

    /**
     * addedbytest.com will fail.
     */
    public function testFailingDomain()
    {
        $cmd = '/home/interworx/cron/iworx.pex --run-one Autossl';
        IWorxExec::exec($cmd, $result, $retval);
        $this->assertEquals($retval, 0);

        $cmd = 'tail -n500 /home/interworx/var/log/cron.log';
        IWorxExec::exec($cmd, $result, $retval);

        $this->assertFalse(empty(preg_grep('/Autossl failed for: addedbytest.com: DNS check failed/', $result)));
    }

    /**
     * addedbytest.com will fail.
     */
    public function testDomainWithoutSslAccess()
    {
        $this->_disableSslAccess();
        $cmd = '/home/interworx/cron/iworx.pex --run-one Autossl';
        IWorxExec::exec($cmd, $result, $retval);
        $this->assertEquals($retval, 0);

        $cmd = 'tail -n500 /home/interworx/var/log/cron.log';
        IWorxExec::exec($cmd, $result, $retval);

        $this->assertFalse(empty(preg_grep('/addedbytest.com does not have SSL access./', $result)));
    }

    /**
     * Enable SSL Access for addedbytest.com'
     */
    private function _disableSslAccess()
    {
        $NW = self::getMasterNodeWorx();
        $input = array('domain' => 'addedbytest.com',
                       'OPT_SSL' => 0);
        IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Siteworx',
            'edit',
            $input,
            $NW
        );
    }

    /**
     * Test a working domain.
     */
    public function testWorkingDomain()
    {
        $this->markTestSkipped('Unable to test live domains at this time');
        $domain = 'newdomain.com';
        $this->deleteSWAccountIfNecessary();
        $this->addSWAccount(false, ['master_domain' => $domain]);

        $cmd = '/home/interworx/cron/iworx.pex --run-one Autossl';
        IWorxExec::exec($cmd, $result, $retval);
        $this->assertEquals($retval, 0);

        $SW = new SiteWorx($domain);
        $this->assertTrue($SW->getMasterDomain()->isSSL());
        $this->deleteSWAccountStatic($domain);
    }

    /**
     *  test _sslNoInstalled
     */
    public function testsslNotInstalledMethod()
    {
        $this->deleteSWAccountIfNecessary();
        $this->addSWAccountIfNecessary();

        $object = new Cron_Autossl('run-one');
        $result = $this->invokeMethod($object, '_sslNotInstalled');
        $this->assertTrue(in_array('addedbytest.com', $result));
    }

    /**
     * Adds self signed certificate to addedbytest.com
     *
     * @throws IWorx_Exception
     */
    public function testSettingSelfSignedCert()
    {
        $csr_test_data = $this->_getCsrTestData();
        $SW = new SiteWorx(self::SW_DOMAIN);
        $Ssl = new SSL($SW);
        $Ssl->setDomain('addedbytest.com');
        $Ssl->generatePrivateKey();
        $Ssl->generateCSR($csr_test_data);
        $Ssl->generateCertificate();
        //write the ssl certs to the vhost
        $domain = IW::SW()->getWorkingDomainName();
        $dir    = $SW->getLocalStateDir(SiteWorx::DIR_SSL);
        $cmd = Ini::get(Ini::APACHE_BIN, 'httpd.php') .
               " --domain=$domain " .
               " --ssldir=$dir " .
               ' --add-ssl ';

        IWorxExec::exec($cmd, $result, $retval, IWorxExec::NO_OUTPUT);

        //clear apache vhost cache
        $VHR = ApacheVhostReader::getInstance();
        $VHR->clearVHostCache();

        //make sure the domain is reporting that it is ssl
        $Domain = new Domain('addedbytest.com');
        $this->assertTrue($Domain->canSSL());
        $this->assertTrue($Domain->isSSL());
    }

    /**
     * test _sslIsSelfSigned
     *
     * @throws IWorx_Exception
     */
    public function testSslIsSelfSigned()
    {
        $object = new Cron_Autossl('run-one');
        $result = $this->invokeMethod($object, '_sslIsSelfSigned');
        $this->assertTrue(in_array('addedbytest.com', $result));
    }

    /**
     * test _sslCpanelAutoSsl
     */
    public function testSslCpanelAutoSslMethod()
    {
        //random cpanel cert pulled off a random domain.
        $cpanel_cert = $this->_getCpanelTestCert();

        IWorxFileSys::smartBarf('/home/addedbyt/var/addedbytest.com/ssl/addedbytest.com.crt', $cpanel_cert, false, false, 0600, false, false);

        $object = new Cron_Autossl('run-one');
        $result = $this->invokeMethod($object, '_cpanelAutoSsl');
        $this->assertTrue(in_array('addedbytest.com', $result));
    }

    /**
     * test _letsEncryptAutoRenew
     */
    public function testLetsEncryptAutoRenewMethod()
    {
        $le_cert = $this->_getLETestCert();
        IWorxFileSys::smartBarf('/home/addedbyt/var/addedbytest.com/ssl/addedbytest.com.crt', $le_cert, false, false, 0600, false, false);
        $object = new Cron_Autossl('run-one');
        $result = $this->invokeMethod($object, '_letsEncryptAutoRenew');
        $this->assertTrue(in_array('addedbytest.com', $result));
    }

    /**
     * Retruns csr test data
     *
     * @return array
     */
    protected function _getCsrTestData()
    {
        return array (
          'countryName' => 'US',
          'stateOrProvinceName' => 'PA',
          'localityName' => 'Pittsburgh',
          'organizationName' => 'InterWorx',
          'organizationalUnitName' => 'InterWorx',
          'commonName' => 'addedbytest.com',
          'subjectAltName' =>
            array (
              0 => 'addedbytest.com',
            ),
          'emailAddress' => 'test@example.com',
        );
    }

    /**
     * Returns a cPanel test certificate
     *
     * @return string
     */
    protected function _getCpanelTestCert()
    {
        return '-----BEGIN CERTIFICATE-----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-----END CERTIFICATE-----';
    }

    /**
     * Returns a test LE cert
     *
     * @return string
     */
    protected function _getLETestCert()
    {
        return '-----BEGIN CERTIFICATE-----
MIIFWzCCBEOgAwIBAgISAyWvrsvvFLmSyNcYI1GJHgDaMA0GCSqGSIb3DQEBCwUA
MEoxCzAJBgNVBAYTAlVTMRYwFAYDVQQKEw1MZXQncyBFbmNyeXB0MSMwIQYDVQQD
ExpMZXQncyBFbmNyeXB0IEF1dGhvcml0eSBYMzAeFw0yMDAyMTExMzI2MzVaFw0y
MDA1MTExMzI2MzVaMB0xGzAZBgNVBAMTEml3b3J4LXNzbC10ZXN0LmNvbTCCASIw
DQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAL2uQgvXHlOJsMQKXEahUlbYQ+62
Lm7NphwtoqE2inDcAjTqXl7JsDScT6AifTDilTQEY7ItXGjCWyQ0DSzpp3cmVqPe
0eFlmNJnCtsXwvFlqi9TBmozz2SYEaWGZ3dhT7p+np7ubqEKEVYfirMBavX33WTh
Lx43j9p4LzRz8ZrGs0B5n8ipiiIfu9FV+Z9GN1Lf9KyYC68aBcdt5PZHTWZbRsOY
Dt/+FxgkYDT/q84dehHQUT4VJ5CcFCTKrTC450URvvQsStAn3wdZSY4/4Bv4EU+Y
0lVMeJLR94h9Mei7wnsGGEoKlpbpC8BaFBzbcG/fNOTf2I6boMWnuOuVIU0CAwEA
AaOCAmYwggJiMA4GA1UdDwEB/wQEAwIFoDAdBgNVHSUEFjAUBggrBgEFBQcDAQYI
KwYBBQUHAwIwDAYDVR0TAQH/BAIwADAdBgNVHQ4EFgQUjno7IBtLqTNq+jE8tYxy
mnmgqBkwHwYDVR0jBBgwFoAUqEpqYwR93brm0Tm3pkVl7/Oo7KEwbwYIKwYBBQUH
AQEEYzBhMC4GCCsGAQUFBzABhiJodHRwOi8vb2NzcC5pbnQteDMubGV0c2VuY3J5
cHQub3JnMC8GCCsGAQUFBzAChiNodHRwOi8vY2VydC5pbnQteDMubGV0c2VuY3J5
cHQub3JnLzAdBgNVHREEFjAUghJpd29yeC1zc2wtdGVzdC5jb20wTAYDVR0gBEUw
QzAIBgZngQwBAgEwNwYLKwYBBAGC3xMBAQEwKDAmBggrBgEFBQcCARYaaHR0cDov
L2Nwcy5sZXRzZW5jcnlwdC5vcmcwggEDBgorBgEEAdZ5AgQCBIH0BIHxAO8AdgDn
EvKwN34aYvuOyQxhhPHqezfLVh0RJlvz4PNL8kFUbgAAAXA0pQiiAAAEAwBHMEUC
ICBBa23b5nZIWBFg7MBwL/IW/7QdSM7LrbxOlV3NBQIOAiEAz1hN90Lp1DCvxV3H
Si+JXdaD9uKjgs+u/kFT9uVL7IcAdQAHt1wb5X1o//Gwxh0jFce65ld8V5S3au68
YToaadOiHAAAAXA0pQjPAAAEAwBGMEQCIBM7nD1stIUpmZx+PV57HBdQzWo1KJA+
szFRMZfXfp0RAiATMBG7BIP4vddwralE2mMQgFd1cgUNlHobUMc6VPlkkDANBgkq
hkiG9w0BAQsFAAOCAQEAMGocrQ8RoA30kQAbestTN5fWSqbMM0S7xeBMbTPzvQuu
X5HFz2oVihsLKtABMEOYTMr6qhbshFhsHFVVHUwxSaBG+CwMp9NZ7kg6196BwWWT
AVPeSSfy60yQwYCZyAEcNreXJSUz15sXaI+rcIdTSP7nTRLSUnU/Lcl/pN+TXu7e
+8z8NmE6giR0x51nZqyzlCxqEC2Vv3hJix7aoXjrgMk6uIHVqqwtyJfRx12Kyvwq
yeb303rWDDn5eKhKy4cSYz7VirS8IOueSHDHFlajzPfmXnFfwRDZx/143W35GhCB
wjsIR0PZVvhJLVJyTho6HU6VPGO4pI1Dpxi+bWFmtg==
-----END CERTIFICATE-----';
    }
}
