<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Nathan Roth <nroth@interworx.com>
 * @date    Apr 11, 2019
 */

/**
 * Test the --preserve-siteworx-account flag on import
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class BackupRestore_PreserveSiteWorxTest extends IWorx_Framework_TestCase
{

    /**
     * Tests that a full backup will properly restore.
     */
    public function testPreserveSiteWorxAccount()
    {
        //cleanup
        $this->deleteSWAccountIfNecessary();

        //create first account
        $input = $this->_getSWAccountInput();
        $input['uniqname'] = 'backup';
        $this->addSWAccountFromInput($input);
        $this->addSecondaryDomainIfNecessary();

        //ensure the secondary domain exists
        $SW = new SiteWorx(self::SW_DOMAIN);
        $this->assertTrue($SW->slaveDomainExists(self::SW_SECONDARY_DOMAIN));

        //ensure the pointer domain exists
        $SW->addPointerDomain(self::SW_POINTER_DOMAIN, self::SW_DOMAIN);
        $this->assertTrue($SW->pointerDomainExists(self::SW_POINTER_DOMAIN));

        //ensure the subdomain exists
        $SW->addSubdomain('sub');
        $this->assertTrue($SW->subDomainExists('sub.' . self::SW_DOMAIN));

        //create backup of first account
        $this->_doFullBackup('backup');

        //delete first account
        $this->deleteSWAccount();

        //create second account
        $input['uniqname'] = 'existing';
        $this->addSWAccountFromInput($input);

        //ensure the secondary domain doesn't exist
        $this->deleteSecondaryDomainIfNecessary();
        $SW = new SiteWorx(self::SW_DOMAIN);
        $this->assertFalse($SW->slaveDomainExists(self::SW_SECONDARY_DOMAIN));

        //ensure the pointer domain doesn't exist
        $this->assertFalse($SW->pointerDomainExists(self::SW_POINTER_DOMAIN));

        //ensure the subdomain doesn't exist
        $this->assertFalse($SW->subDomainExists('sub.' . self::SW_DOMAIN));

        //restore first account over second without deleting
        $this->_doFullRestore('backup', '_verifyRestoreSucceeded');

        //username should still be from second account
        $SW = new SiteWorx(self::SW_DOMAIN);
        $this->assertEquals('existing', $SW->getUniqname());

        //secondary domain should now exist
        $SW = new SiteWorx(self::SW_DOMAIN);
        $this->assertTrue($SW->slaveDomainExists(self::SW_SECONDARY_DOMAIN));

        //pointer domain should now exist
        $this->assertTrue($SW->pointerDomainExists(self::SW_POINTER_DOMAIN));

        //subdomain should now exist
        $this->assertTrue($SW->subDomainExists('sub.' . self::SW_DOMAIN));

        //cleanup
        $this->deleteSWAccountIfNecessary();
    }


    /**
     * Verify that the restore was successful.
     *
     * @param array   $result
     * @param integer $retval
     */
    private function _verifyRestoreSucceeded(array $result, $retval)
    {
        $this->assertEquals(0, $retval, implode("\n", $result));
    }

    /**
     * Verifies that the restore failed.
     *
     * @param array   $result
     * @param integer $retval
     */
    private function _verifyRestoreFailed(array $result, $retval)
    {
        $this->assertNotEquals(0, $retval, implode("\n", $result));
    }

    /**
     * Do the actual restore.
     *
     * Accepts a callback to do additional testing against the $result and
     * $retval arguments return from the exec.
     *
     * @param string $backup_name
     * @param string $callback
     * @return boolean
     */
    private function _doFullRestore($backup_name, $callback = null)
    {
        $import = Ini::get(Ini::IWORX_BIN, 'import');
        $tmp    = Ini::get(Ini::DIR, 'tmp');

        $cmd = $import  .
               " --archive {$tmp}/{$backup_name}.tgz" .
               ' --control-panel siteworx' .
               ' --ip-address ' . self::_getFirstSharedIP() .
               ' --preserve-siteworx-account' .
               ' --force' . //to avoid y/n when importing over account
               ' --reseller-id=1';

        IWorxExec::exec($cmd, $result, $retval);

        if (!is_null($callback)) {
            $this->{$callback}($result, $retval);
        }

        return $retval == 0;
    }

    /**
     * Create a partial backup of the test SiteWorx account.
     *
     * @param string $backup_name
     * @return boolean
     */
    private function _doFullBackup($backup_name)
    {
        $SiteWorx = $this->getSWAccount();
        $tmp      = Ini::get(Ini::DIR, 'tmp');
        $filename = $backup_name;

        $backup = Ini::get(Ini::IWORX_BIN, 'backup');

        $cmd = $backup .
               " --domains {$SiteWorx->getMasterDomainName()}" .
               " --output-dir {$tmp}" .
               " --filename-format {$filename}";

        IWorxExec::exec($cmd, $result, $retval);

        return $retval == 0;
    }
}
