<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Nathan Roth <nroth@interworx.com>
 * @date    Jan 23, 2020
 */

/**
 * Test various aspects about partial backups and their restoration.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class BackupRestore_s3BackupTest extends IWorx_Framework_TestCase
{
    /**
     * Tests that credentials can be set and updated.
     */
    public function testSetCredentials()
    {
        $profile = 'testprofile';
        $credential_file = '/tmp/testcredentialfile';
        $aws_credential_file = Ini::get(Ini::IWORX_BACKUP, 's3_credential_file');
        $aws_config_file = Ini::get(Ini::IWORX_BACKUP, 's3_config_file');

        if (!is_dir(dirname(Ini::get(Ini::IWORX_BACKUP, 's3_credential_file')))) {
            mkdir(dirname(Ini::get(Ini::IWORX_BACKUP, 's3_credential_file')));
        }

        file_put_contents($aws_credential_file, '');
        file_put_contents($aws_config_file, '');

        $test_credentials['testprofile'] = ['aws_access_key_id' => '123456789012',
                                            'aws_secret_access_key' => 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKE'];

        //write credentials
        $this->_writeCredentialFile($credential_file, $test_credentials, $profile);

        $this->assertEquals($test_credentials, parse_ini_file($credential_file, true));
        $this->assertEquals($test_credentials, parse_ini_file($aws_credential_file, true));

        $test_credentials2['testprofile'] = ['aws_access_key_id' => '343243242343',
                                            'aws_secret_access_key' => 'SOMETHINGDIFFERENT/K7MDENG/bPxRfiCYEXAMPLEKE'];

        //update credentials
        $this->_writeCredentialFile($credential_file, $test_credentials2, $profile);
        $this->assertEquals($test_credentials2, parse_ini_file($credential_file, true));
        $this->assertNotEquals($test_credentials, parse_ini_file($credential_file, true));
        $this->assertEquals($test_credentials2, parse_ini_file($aws_credential_file, true));
    }

    /**
     * Tests that configs can be set and updated.
     */
    public function testSetConfig()
    {
        $profile = 'testprofile';
        $credential_file = '/tmp/testcredentialfile';
        $aws_credential_file = Ini::get(Ini::IWORX_BACKUP, 's3_credential_file');
        $aws_config_file = Ini::get(Ini::IWORX_BACKUP, 's3_config_file');
        $test_credentials['testprofile'] = ['aws_access_key_id' => '123456789012',
                                            'aws_secret_access_key' => 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKE'];

        //cleanup
        file_put_contents($aws_credential_file, '');
        file_put_contents($aws_config_file, '');

        //set config
        $args = ['region' => 'us-east-2'];

        //setup credential file so profile name is legit.
        $this->_writeCredentialFile($credential_file, $test_credentials, $profile, $args);

        //start of actual config testing


        $this->_writeConfigFile($profile, $args);
        $config = parse_ini_file($aws_config_file);
        $this->assertEquals($config['region'], 'us-east-2');

        //update config
        $args = ['region' => 'us-east-1'];
        $this->_writeConfigFile($profile, $args);
        $config = parse_ini_file($aws_config_file);
        $this->assertEquals($config['output'], 'json');
        $this->assertEquals($config['region'], 'us-east-1');

        //fail region
        $args = ['region' => 'NOTAVALIDREGION'];
        $result = $this->_writeConfigFile($profile, $args);
        $config = parse_ini_file($aws_config_file);
        $this->assertEquals($config['region'], 'us-east-1');
        $this->assertEquals($result[1], 'NOTAVALIDREGION is not a valid region.');
    }

    /**
     * Tests setting a target.
     */
    public function testAddTarget()
    {
        $profile = 'testprofile';
        $credential_file = '/tmp/testcredentialfile';
        $aws_credential_file = Ini::get(Ini::IWORX_BACKUP, 's3_credential_file');
        $aws_config_file = Ini::get(Ini::IWORX_BACKUP, 's3_config_file');
        $test_credentials['testprofile'] = ['aws_access_key_id' => '123456789012',
                                            'aws_secret_access_key' => 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKE'];

        //cleanup
        file_put_contents($aws_credential_file, '');
        file_put_contents($aws_config_file, '');


        //setup credential file and config.
        $this->_writeCredentialFile($credential_file, $test_credentials, $profile);
        $args = ['region' => 'us-east-2'];
        $this->_writeConfigFile($profile, $args);

        $args = $this->_setTargetArgs('target-name', $profile, 'bucketname', 'path');
        $this->_addTarget($profile, $args);

        $results = $this->_listTargets();
        $results = json_decode($results[0], true);
        $this->assertNotEmpty($results);
        foreach ($results as $result) {
            $this->_deleteTarget($result['name']);
        }

        $results = $this->_listTargets();
        $results = json_decode($results[0], true);
        $this->assertEmpty($results);


        //actual start of target testing
        $this->_addTarget($profile, $args);
        $results = $this->_listTargets();
        $results = json_decode($results[0], true)[0];
        $this->assertEquals($results['name'], $args['target']);

        $args = ['profile' => $profile, 'bucket' => 'bucketname', 'path' => 'path'];
        $result = $this->_addTarget($profile, $args);
        $this->assertEquals('--add-target option requires --target', $result[0]);

        $args = ['target' => 'target-name', 'profile' => $profile, 'path' => 'path'];
        $result = $this->_addTarget($profile, $args);
        $this->assertEquals('--add-target option requires --bucket', $result[0]);
    }

    /**
     *
     * Setup target arguments
     *
     * @param string $target
     * @param string $profile
     * @param string $bucket
     * @param string $path
     * @return array
     */
    private function _setTargetArgs($target, $profile, $bucket, $path)
    {
        return [
          'target' => $target,
          'profile '=> $profile,
          'bucket' => $bucket,
          'path' => $path
        ];
    }

    /**
     *
     * Delete a target.
     *
     * @param string $name
     * @return mixed
     */
    private function _deleteTarget($name)
    {
        $configure_s3 = Ini::get(Ini::IWORX_BIN, 's3');
        $cmd = "{$configure_s3} --delete-target --target {$name}";
        IWorxExec::exec($cmd, $result, $retval);
        return $result;
    }

    /**
     * Set a target.
     *
     * @param string $profile
     * @param array  $args
     *
     * @return mixed
     */
    private function _addTarget($profile, array $args)
    {
        $configure_s3 = Ini::get(Ini::IWORX_BIN, 's3');
        $cmd = "{$configure_s3} --add-target --profile {$profile} --test";
        foreach ($args as $key => $value) {
            $cmd .= " --{$key} {$value}";
        }
        IWorxExec::exec($cmd, $result, $retval);
        return $result;
    }

    /**
     * Set a target.
     *
     */
    private function _listTargets()
    {
        $configure_s3 = Ini::get(Ini::IWORX_BIN, 's3');
        $cmd = "{$configure_s3} --list-targets --json";
        IWorxExec::exec($cmd, $result, $retval);
        return $result;
    }

    /**
     * Write the config values for the provided profile.
     *
     * @param string $profile
     * @param array  $args
     *
     * @return mixed
     */
    private function _writeConfigFile($profile, array $args)
    {
        $configure_s3 = Ini::get(Ini::IWORX_BIN, 's3');
        $cmd = "{$configure_s3} --configure --profile {$profile} --test";
        foreach ($args as $key => $value) {
            $cmd .= " --{$key} {$value}";
        }
        IWorxExec::exec($cmd, $result, $retval);
        return $result;
    }

    /**
     * Writes the credential data.
     *
     * @param string $file
     * @param array  $test_data
     * @param string $profile
     * @param array  $args

     */
    private function _writeCredentialFile($file, array $test_data, $profile, array $args = [])
    {
        $configure_s3 = Ini::get(Ini::IWORX_BIN, 's3');
        INIConfig::writeConfig($test_data, $file, false, true, false, false);

        $cmd = "{$configure_s3} --configure --credential-file {$file} --profile {$profile} --test";
        foreach ($args as $key => $value) {
            $cmd .= " --{$key} {$value}";
        }
        IWorxExec::exec($cmd, $result, $retval);
    }
}
