<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2015 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Nov 30, 2015
 */

/**
 * Class CLI_LogTest
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class CLI_LogTest extends IWorx_Framework_TestCase
{

    /**
     * Confirm that Monolog is styling the log_level properly.
     * Logs should set the log_level the same way Zend_log did.
     */
    public function testConfirmZendStyleLogLevel()
    {
        // used in assertLogEntryExists() to confirm the log level written in the log is in the Zend_log format.
        $lvl = IW::getLog()->getZendConstantStringFromMonologConstant(Monolog\Logger::EMERGENCY);
        $this->assertSame($lvl, 'EMERG');

        // used in the log.pex to convert the string level back to a constant before writing the log.
        $lvl = IW::getLog()->getMonologLvlFromZendConstantString("EMERG");
        $this->assertSame($lvl, Monolog\Logger::EMERGENCY);

        // used in Monolog to actually write the Zend_log style log_level into the logs
        $lvl = IW::getLog()->getZendConstantStringFromMonologConstantString('EMERGENCY');
        $this->assertSame($lvl, 'EMERG');
    }

    /**
     * Test that the log.pex works.
     */
    public function testLogMessageWithDefaultLevel()
    {
        IW::getLog()->info('logging to get a request id set');

        $message = 'Test message ' . __FUNCTION__;
        $log     = Ini::get(Ini::IWORX_BIN, 'log');
        $Cmd     = new Command("{$log} --message '{$message}'");
        $Cmd->allowQuoteSingle();
        $Cmd->exec();

        $log_id = 'logpex-' . IW::Env()->getLogRequestId();
        $this->assertLogEntryExists($message, $log_id, null, Monolog\Logger::INFO);
    }

    /**
     * Test that the log.pex works with custom levels.
     */
    public function testLogMessageWithCustomLevel()
    {
        IW::getLog()->info('logging to get a request id set');

        $message = 'Test message ' . __FUNCTION__;
        $log     = Ini::get(Ini::IWORX_BIN, 'log');
        $Cmd     = new Command("{$log} --message '{$message}' --level ALERT");
        $Cmd->allowQuoteSingle();
        $Cmd->exec();

        $log_id = 'logpex-' . IW::Env()->getLogRequestId();
        $this->assertLogEntryExists($message, $log_id, null, Monolog\Logger::ALERT);
    }

    /**
     * Test that the log.pex works with custom request ids.
     */
    public function testLogMessageWithCustomRequestId()
    {
        IW::getLog()->info('logging to get a request id set');

        $message = 'Test message ' . __FUNCTION__;
        $log     = Ini::get(Ini::IWORX_BIN, 'log');
        $cmd     = "{$log} --message '{$message}' --level ALERT --request_id test";
        $Cmd     = new Command($cmd);
        $Cmd->allowQuoteSingle();
        $Cmd->exec();

        $log_id = 'logpex-test';
        $this->assertLogEntryExists($message, $log_id, null, Monolog\Logger::ALERT);
    }

    /**
     * Test to confirm that the IWorxLog class still works while routing through monolog
     */
    public function testIWorxLog()
    {
        $message = 'Test using IWorxLog::log ' . __FUNCTION__;
        IWorxLog::log($message);
        $this->assertLogEntryExists($message);
    }

    /**
     * Test that debugLog writes to the log when debug is on.
     */
    public function testIWorxLogDebugLog()
    {
        if (IW::Env()->isDebugOn()) {
            $initial_debug_state = Ini::get(Ini::IWORX, 'debug');

            Ini::set(Ini::IWORX, 'debug', '1');

            $this->assertTrue(IW::Env()->isDebugOn());

            $message = 'Test using IWorxLog::debugLog ' . __FUNCTION__;
            IWorxLog::debugLog($message, IWorxLog::DEBUG);
            $this->assertLogEntryExists($message, '', null, Monolog\Logger::DEBUG);

            Ini::set(Ini::IWORX, 'debug', $initial_debug_state);
        } else {
            // Skipping because something is seriously weird, but it works if debug is
            // on before the test runs. Turning debug on mid-process is the problem
            // somehow.
            $this->markTestSkipped('Everything is stupid and you\'re stupid too');
        }
    }

    /**
     * Test logging to cron.log
     */
    public function testLogToCronlog()
    {
        $cron_log = Ini::get(Ini::LOG, 'cronfile');
        $message = 'Test message ' . __FUNCTION__;
        IW::getLog()->info("Logging to cron for test");

        $php = Ini::get(Ini::IWORX_BIN, 'php');
        $log = Ini::get(Ini::IWORX_DIR, 'cron').'/log.php';

        $Cmd = new Command("$php $log --message '{$message}' --level ALERT");
        $Cmd->allowQuoteSingle();
        $Cmd->exec();

        $this->assertLogEntryExists($message, '', null, Monolog\Logger::ALERT, $cron_log);
    }
}
