<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2013 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Tim College <tcollege@interworx.com>
 * @date       Nov 7, 2013
 */

/**
 * Tests of registered functions.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class CLI_RegisteredFunctionsTest extends IWorx_Framework_TestCase
{
    /**
     * Test an actual function, old style.
     */
    public function testRealFunction()
    {
        $cmd = "{$this->_cmd()} --actual-function";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(CLI::OK, $retval);
        $this->assertEquals(array( 'actual function' ), $result);
    }

    /**
     * Test an actual function alias, old style.
     */
    public function testRealFunctionAlias()
    {
        $cmd = "{$this->_cmd()} -f";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(CLI::OK, $retval);
        $this->assertEquals(array( 'actual function' ), $result);
    }

    /**
     * Tests that missing requirements fails appropriately.
     */
    public function testMissingRequirements()
    {
        $cmd = "{$this->_cmd()} --actual-function-with-requirements";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(CLI::PARAM_REQUIRES_CODE, $retval);
    }

    /**
     * Tests that requirements succeed when present.
     */
    public function testRequirements()
    {
        $cmd = "{$this->_cmd()} --actual-function-with-requirements --required-thing 1";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(CLI::OK, $retval);
    }

    /**
     * Test a magic __call function.
     */
    public function testRegisteredFunction()
    {
        $cmd = "{$this->_cmd()} --hello-world";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(CLI::OK, $retval);
    }

    /**
     * Test an unknown function.
     */
    public function testUnknownFunction()
    {
        $cmd = "{$this->_cmd()} --not-real-thing";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(CLI::NO_ACTION_FOUND, $retval);
    }

    /**
     * Test resolution of relative directory paths.
     */
    public function testRelativeDirectory()
    {
        touch('/home/interworx/tmp/test-file');
        $cwd = getcwd();
        chdir('/home/interworx/bin');

        $cmd = "{$this->_cmd()} --test-dir --actual-directory=../tmp";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        chdir($cwd);

        $this->assertEquals(CLI::OK, $retval);
        $this->assertEquals('/usr/local/interworx/tmp', $result[0]);
    }

    /**
     * Test resolution of relative file paths.
     */
    public function testRelativeFile()
    {
        touch('/home/interworx/tmp/test-file');
        $cwd = getcwd();
        chdir('/home/interworx/bin');

        $cmd = "{$this->_cmd()} --test-file --actual-file=../tmp/test-file";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        chdir($cwd);

        $this->assertEquals(CLI::OK, $retval);
        $this->assertEquals('/usr/local/interworx/tmp/test-file', $result[0]);
    }

    /**
     * Test resolution of file paths with a tilde and explicit user.
     */
    public function testUserDirectoryTildeFile()
    {
        $cmd = "{$this->_cmd()} --test-file --actual-file=~iworx/bin/php-fpm.php";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(CLI::OK, $retval);
        $this->assertEquals('/usr/local/interworx/bin/php-fpm.php', $result[0]);
    }

    /**
     * Test resolution of directory paths with a tilde and explicit user.
     */
    public function testUserDirectoryTildeDirectory()
    {
        $cmd = "{$this->_cmd()} --test-dir --actual-directory=~iworx/bin/";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(CLI::OK, $retval);
        $this->assertEquals('/usr/local/interworx/bin', $result[0]);
    }

    /**
     * Test resolution of directory paths with a tilde and explicit user sans /.
     */
    public function testUserDirectoryTildeDirectorySansSlash()
    {
        $cmd = "{$this->_cmd()} --test-dir --actual-directory=~iworx";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(CLI::OK, $retval);
        $this->assertEquals('/usr/local/interworx', $result[0]);
    }

    /**
     * Test resolution of file paths with a tilde and no explicit user.
     */
    public function testMyUserDirectoryTildeFile()
    {
        touch('/root/iw-test-realfile');

        $cmd = "{$this->_cmd()} --test-file --actual-file=~/iw-test-realfile";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(CLI::OK, $retval);
        $this->assertEquals('/root/iw-test-realfile', $result[0]);
    }

    /**
     * Test resolution of directory paths with a tilde and no explicit user.
     */
    public function testMyUserDirectoryTildeDirectory()
    {
        $cmd = "{$this->_cmd()} --test-dir --actual-directory=~/";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(CLI::OK, $retval);
        $this->assertEquals('/root', $result[0]);
    }

    /**
     * Gets the command.
     *
     * @return string
     */
    private function _cmd()
    {
        return '/home/interworx/bin/test-class.pex';
    }
}
