<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2018 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Dec 06, 2018
 */

/**
 * Class ConfigurableFallbackTest
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class ConfigurableFallbackTest extends IWorx_Framework_TestCase
{
    const TEST_VALUE_NAME     = 'IW_TEST_VALUE';
    const TEST_VALUE_DOMAIN   = 'domain';
    const TEST_VALUE_PACKAGE  = 'package';
    const TEST_VALUE_NODEWORX = 'nodeworx';
    const TEST_VALUE_ENV      = 'global';
    const TEST_VALUE_SERVER   = 'server';
    const TEST_VALUE_DEFAULT  = 'default';

    /**
     * Get a chain of configurable objects.
     *
     * @return IWorx_Configurable[]
     */
    public function dataProviderConfigurables()
    {
        return [
           [IW::Env()],
           [new Server()],
           [NodeWorx::getDefaultNodeWorx()],
           [$this->getSWAccount()],
           [$this->getSWAccount()->getPackage()],
           [$this->getSWAccount()->getMasterDomain()]
        ];
    }

    /**
     * Clear things out.
     */
    public static function setUpBeforeClass()
    {
        $q_name = q(self::TEST_VALUE_NAME);
        $sql = "DELETE FROM config_defaults WHERE config_name = {$q_name}";
        IWorxDbUtil::db_query(IW::DB(), $sql);

        $sql = "DELETE FROM config WHERE config_name = {$q_name}";
        IWorxDbUtil::db_query(IW::DB(), $sql);
    }

    /**
     * @param IWorx_Configurable $Configurable
     * @dataProvider dataProviderConfigurables
     */
    public function testDefaultIsNullInitially(IWorx_Configurable $Configurable)
    {
        $this->assertNull($Configurable->getConfig(self::TEST_VALUE_NAME));
    }

    /**
     * Test setting the global value.
     */
    public function testSettingGlobal()
    {
        IW::Env()->setConfig(self::TEST_VALUE_NAME, self::TEST_VALUE_ENV);
        $this->assertEquals(self::TEST_VALUE_ENV, IW::Env()->getConfig(self::TEST_VALUE_NAME));
    }

    /**
     * @param IWorx_Configurable $Configurable
     * @dataProvider dataProviderConfigurables
     * @depends testSettingGlobal
     */
    public function testGlobalValueInheritance(IWorx_Configurable $Configurable)
    {
        $this->assertEquals(self::TEST_VALUE_ENV, $Configurable->getConfig(self::TEST_VALUE_NAME));
    }

    /**
     * Test setting the server value.
     */
    public function testSettingServer()
    {
        $Server = new Server();
        $Server->setConfig(self::TEST_VALUE_NAME, self::TEST_VALUE_SERVER);
        $this->assertEquals(self::TEST_VALUE_SERVER, $Server->getConfig(self::TEST_VALUE_NAME));
    }

    /**
     * @depends testSettingServer
     */
    public function testServerValueInheritance()
    {
        $NW = NodeWorx::getDefaultNodeWorx();

        $this->assertEquals(self::TEST_VALUE_ENV, IW::Env()->getConfig(self::TEST_VALUE_NAME));
        $this->assertEquals(self::TEST_VALUE_SERVER, $NW->getConfig(self::TEST_VALUE_NAME));

        $SW = $this->getSWAccount();
        $this->assertEquals(self::TEST_VALUE_SERVER, $SW->getConfig(self::TEST_VALUE_NAME));

        $Domain = $SW->getMasterDomain();
        $this->assertEquals(self::TEST_VALUE_SERVER, $Domain->getConfig(self::TEST_VALUE_NAME));
    }

    /**
     * Test effects of overwriting at the reseller level.
     *
     * @depends testSettingGlobal
     */
    public function testResellerValueInheritance()
    {
        $NW = NodeWorx::getDefaultNodeWorx();
        $NW->setConfig(
            self::TEST_VALUE_NAME,
            self::TEST_VALUE_NODEWORX
        );

        $this->assertEquals(self::TEST_VALUE_ENV, IW::Env()->getConfig(self::TEST_VALUE_NAME));
        $this->assertEquals(self::TEST_VALUE_NODEWORX, $NW->getConfig(self::TEST_VALUE_NAME));

        $SW = $this->getSWAccount();
        $this->assertEquals(self::TEST_VALUE_NODEWORX, $SW->getConfig(self::TEST_VALUE_NAME));

        $Domain = $SW->getMasterDomain();
        $this->assertEquals(self::TEST_VALUE_NODEWORX, $Domain->getConfig(self::TEST_VALUE_NAME));
    }

    /**
     * Test effects of overwriting at the siteworx level.
     *
     * @depends testResellerValueInheritance
     */
    public function testSiteworxValueInheritance()
    {
        $NW = NodeWorx::getDefaultNodeWorx();
        $SW = $this->getSWAccount();
        $SW->setConfig(
            self::TEST_VALUE_NAME,
            self::TEST_VALUE_PACKAGE
        );

        $this->assertEquals(self::TEST_VALUE_ENV, IW::Env()->getConfig(self::TEST_VALUE_NAME));
        $this->assertEquals(self::TEST_VALUE_NODEWORX, $NW->getConfig(self::TEST_VALUE_NAME));
        $this->assertEquals(self::TEST_VALUE_PACKAGE, $SW->getConfig(self::TEST_VALUE_NAME));

        $Domain = $SW->getMasterDomain();
        $this->assertEquals(self::TEST_VALUE_PACKAGE, $Domain->getConfig(self::TEST_VALUE_NAME));
    }

    /**
     * Test effects of overwriting at the domain level.
     *
     * @depends testSiteworxValueInheritance
     */
    public function testDomainValueInheritance()
    {
        $NW     = NodeWorx::getDefaultNodeWorx();
        $SW     = $this->getSWAccount();
        $Domain = $SW->getMasterDomain();
        $Domain->setConfig(
            self::TEST_VALUE_NAME,
            self::TEST_VALUE_DOMAIN
        );

        $this->assertEquals(self::TEST_VALUE_ENV, IW::Env()->getConfig(self::TEST_VALUE_NAME));
        $this->assertEquals(self::TEST_VALUE_NODEWORX, $NW->getConfig(self::TEST_VALUE_NAME));
        $this->assertEquals(self::TEST_VALUE_PACKAGE, $SW->getConfig(self::TEST_VALUE_NAME));
        $this->assertEquals(self::TEST_VALUE_DOMAIN, $Domain->getConfig(self::TEST_VALUE_NAME));
    }
}
