<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    May 6, 2009
 */

/**
 * General tests for the CLI interface to controllers.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_CLI_GeneralTest extends IWorx_Framework_TestCase
{
    /**
     * Gets the command to issue for getSession.
     *
     * @param string $user
     * @param string $pass
     * @return string
     */
    private function _getNWSessionCmd($user = self::NW_USER, $pass = self::NW_PASS)
    {
        $this->_skipNWIfNeeded();

        $cmd = 'nodeworx --controller index ' .
        '--action getSession ' .
        '--login_email ' . $user . ' ' .
        '--login_password ' . $pass . ' ' .
        '-n';
        return $cmd;
    }

    /**
     * Gets the command to issue for getSession.
     *
     * @param string $user
     * @param string $pass
     * @param string $domain
     * @return string
     */
    private function _getSWSessionCmd(
        $user = self::SW_EMAIL,
        $pass = self::SW_PASS,
        $domain = self::SW_DOMAIN
    ) {
        $this->_skipSWIfNeeded();

        $cmd = 'siteworx --controller index ' .
        '--action getSession ' .
        '--login_email ' . $user . ' ' .
        '--login_password ' . $pass . ' ' .
        '--login_domain ' . $domain . ' ' .
        '-n';
        return $cmd;
    }

    /**
     * Gets an active session id.
     *
     * @param string $user
     * @param string $pass
     * @return string
     */
    private function _getNWSessionId($user = self::NW_USER, $pass = self::NW_PASS)
    {
        $cmd = $this->_getNWSessionCmd($user, $pass);

        $retval = -1;
        $result = [];

        IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        return trim($result[0]);
    }

    /**
     * Gets an active session id.
     *
     * @param string $user
     * @param string $pass
     * @param string $domain
     * @return string
     */
    private function _getSWSessionId(
        $user = self::SW_EMAIL,
        $pass = self::SW_PASS,
        $domain = self::SW_DOMAIN
    ) {
        $cmd = $this->_getSWSessionCmd($user, $pass, $domain);

        $retval = -1;
        $result = [];

        IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        return trim($result[0]);
    }

    /**
     * Skips the test if the nodeworx command isn't there.
     */
    private function _skipNWIfNeeded()
    {
        if (!IWorxFileSys::isFile('/usr/bin/nodeworx')) {
            $this->markTestSkipped('Nodeworx command is missing');
        }
    }

    /**
     * Skips the test if the siteworx command isn't there.
     */
    private function _skipSWIfNeeded()
    {
        if (!IWorxFileSys::isFile('/usr/bin/siteworx')) {
            $this->markTestSkipped('Siteworx command is missing');
        }
    }

    /**
     * Setup function.
     */
    public function setup()
    {
        self::addSWAccountIfNecessary();
        parent::setup();
        putenv('IWORX_SESSION_ID=""');
    }

    /**
     * Tests that zero is the result with good data.
     */
    public function testGetSessionRetValZeroWithGoodLoginData()
    {
        $this->_skipNWIfNeeded();
        $cmd = $this->_getNWSessionCmd();

        $retval = -1;
        $result = [];

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertEquals(CLI::OK, $retval);
    }

    /**
     * Tests that the result is non-zero if login data is bogus.
     */
    public function testGetSessionRetValNotZeroWithBadLoginData()
    {
        $this->_skipNWIfNeeded();
        $cmd = $this->_getNWSessionCmd('baduser@interworx.com');

        $retval = -1;
        $result = [];

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertNotEquals(CLI::OK, $retval);
    }

    /**
     * Tests that a session id is returned and nothing else.
     */
    public function testGetSessionResult()
    {
        $this->_skipNWIfNeeded();
        $cmd = $this->_getNWSessionCmd();

        $retval = -1;
        $result = [];

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertCount(1, $result, 'Unexpected output');
    }

    /**
     * Tests that a session id is returned and nothing else.
     */
    public function testGetSessionResultIsSession()
    {
        $this->_skipNWIfNeeded();
        $cmd = $this->_getNWSessionCmd();

        $retval = -1;
        $result = [];

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertFileExists('/usr/local/interworx/var/session/sess_' . $result[0]);
    }

    /**
     * Tests that invalid session ids fail.
     */
    public function testActionBadSessionId()
    {
        $this->_skipNWIfNeeded();

        $cmd    = 'nodeworx --controller siteworx
                        --action listMasterDomains
                        --session-id badsessionid
                        -n';
        $retval = -1;
        $result = [];

        IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertEquals(CLI::NOT_AUTHORIZED, $retval);
    }

    /**
     * Tests that a session id is returned and nothing else.
     */
    public function testLoggingInFromCLI()
    {
        $this->_skipNWIfNeeded();
        $session_id = $this->_getNWSessionId();

        $cmd    = 'nodeworx --controller index
                        --action getSession
                        --session-id ' . $session_id . '
                        -n';
        $retval = -1;
        $result = [];

        IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertEquals(CLI::OK, $retval);
    }

    /**
     * Tests that a session id is returned and nothing else.
     */
    public function testLoggingInFromCLIUsesSameSession()
    {
        $this->_skipNWIfNeeded();
        $session_id = $this->_getNWSessionId();

        $cmd    = 'nodeworx --controller index
                        --action getSession
                        --session-id ' . $session_id . '
                        -n';
        $retval = -1;
        $result = [];

        IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertEquals($session_id, $result[0]);
    }

    /**
     * Very rough test of the -h parameter.
     */
    public function testHelpSeemsToWorkMoreOrLess()
    {
        $this->_skipNWIfNeeded();
        $cmd    = 'nodeworx -h';
        $retval = -1;
        $result = [];

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);
        $output  = implode("\n", $result);

        $this->assertContains('--help|-h', $output);
    }

    /**
     * Very rough test of the -h parameter.
     */
    public function testHelpSeemsToWorkMoreOrLessAgain()
    {
        $this->_skipNWIfNeeded();
        $session_id = $this->_getNWSessionId();

        $cmd    = 'nodeworx --controller siteworx
                        --action listMasterDomains
                        --session-id ' . $session_id . '
                        -nh';
        $retval = -1;
        $result = [];

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);
        $output  = implode("\n", $result);

        $this->assertContains('--datasource', $output);
        // If it's not OK, that means the usage message was part of an error
        $this->assertEquals(CLI::OK, $retval);
    }

    /**
     * Tests that there's no output if verbose is not specified.
     */
    public function testVerboseModeIsNotDefaultForNonInteractive()
    {
        $this->_skipNWIfNeeded();
        $session_id = $this->_getNWSessionId();

        $cmd    = 'nodeworx --controller siteworx
                        --action toggleFavorite
                        --session-id ' . $session_id . '
                        --domain ' . self::SW_DOMAIN . '
                        -n';
        $result = [];
        $retval = -1;

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertCount(0, $result);
    }

    /**
     * Tests that there is output when verbose is specified.
     */
    public function testVerboseModeWorksWhenSpecified()
    {
        $this->_skipNWIfNeeded();
        $session_id = $this->_getNWSessionId();

        $cmd    = 'nodeworx --controller siteworx
                        --action toggleFavorite
                        --session-id ' . $session_id . '
                        --domain ' . self::SW_DOMAIN . '
                        -nv';
        $result = [];
        $retval = -1;

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $result = implode("\n", $result);

        $this->assertContains('Success', $result);
    }

    /**
     * Tests that the -c and -a options work.
     */
    public function testControllerAndActionAlias()
    {
        $this->_skipNWIfNeeded();
        $session_id = $this->_getNWSessionId();

        $cmd    = 'nodeworx -c siteworx
                        -a toggleFavorite
                        --session-id ' . $session_id . '
                        --domain ' . self::SW_DOMAIN . '
                        -nv';
        $result = [];
        $retval = -1;

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertEquals(CLI::OK, $retval);
    }

    /**
     * Tests that zero is the result with good data.
     */
    public function testSWGetSessionRetValZeroWithGoodLoginData()
    {
        $this->_skipSWIfNeeded();
        $cmd = $this->_getSWSessionCmd();

        $retval = -1;
        $result = [];

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertEquals(CLI::OK, $retval);
    }

    /**
     * Tests that the result is non-zero if login data is bogus.
     */
    public function testSWGetSessionRetValNotZeroWithBadLoginData()
    {
        $this->_skipSWIfNeeded();
        $cmd = $this->_getSWSessionCmd('baduser@interworx.com');

        $retval = -1;
        $result = [];

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertNotEquals(CLI::OK, $retval);
    }

    /**
     * Tests that a session id is returned and nothing else.
     */
    public function testSWGetSessionResult()
    {
        $this->_skipSWIfNeeded();
        $cmd = $this->_getSWSessionCmd();

        $retval = -1;
        $result = [];

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertCount(1, $result, 'Unexpected output');
    }

    /**
     * Tests that a session id is returned and nothing else.
     */
    public function testSWGetSessionResultIsSession()
    {
        $this->_skipSWIfNeeded();
        $cmd = $this->_getSWSessionCmd();

        $retval = -1;
        $result = [];

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertFileExists('/usr/local/interworx/var/session/sess_' . $result[0]);
    }

    /**
     * Tests that using a bad session id fails.
     */
    public function testSWActionBadSessionId()
    {
        $this->_skipSWIfNeeded();
        $cmd    = 'siteworx --controller emailBox
                        --action list
                        --session-id badsessionid
                        -n';
        $retval = -1;
        $result = [];

        IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertEquals(CLI::NOT_AUTHORIZED, $retval);
    }

    /**
     * Tests that a session id is returned and nothing else.
     */
    public function testSWLoggingInFromCLI()
    {
        $this->_skipSWIfNeeded();
        $session_id = $this->_getSWSessionId();

        $cmd    = 'siteworx --controller index
                        --action getSession
                        --session-id ' . $session_id . '
                        -n';
        $retval = -1;
        $result = [];

        IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertEquals(CLI::OK, $retval);
    }

    /**
     * Tests that a session id is returned and nothing else.
     */
    public function testSWLoggingInFromCLIUsesSameSession()
    {
        $this->_skipSWIfNeeded();
        $session_id = $this->_getSWSessionId();

        $cmd    = 'siteworx --controller index
                        --action getSession
                        --session-id ' . $session_id . '
                        -n';
        $retval = -1;
        $result = [];

        IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertEquals($session_id, $result[0]);
    }

    /**
     * Very rough test of the -h parameter.
     */
    public function testSWHelpSeemsToWorkMoreOrLess()
    {
        $this->_skipSWIfNeeded();
        $cmd    = 'siteworx -h';
        $retval = -1;
        $result = [];

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);
        $output  = implode("\n", $result);

        $this->assertContains('--help|-h', $output);
    }

    /**
     * Very rough test of the -h parameter.
     */
    public function testSWHelpSeemsToWorkMoreOrLessAgain()
    {
        $this->_skipSWIfNeeded();
        $session_id = $this->_getSWSessionId();

        $cmd    = 'siteworx --controller emailBox
                        --action list
                        --session-id ' . $session_id . '
                        -nh';
        $retval = -1;
        $result = [];

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);
        $output  = implode("\n", $result);

        $this->assertContains('--datasource', $output);
        // If it's not OK, that means the usage message was part of an error
        $this->assertEquals(CLI::OK, $retval);
    }

    /**
     * Tests that the -c and -a options work.
     */
    public function testSWControllerAndActionAlias()
    {
        $this->_skipSWIfNeeded();
        $session_id = $this->_getSWSessionId();

        $cmd    = 'siteworx -c emailBox
                        -a list
                        --session-id ' . $session_id . '
                        -n';
        $result = [];
        $retval = -1;

        $success = IWorxExec::exec($cmd, $result, $retval, IWorxExec::KILL_STDERR);

        $this->assertEquals(CLI::OK, $retval);
    }
}
