<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Jan 22, 2020
 */

/**
 * Base class for testing db mapping in MysqlRemote Ctrl.
 */
class Ctrl_Nodeworx_Map extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Nodeworx_MysqlRemote';

    private $_unmappedDbs = ['testDb1', 'testDb2', 'testDb3'];
    private $_unmappedUsers = ['testuser2', 'testuser4', 'testuser1'];


    /**
     * Add unmapped dbs.
     */
    public function addUnmappedDbs()
    {
        $Driver = $this->_getDriver();
        $dbConnection = $this->_getConnectedDatabase($Driver);

        foreach ($this->_unmappedDbs as $dbToAdd) {
            if (!$Driver->systemDatabaseExistsFQ($dbToAdd)) {
                $this->_addDb($dbConnection, $dbToAdd);
            }
        }
    }

    /**
     * Add db.
     *
     * @param mixed  $dbConnection
     * @param string $dbToAdd
     */
    private function _addDb($dbConnection, $dbToAdd)
    {
        $sql = "CREATE DATABASE `{$dbToAdd}`";
        IWorxDbUtil::db_query($dbConnection, $sql);
    }


    /**
     * Add unmapped db users.
     */
    public function addUnmappedUsers()
    {
        $Driver = $this->_getDriver();
        $dbConnection = $this->_getConnectedDatabase($Driver);

        foreach ($this->_unmappedUsers as $user) {
            if (!$Driver->systemDatabaseUserExistsFQ($user)) {
                $this->_addUser($dbConnection, $user);
            }
        }
    }

    /**
     * Add unmapped db user.
     *
     * @param object $dbConnection
     * @param string $userToAdd
     */
    private function _addUser($dbConnection, $userToAdd)
    {
        $qUser = q($userToAdd);
        $sql = "CREATE USER '$userToAdd'@'localhost'";
        IWorxDbUtil::db_query($dbConnection, $sql);
    }

    /**
     * Drop dbs and remove reference from siteworx map db.
     */
    public function deleteDbs()
    {
        $Driver = $this->_getDriver();
        $dbConnection = $this->_getConnectedDatabase($Driver);

        foreach ($this->_unmappedDbs as $db) {
            if ($Driver->systemDatabaseExistsFQ($db)) {
                $sql = "DROP DATABASE `$db`";
                IWorxDbUtil::db_query($dbConnection, $sql);

                // deletes db from siteworx_db_name_map
                SiteworxDbMapper::getInstance()->deleteDb($db);
            }
        }
    }

    /**
     * Delete users and remove from sw db map.
     */
    public function deleteUsers()
    {
        $Driver = $this->_getDriver();
        $dbConnection = $this->_getConnectedDatabase($Driver);

        foreach ($this->_unmappedUsers as $user) {
            if ($Driver->systemDatabaseUserExistsFQ($user)) {
                $qUser = q($user);

                // delete form mysql.user
                $sqlUserTable = "DELETE FROM mysql.user WHERE User = $qUser";
                $resultDeleteFromUser = IWorxDbUtil::db_query($dbConnection, $sqlUserTable);
                $this->_checkForErrors($resultDeleteFromUser, "could not delete from mysql.user");

                // delete from mysql.db
                $sqlDbTable = "DELETE FROM mysql.db WHERE User = $qUser";
                $resultDeleteFromDb = IWorxDbUtil::db_query($dbConnection, $sqlDbTable);
                $this->_checkForErrors($resultDeleteFromDb, "could not delete from mysql.db");

                // delete from sw map db
                SiteworxDbMapper::getInstance()->deleteUser($user);
                IWorxDbUtil::db_query($dbConnection, 'FLUSH PRIVILEGES');
            }
        }
    }

    /**
     * Check if sql resulted in error.
     *
     * @param mixed  $result
     * @param string $msg
     */
    private function _checkForErrors($result, $msg)
    {
        $error = IWorxDbUtil::db_lastError();
        if (DB::isError($result) || !empty($error)) {
            echo $msg;
        }
    }
    /**
     * Get unmapped dbs to compare with result.
     *
     * @return array
     */
    public function getUnmappedDbs()
    {
        return $this->_unmappedDbs;
    }

    /**
     * Randomly return one of the unmapped dbs.
     *
     * @return string
     */
    public function getUnmappedDb() {
        $index = array_rand($this->_unmappedDbs);
        return $this->_unmappedDbs[$index];
    }

    /**
     * Randomly return one of the unmapped db users.
     *
     * @return array
     */
    public function getUnmappedUser()
    {
        $index = array_rand($this->_unmappedUsers);
        return $this->_unmappedUsers[$index];
    }

    /**
     * Get connected db for queries.
     *
     * @param Driver $Driver
     * @return object
     */
    private function _getConnectedDatabase(Driver $Driver)
    {
        $dsn = $Driver->getClientDsn();
        return IWorxDbUtil::db_connect($dsn);
    }

    /**
     * Get driver for db operations.
     *
     * @return Driver
     */
    private function _getDriver()
    {
        $Driver   = DriverManager::getDefaultDriver('db');
        return $Driver;
    }
}
