<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Jan 20, 2020
 */
require_once(__DIR__ . '/Map.php');

/**
 * Tests for the Mysql Map controller.
 *
 * @package Interworx
 * @subpackage Tests
 */
class Ctrl_Nodeworx_MapUserTest extends Ctrl_Nodeworx_Map
{
    /**
     * Make sure unmapped users are added before each test is run.
     */
    public function setUp()
    {
        parent::setUp();
        $this->addUnmappedUsers();
    }

    /**
     * Cleanup after each test.
     */
    public function tearDown()
    {
        $this->deleteUsers();
    }

    /**
     * Empty string as sw id should cause mapUser to fail.
     */
    public function testMapUserActionShouldFailWithoutSW()
    {
        $user = $this->getUnmappedUser();
        $masterDomainEmptyString = '';
        $Reply = $this->_getReplyMapDatabaseUserAction($masterDomainEmptyString, $user);
        $this->assertReplyNotSuccessful($Reply, "mapUser should fail with empty sw id.");
    }

    /**
     * Invalid/nonexisting sw id should cause mapUser to fail.
     */
    public function testMapUserActionShouldFailWithInvalidSW()
    {
        $invalidmasterDomain = 'invaliddomain';
        $user = $this->getUnmappedUser();
        $Reply = $this->_getReplyMapDatabaseUserAction($invalidmasterDomain, $user);
        $this->assertReplyNotSuccessful($Reply, "mapUser should fail with invalid sw id.");
    }

    /**
     * Empty array of users should cause mapUser to fail.
     */
    public function testMapUserActionShouldFailWithoutUsers()
    {
        $SW = $this->getSWAccount();
        $user = '';
        $Reply = $this->_getReplyMapDatabaseUserAction($SW->getMasterDomainName(), $user);
        $this->assertReplyNotSuccessful($Reply, "mapUser should fail with empty users array.");
    }

    /**
     * mapUser should fail if db users do not exist.
     */
    public function testMapUserActionShouldFailWithInvalidUser()
    {
        $SW = $this->getSWAccount();
        $user = 'nonexistinguser';
        $Reply = $this->_getReplyMapDatabaseUserAction($SW->getMasterDomainName(), $user);
        $this->assertReplyNotSuccessful($Reply, "mapUser should fail with nonexisting db users.");
    }

    /**
     * Test successful reply & that users are mapped as expected.
     */
    public function testMapUserActionShouldMapUsersWithValidArgs()
    {
        $SW = $this->getSwAccount();
        $user = $this->getUnmappedUser();
        $Reply = $this->_getReplyMapDatabaseUserAction($SW->getMasterDomainName(), $user);
        $this->assertReplySuccessful($Reply, "mapUser request fails with valid arguments.");

        $swUsers = SiteworxDbMapper::getInstance()->getUsers($SW);
        $this->assertContains($user, $swUsers);
    }

    /**
     * Get Reply from mapUser action.
     *
     * @param string $masterDomain
     * @param string $user
     * @return Reply
     */
    private function _getReplyMapDatabaseUserAction($masterDomain, $user)
    {
        $NW = self::getMasterNodeWorx();
        $nickname = MySQL::getLocalServer()->getNickname();

        $input = array(
            'server' => $nickname,
            'siteworx_account' => $masterDomain,
            'database_user' => $user,
        );

        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'mapDatabaseUser',
            $input,
            $NW
        );

        return $Reply;
    }
}
