<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Nov 24, 2020
 */

/**
 * Test Suite for Ctrl_Nodeworx_Upgrade
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Nodeworx_UpgradeTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Nodeworx_Upgrade';

    private $_keys = array(
        'is-opted-in',
        'httpd-syntax',
        'is-litespeed',
        'is-cloud-linux',
        'is-cluster',
        'has-custom-themes',
        'has-custom-vhosts',
        'yum-upgrade',
        'is-up-to-date',
        'is-eol',
        'is-using-dovecot-submission'
    );

    /**
     * Make sure always opted out after tests.
     * Do not want server to upgrade unexpectedly.
     */
    public function tearDown()
    {
        $this->_optOut();
    }

    /**
     * Should return perms for UPDATES.
     */
    public function testGetRequiredPermsShouldReturnUpdates()
    {
        $actual = Ctrl_Nodeworx_Upgrade::getRequiredPerms()[0];
        $expected = 'UPDATES';
        $this->assertEquals($actual, $expected);
    }

    /**
     * test that list upgrade step status includes all the
     * upgrade steps we expect.
     */
    public function testListUpgradePrepStatusActionShouldContainUpgradeSteps()
    {
        $action = 'listUpgradePrepStatus';
        $Reply = $this->_sendRequest($action);

        $this->assertReplySuccessful($Reply);

        $upgradePrepSteps = $Reply->getPayloadData();

        foreach ($upgradePrepSteps as $step) {
            $this->assertContains($step->key, $this->_keys);
        }
    }


    /**
     * Should fail when upgrade step not forcable.
     */
    public function testignoreCommitActionShouldFail()
    {
        $args = array(
            'key' => 'httpd-syntax'
        );

        $Reply = $this->_sendRequest('ignoreCommit', $args);
        $this->assertReplyNotSuccessful($Reply, 'expected ignoreCommitAcion to fail when upgrade step not forcable');
    }

    /**
     * Should succeed when upgrade step is forcable.
     */
    public function testignoreCommitActionShouldSucceed()
    {
        // ensure httpd-syntax step not
        // forced before testing
        IW::Env()->clearEnvFlag('upgrade-prep-7-is-litespeed-ignore');

        $args = array(
            'key' => 'is-litespeed'
        );

        $Reply = $this->_sendRequest('ignoreCommit', $args);
        $this->assertReplySuccessful($Reply, 'expected ignoreCommitAcion to succeed when upgrade step is forcable');
    }

    /**
     * Expects clear force to succeed when forced.
     *
     * @depends testignoreCommitActionShouldSucceed
     */
    public function testUndoIgnoreCommitActionShouldSucceed()
    {
        $args = array(
            'key' => 'is-litespeed'
        );

        $Reply = $this->_sendRequest('undoIgnoreCommit', $args);
        $this->assertReplySuccessful($Reply, 'expected undoIgnoreCommitAcion to succeed when upgrade step forced');
    }

    /**
     * Expects clear force to fail when not forced.
     *
     * @depends testUndoIgnoreCommitActionShouldSucceed
     */
    public function testUndoIgnoreCommitActionShouldFail()
    {
        $args = array(
            'key' => 'is-litespeed'
        );

        $Reply = $this->_sendRequest('undoIgnoreCommit', $args);
        $this->assertReplyNotSuccessful($Reply, 'expected undoIgnoreCommitAcion to fail when upgrade step not forced');
    }

    /**
     * Expects optInCommitAction to:
     * 1. set selected software channel as env flags
     * 2. opt in to upgrade
     * 3. return successful reply
     */
    public function testOptInCommitActionShouldSucceed()
    {
        // TODO: uncomment when further channels are supported in iw7
        //$this->_testOptInCommitActionShouldSetEnvFlag('stable');
        $this->_testOptInCommitActionShouldSetEnvFlag('beta');
        //$this->_testOptInCommitActionShouldSetEnvFlag('release');
        //$this->_testOptInCommitActionShouldSetEnvFlag('release-candidate');
    }

    /**
     * expects optInCommitAction to succeed & set env flag.
     *
     * @param string $expectedChannel
     */
    private function _testOptInCommitActionShouldSetEnvFlag($expectedChannel)
    {
        $this->_optOut();

        $Reply = $this->_optIn($expectedChannel);

        $this->assertReplySuccessful($Reply, 'expected successful reply when opting in with valid args');

        $actual = IW::Env()->getEnvFlag(Upgrade_Prep_Version_6::SOFTWARE_CHANNEL_FLAG);
        $this->assertSame($expectedChannel, $actual);

        $OptIn = new Upgrade_Prep_IsOptedIn();
        $this->assertTrue($OptIn->isOptedIn());
    }

    /**
     * @param string $channel
     * @return Reply
     */
    private function _optIn($channel)
    {
        $args = array(
            'software_channel' => $channel
        );

        return $this->_sendRequest('optInCommit', $args);
    }

    /**
     * Opt out if opted in.
     */
    private function _optOut()
    {
        $OptIn = new Upgrade_Prep_IsOptedIn();

        if ($OptIn->isOptedIn()) {
            $OptIn->optOut();
        }
    }

    /**
     *
     */
    public function testOptInShouldFailWhenOptedIn()
    {
        $this->_optIn('beta');

        // opt in again - should fail
        $Reply = $this->_optIn('beta');
        $this->assertReplyNotSuccessful($Reply, 'expected fail reply when already opted in');
    }

    /**
     * Expects successful reply & optout
     *
     * @depends testOptInShouldFailWhenOptedIn
     */
    public function testOptOutShouldSucceed()
    {
        $this->_optIn('beta');

        $Reply = $this->_sendRequest('optOutCommit');
        $this->assertReplySuccessful($Reply, 'expected successful reply when opted in');

         $OptIn = new Upgrade_Prep_IsOptedIn();
         $this->assertFalse($OptIn->isOptedIn());
    }


    /**
     * Expects fail reply if opted out.
     */
    public function testOptOutShouldFail()
    {
        // initial state is opted out.
        // opting out when opted out should
        // result in failed response.
        $Reply = $this->_sendRequest('optOutCommit');
        $this->assertReplyNotSuccessful($Reply, 'expected fail when opted out');
    }

    /**
     * send request to controller.
     *
     * @param string $action
     * @param array  $args
     * @return Reply
     */
    private function _sendRequest($action, array $args = array())
    {
        $NW = IW::NW();
        $Reply = IW::FC()->routeFromPHP(self::CTRL, $action, $args, $NW);
        return $Reply;
    }
}
