<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 */

/**
 * Tests for Ctrl_Siteworx_EmailAutorespond.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_EmailAutorespondTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Siteworx_EmailAutorespond';

    const EMAIL   = 'otto';
    const COPYTO  = 'copy@to.com';
    const MESSAGE = 'Look, I am an autoresponder!';

    /**
     * Deletes teh account if needed.
     */
    public function testSetup()
    {
        $data = $this->getSWIndexData(self::CTRL);

        if (count($data['iw_payload']->getData()) !== 0) {
            $this->deleteSWAccountIfNecessary();
        }
    }

    /**
     * Tests the output of the index action.
     */
    public function testIndexActionReturnsArray()
    {
        $output = $this->getSWIndexData(self::CTRL);

        $this->assertInternalType('array', $output);
    }

    /**
     * Verifies initial count of autoresponders.
     */
    public function testInitialAutoresponderCount()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $this->assertCount(
            0,
            $data['iw_payload']->getData(),
            'Autoresponders already exist!'
        );
    }

    /**
     * Tests that addAction fails without input.
     */
    public function testAddFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'add', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that addAction returns true.
     */
    public function testAddReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'username'           => self::EMAIL,
                    'copyto'             => self::COPYTO,
                    'autorespondmessage' => self::MESSAGE );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test that the add action seemed to work.
     */
    public function testAddActionWorked()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $this->assertCount(1, $data['iw_payload']->getData());
    }

    /**
     * Tests that editAction fails without input.
     */
    public function testEditFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'edit', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that editAction returns true.
     */
    public function testEditReturnsTrue()
    {
        Form::resetInstances();

        $SW = $this->getSWAccount();

        $input = array( 'username' => self::EMAIL,
                    'copyto'   => self::SW_EMAIL );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test that the edit action seemed to work.
     */
    public function testEditActionWorked()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $autos = $data['iw_payload']->getData();
        // Verify that the change happened
        $this->assertEquals(self::SW_EMAIL, $autos[0]->copyto[0]);
    }

    /**
     * Test that the mail driver detects autoresponders with a copy-to set
     * IWX-8267
     */
    public function testExistsReturnsTrue()
    {
        $SW = $this->getSWAccount();
        $Driver = $SW->getIncomingMailDriver();
        $this->assertTrue($Driver->autoresponderExists(self::EMAIL));
    }

    /**
     * Tests that deleteAction fails without input.
     */
    public function testDeleteFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'delete', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that deleteAction returns true.
     */
    public function testDeleteReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'username' => self::EMAIL );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test that the delete action seemed to work.
     */
    public function testDeleteActionWorked()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $this->assertCount(0, $data['iw_payload']->getData());
    }

    /**
     * Test.
     */
    public function testAddAutoresponderSimpleCopyToLoopRejection()
    {
        $SW = $this->getSWAccount();

        $self = self::EMAIL . '@' . $SW->getWorkingDomainName();
        $Reply = $this->_addOneAutoresponder(self::EMAIL, array( $self ));
        $this->assertReplyNotSuccessful(
            $Reply,
            'Failed to rejected mail loop creation'
        );

        $this->assertContains(
            IW::tr('##LG_EMAIL_ALIAS_RECURSIVE##'),
            $Reply->getErrorsAsString(),
            'Unexpected error: ' . $Reply->getErrorsAsString()
        );
    }

    /**
     * Test.
     */
    public function testEditAutoresponderSimpleCopyToLoopRejection()
    {
        $this->_addOneAutoresponder(self::EMAIL, array());
        $SW = $this->getSWAccount();

        $self = self::EMAIL . '@' . $SW->getWorkingDomainName();
        $input = array( 'username'         => self::EMAIL,
                    'copyto'           => array( $self ) );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $SW);

        $this->assertReplyNotSuccessful(
            $Reply,
            'Failed to rejected mail loop creation'
        );

        $this->assertContains(
            IW::tr('##LG_EMAIL_ALIAS_RECURSIVE##'),
            $Reply->getErrorsAsString(),
            'Unexpected error: ' . $Reply->getErrorsAsString()
        );
        $this->_delOneAutoresponder(self::EMAIL);
    }

    /**
     * Test.
     */
    public function testAddAutoresponderComplexCopyToLoopRejection()
    {
        $SW = $this->getSWAccount();

        $emails = [];
        for ($i = 0; $i < 10; $i++) {
            $emails[] = self::EMAIL . "{$i}@{$SW->getWorkingDomainName()}";
        }

        $i = 0;
        foreach ($emails as $email) {
            $Reply = $this->_delOneAutoresponder(self::EMAIL . $i++);
        }

        $i = 0;
        foreach ($emails as $email) {
            $Reply = $this->_addOneAutoresponder(self::EMAIL . $i, $emails[($i+1) % 10]);
            if ($i === 9) {
                $this->assertReplyNotSuccessful(
                    $Reply,
                    'Failed to rejected mail loop creation'
                );
                $this->assertContains(
                    IW::tr('##LG_EMAIL_ALIAS_RECURSIVE##'),
                    $Reply->getErrorsAsString(),
                    'Unexpected error: ' . $Reply->getErrorsAsString()
                );
            } else {
                $this->assertReplySuccessful($Reply);
            }
            $i++;
        }
    }

    /**
     * Helper.
     *
     * @param string       $username
     * @param string|array $cc
     * @return Reply
     */
    private function _addOneAutoresponder($username, $cc)
    {
        $SW = $this->getSWAccount();
        $input = array( 'username'           => $username,
                    'copyto'             => $cc,
                    'autorespondmessage' => self::MESSAGE );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);
        return $Reply;
    }

    /**
     * Helper.
     *
     * @param string $username
     * @return Reply
     */
    private function _delOneAutoresponder($username)
    {
        $SW = $this->getSWAccount();
        $input = array( 'username' => $username );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);
        return $Reply;
    }
}
