<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Feb 26, 2009
 */

/**
 * Tests for Ctrl_Siteworx_EmailBox.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class Ctrl_Siteworx_EmailBoxTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Siteworx_EmailBox';

    const EMAIL = 'bob';
    const PASS  = 'bobspassword';
    const QUOTA = '1234';

    /**
     * This test needs a clean SW account each time.
     */
    public function testSetup()
    {
        $data = $this->getSWIndexData(self::CTRL);

        if (count($data['iw_payload']->getData() !== 1)) {
            $this->deleteSWAccountIfNecessary();
        }
    }

    /**
     * Checks the initial count of boxes.
     */
    public function testInitialMailboxCount()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $this->assertCount(
            1,
            $data['iw_payload']->getData(),
            'More email accounts than expected'
        );
    }

    /**
     * Checks that the form has been assigned.
     */
    public function testForPresenceOfIwForm()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $this->assertArrayHasKey('iw_form', $data);
    }

    /**
     * Tests adding an email account.
     */
    public function testAddActionReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'username'     => self::EMAIL,
                    'password'         => self::PASS,
                    'confirm_password' => self::PASS,
                    'diskspacequota'   => self::QUOTA );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Checks that a second email account was actually added.
     */
    public function testAddActionAdded()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $this->assertCount(2, $data['iw_payload']->getData(), 'Account was not added');
    }

    /**
     * Tests adding an email account duplicate fails.
     */
    public function testAddActionDuplicateFails()
    {
        $SW = $this->getSWAccount();

        $input = array( 'username'     => self::EMAIL,
                    'password'         => self::PASS,
                    'confirm_password' => self::PASS,
                    'diskspacequota'   => self::QUOTA );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);

        $this->assertIsFormErrorArray($Reply->getData());
    }

    /**
     * Tests that editing returns true.
     */
    public function testEditActionReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'username'       => self::EMAIL,
                    'diskspacequota' => '123' );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Makes sure that the value changed to the new one.
     */
    public function testEditValueChanged()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $pdata = $data['iw_payload']->getData();
        $user = $pdata[0];
        $this->assertEquals('123MB', $user->quota, 'Quota was not changed!');
    }

    /**
     * Tests delete action.
     */
    public function testDeleteActionReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'username' => self::EMAIL );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Checks that the user was deleted.
     */
    public function testDeleteActionDeleted()
    {
        $data = $this->getSWIndexData(self::CTRL);

        $this->assertCount(
            1,
            $data['iw_payload']->getData(),
            'More email accounts than expected'
        );
    }

    /**
     * Test.
     */
    public function testAddEmailSimpleCopyToLoopRejection()
    {
        $SW = $this->getSWAccount();

        $self = self::EMAIL . '@' . $SW->getWorkingDomainName();
        $Reply = $this->_addOneBox(self::EMAIL, array( $self ));

        $this->assertReplyNotSuccessful(
            $Reply,
            'Failed to rejected mail loop creation'
        );

        $this->assertContains(
            IW::tr('##LG_EMAIL_ALIAS_RECURSIVE##'),
            $Reply->getErrorsAsString(),
            'Unexpected error: ' . $Reply->getErrorsAsString()
        );
    }

    /**
     * Test.
     */
    public function testEditEmailSimpleCopyToLoopRejection()
    {
        $this->_addOneBox(self::EMAIL, array());
        $SW = $this->getSWAccount();

        $self = self::EMAIL . '@' . $SW->getWorkingDomainName();
        $input = array( 'username'         => self::EMAIL,
                    'copyto'           => array( $self ) );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $SW);

        $this->assertReplyNotSuccessful(
            $Reply,
            'Failed to rejected mail loop creation'
        );

        $this->assertContains(
            IW::tr('##LG_EMAIL_ALIAS_RECURSIVE##'),
            $Reply->getErrorsAsString(),
            'Unexpected error: ' . $Reply->getErrorsAsString()
        );
        $this->_delOneBox(self::EMAIL);
    }

    /**
     * Test.
     */
    public function testAddEmailComplexCopyToLoopRejection()
    {
        $SW = $this->getSWAccount();

        $emails = [];
        for ($i = 0; $i < 10; $i++) {
            $emails[] = self::EMAIL . "{$i}@{$SW->getWorkingDomainName()}";
        }

        $i = 0;
        foreach ($emails as $email) {
            $Reply = $this->_delOneBox(self::EMAIL . $i++);
        }

        $i = 0;
        foreach ($emails as $email) {
            $Reply = $this->_addOneBox(self::EMAIL . $i, $emails[($i+1) % 10]);
            if ($i === 9) {
                $this->assertReplyNotSuccessful(
                    $Reply,
                    'Failed to rejected mail loop creation'
                );
                $this->assertContains(
                    IW::tr('##LG_EMAIL_ALIAS_RECURSIVE##'),
                    $Reply->getErrorsAsString(),
                    'Unexpected error: ' . $Reply->getErrorsAsString()
                );
            } else {
                $this->assertReplySuccessful($Reply);
            }
            $i++;
        }
    }

    /**
     * Helper.
     *
     * @param string       $username
     * @param string|array $cc
     * @return Reply
     */
    private function _addOneBox($username, $cc)
    {
        $SW = $this->getSWAccount();
        $input = array( 'username'         => $username,
                    'password'         => self::PASS,
                    'confirm_password' => self::PASS,
                    'diskspacequota'   => self::QUOTA,
                    'copyto'           => $cc );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);
        return $Reply;
    }

    /**
     * Helper.
     *
     * @param string $username
     * @return Reply
     */
    private function _delOneBox($username)
    {
        $SW = $this->getSWAccount();
        $input = array( 'username' => $username );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);
        return $Reply;
    }
}
