<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 26, 2009
 */

/**
 * Tests for Ctrl_Siteworx_EmailDomainkeys.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_EmailDomainkeysTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Siteworx_EmailDomainkeys';

    /**
     * Setup.
     */
    public static function setUpBeforeClass()
    {
        $Test = new self();
        $Test->_deleteAccountIfNecessary();
    }

    /**
     * Gets an array of record ids for domainkeys records.
     *
     * @return array
     */
    private function _getDomainKeysRecordIds()
    {
        $SW   = $this->getSWAccount();

        $Search = new IWorx_Search_Dns();
        $Search->setOption('type', Dns_Record::TXT);
        $Search->setOption('host', '*_domain.' . $SW->getWorkingDomainName() . '*');
        $Search->addSelectableField('record_id');

        return $Search->search(IWorx_Search_Engine::FETCH_COLUMN);
    }

    /**
     * Deletes account if necessary.
     */
    private function _deleteAccountIfNecessary()
    {
        if (count($this->_getDomainKeysRecordIds()) !== 0) {
            $this->deleteSWAccountIfNecessary();
        }
    }

    /**
     * Verifies that there are not keys initially.
     */
    public function testInitialKeyStatus()
    {
        $this->assertCount(0, $this->_getDomainKeysRecordIds());
    }

    /**
     * Tests that addAction fails without input.
     */
    public function testAddFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'add', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that addAction returns true.
     */
    public function testAddReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'domain' => self::SW_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests that the action set the domain key dns record correctly.
     *
     * @depends testAddReturnsTrue
     */
    public function testAddActionSetDomainkeyRecordCorrectly()
    {
        $SW     = $this->getSWAccount();
        $domain = $SW->getWorkingDomainName();
        $host   = "_domainkey.{$domain}";

        $Zone  = new Dns_Zone('domain = ' . q($domain));
        $where = 'host = ' . q($host) . ' AND zone_id = ' . q($Zone->getId());

        $this->assertTrue(Dns_Record::exists($where));

        $Record   = new Dns_Record($where);
        $e_target = 'v=DKIM1; k=rsa; t=y;';

        $this->assertEquals($e_target, $Record->getRdata());
    }

    /**
     * Tests that the action set the default dns record correctly.
     *
     * @depends testAddReturnsTrue
     */
    public function testAddActionSetDefaultRecordCorrectly()
    {
        $SW     = $this->getSWAccount();
        $domain = $SW->getWorkingDomainName();
        $host   = "default._domainkey.{$domain}";

        $Zone  = new Dns_Zone('domain = ' . q($domain));
        $where = 'host = ' . q($host) . ' AND zone_id = ' . q($Zone->getId());

        $this->assertTrue(Dns_Record::exists($where));

        $dkdir = Ini::get(Ini::DIR, 'domainkeys');

        $Record = new Dns_Record($where);
        $cmd    = 'grep -v ^- ' . "{$dkdir}/{$domain}/" . 'rsa.public | ' .
        'perl -e \'while(<>){chop;$l.=$_;}' .
        'print "v=DKIM1; k=rsa; t=y; p=$l;\n";\'';

        IWorxExec::exec_NOSCRUB($cmd, $result, $retval);

        $e_target = trim(implode("\n", $result));

        $this->assertEquals($e_target, $Record->getRdata());
    }

    /**
     * Tests the view action.
     *
     * @depends testAddReturnsTrue
     */
    public function testViewAction()
    {
        $SW     = $this->getSWAccount();
        $domain = $SW->getWorkingDomainName();
        $dkdir  = Ini::get(Ini::DIR, 'domainkeys');

        $input = array( 'domain' => $SW->getWorkingDomainName() );

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'view', $input, $SW);
        $result = $Reply->getData();

        $this->assertArrayHasKey('iw_private_key', $result);
        $this->assertArrayHasKey('iw_public_key', $result);

        $public_key  = file_get_contents("{$dkdir}/{$domain}/rsa.public");
        $private_key = file_get_contents("{$dkdir}/{$domain}/default");

        $this->assertEquals($public_key, $result['iw_public_key']);
        $this->assertEquals($private_key, $result['iw_private_key']);
    }

    /**
     * Tests editing a Domainkey (to remove testing mode, or something).
     *
     * Tests that  the actual public and private keys do not change.
     *
     * @depends testAddReturnsTrue
     */
    public function testEditActionDoesntChangeKeys()
    {
        $SW     = $this->getSWAccount();
        $domain = $SW->getWorkingDomainName();

        $dkdir       = Ini::get(Ini::DIR, 'domainkeys');
        $public_key  = file_get_contents("{$dkdir}/{$domain}/rsa.public");
        $private_key = file_get_contents("{$dkdir}/{$domain}/default");

        $input = array( 'domain' => $domain );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $SW);

        $this->assertReplySuccessful($Reply);

        $current_public_key  = file_get_contents("{$dkdir}/{$domain}/rsa.public");
        $current_private_key = file_get_contents("{$dkdir}/{$domain}/default");

        $this->assertEquals($public_key, $current_public_key);
        $this->assertEquals($private_key, $current_private_key);
    }

    /**
     * Tests that deleteAction fails without input.
     *
     * @depends testAddReturnsTrue
     */
    public function testDeleteFailsWithoutInput()
    {
        $SW = $this->getSWAccount();

        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'delete', array(), $SW);
        $result = $Reply->getData();

        $this->assertIsFormErrorArray($result);
    }

    /**
     * Tests that deleteAction returns true.
     *
     * @depends testAddReturnsTrue
     */
    public function testDeleteReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'domain' => self::SW_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test that the delete action seemed to work.
     *
     * @depends testAddReturnsTrue
     */
    public function testDeleteActionWorked()
    {
        $this->assertCount(0, $this->_getDomainKeysRecordIds());
    }
}
