<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Feb 23, 2009
 */

/**
 * Tests for EmailMx Controller.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Ctrl_Siteworx_EmailRemotesetupTest extends IWorx_Framework_TestCase
{
    const CTRL = 'Ctrl_Siteworx_EmailRemotesetup';

    const PREF_NUMBER        = '9999';
    const PREF_NUMBER_CHANGE = '9998';
    const HOST               = 'interworx.com';
    const HOST_CHANGE        = 'nexcess.net';

    /**
     * Deletes the account if it exists to make sure there's a fresh one.
     */
    public function testSetup()
    {
        $this->deleteSWAccountIfNecessary();
    }

    /**
     * Tests that the initial setup only has one MX Record.
     */
    public function testMxRecordInitialCountOfRecords()
    {
        $mx_records = $this->_getMxRecords();

        $this->assertCount(1, $mx_records, 'Unexpected count of MX Records');
    }

    /**
     * Tests that the initial preference number is as expected.
     */
    public function testMxRecordInitialPreferenceNumber()
    {
        $mx_records = $this->_getMxRecords();
        $SW         = $this->getSWAccount();
        $this->assertEquals(
            "10 mail.{$SW->getWorkingDomainName()}",
            $mx_records[ 0 ]['target']
        );
    }

    /**
     * Tests that the initial host is as expected.
     */
    public function testMxRecordInitialHost()
    {
        $SW            = $this->getSWAccount();
        $mx_records    = $this->_getMxRecords();
        $expected_host = $SW->getWorkingDomainName();

        $this->assertEquals($expected_host, $mx_records[ 0 ]['host']);
    }

    /**
     * Checks for the variables that the template needs.
     */
    public function testIndexActionAssignedVariables()
    {
        $SW     = $this->getSWAccount();
        $Reply  = IW::FC()->routeFromPHP(self::CTRL, 'index', array(), $SW);
        $result = $Reply->getData();

        $this->assertArrayHasKey('iw_form', $result, 'Local delivery form is missing');
    }

    /**
     * Tests adding an MX record.
     */
    public function testAddActionReturnsTrue()
    {
        $this->_deleteMxRecord();

        $SW = $this->getSWAccount();
        $zone_id = $SW->getWorkingZone()->getId();

        $input = array( 'zone_id'     => $zone_id,
                    'preference'  => self::PREF_NUMBER,
                    'host'        => self::SW_DOMAIN,
                    'mail_server' => self::HOST );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'addmx', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Tests that adding a record a second time fails.
     */
    public function testAddingDuplicateRecordFails()
    {
        $SW = $this->getSWAccount();

        $input = array( 'preference' => self::PREF_NUMBER,
                    'host'       => self::HOST );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'addmx', $input, $SW);

        $this->assertFalse($Reply->wasSuccessful());
    }

    /**
     * Tests that the expected values were added.
     */
    public function testAddActionActuallyAddedSomething()
    {
        $mx_records = $this->_getMxRecords();
        $mx_record  = [];

        foreach ($mx_records as $r) {
            if (strpos($r['target'], self::PREF_NUMBER) !== false) {
                $mx_record = $r;
            }
        }

        $this->assertNotEquals(strpos($mx_record['target'], self::HOST), false);
    }

    /**
     * Changes the preference number of the added record.
     */
    public function testEditActionReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $input = array( 'record_id'  => $this->_getMXRecordId(),
                    'preference' => self::PREF_NUMBER_CHANGE,
                    'host'       => self::SW_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'editmx', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Deletes the edited record.
     */
    public function testDeleteActionReturnsTrue()
    {
        $SW = $this->getSWAccount();

        $record_id = $this->_getMXRecordId(self::PREF_NUMBER_CHANGE);
        $input     = array( 'record_id' => $record_id );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'deletemx', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Verifies that the record was actually deleted.
     */
    public function testDeleteActionDeleted()
    {
        $record_id = $this->_getMXRecordId(self::PREF_NUMBER_CHANGE);

        $this->assertFalse($record_id, 'MX Record found after delete');
    }

    /**
     * Tests that local mail delivery is enabled, and if it is not then it tries.
     *
     * to add it and makes sure that all works.
     */
    public function testLocalDeliveryInitialStatus()
    {
        $SW = $this->getSWAccount();

        $is_enabled = Qmail::checkVirtualDomain($SW->getWorkingDomainName());
        if ($is_enabled !== true) {
            Qmail::addVirtualDomain($SW->getWorkingDomainName());
            $is_enabled = Qmail::checkVirtualDomain($SW->getWorkingDomainName());
        }

        $this->assertTrue($is_enabled, 'Initial status was unexpected!');
    }

    /**
     * Tests that changing local delivery status works.
     */
    public function testLocalDelivery()
    {
        $SW               = $this->getSWAccount();
        $was_enabled      = Qmail::checkVirtualDomain($SW->getWorkingDomainName());
        $was_in_rpcthosts = Qmail::checkRpctHost($SW->getWorkingDomainName());

        $input = array( 'local_delivery_status' => (int)!$was_enabled );
        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'localDelivery', $input, $SW);

        $this->assertReplySuccessful($Reply);

        $is_enabled   = Qmail::checkVirtualDomain($SW->getWorkingDomainName());
        $in_rpcthosts = Qmail::checkRpctHost($SW->getWorkingDomainName());

        $this->assertNotEquals(
            $is_enabled,
            $was_enabled,
            'Toggling status did not happen'
        );

        $this->assertNotEquals(
            $in_rpcthosts,
            $was_in_rpcthosts,
            'Rpcthosts toggling did not happen'
        );
    }

    /**
     * Re-runs testLocalDelivery() to test toggling it back.
     */
    public function testLocalDeliveryAgain()
    {
        $this->testLocalDelivery();
    }

    /**
     * Test to make sure enabling local delivery on all domains works.
     */
    public function testLocalDeliveryEnabledOnAllDomains()
    {
        $SW = $this->getSWAccount();

        if (!$SW->domainExists('addedbytest2.net')) {
            $SW->addSlaveDomain('addedbytest2.net');
        }

        $input = array( 'local_delivery_status' => '1',
                    'all_domains'           => '1' );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'localDelivery', $input, $SW);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test to make sure disabling local delivery on all domains works.
     */
    public function testLocalDeliveryDisabledOnAllDomains()
    {
        $SW = $this->getSWAccount();

        if (!$SW->domainExists('addedbytest2.net')) {
            $SW->addSlaveDomain('addedbytest2.net');
        }

        $input = array( 'local_delivery_status' => '0',
                    'all_domains'           => '1' );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'localDelivery', $input, $SW);
        $this->assertReplySuccessful($Reply);
    }


    /**
     * This test checks that certain menu items are not appearing when local.
     *
     * mail delivery is disabled.
     */
    public function testSiteWorxMenuItemsWhenLocalDeliveryIsDisabled()
    {
        $SW = $this->getSWAccount();

        $is_enabled = Qmail::checkVirtualDomain($SW->getWorkingDomainName());
        if ($is_enabled === true) {
            Qmail::delVirtualDomain($SW->getWorkingDomainName());
        }

        IWorxMenuFactory::clearInstanceCache();

        $MenuFactory = new IWorxMenuFactory($SW, User::MENU_SMALL);
        $Menu        = $MenuFactory->getMenu();

        $elements_found    = $this->_getEnabledMenuItems($Menu);
        $expected_elements = $this->_getExpectedMailMenuIds();

        /**
         * Done because when the first parameter of in_array was an array (which.
         *
         * according to PHP Docs is supported) then the function fails.
         */
        $test_result = true;
        foreach ($expected_elements as $element) {
            $test_result = $test_result && !in_array($element, $elements_found);
        }

        $error_msg = 'Unexpected menu items were found when Local Mail Delivery' .
        'was disabled';
        $this->assertTrue($test_result, $error_msg);
    }

    /**
     * This test checks that certain menu items are appearing when local mail.
     *
     * delivery is enabled.
     */
    public function testSiteWorxMenuItemsWhenLocalDeliveryIsEnabled()
    {
        $SW = $this->getSWAccount();

        $is_enabled = Qmail::checkVirtualDomain($SW->getWorkingDomainName());
        if ($is_enabled !== true) {
            Qmail::addVirtualDomain($SW->getWorkingDomainName());
        }

        IWorxMenuFactory::clearInstanceCache();

        $MenuFactory = new IWorxMenuFactory($SW, User::MENU_SMALL);
        $Menu        = $MenuFactory->getMenu();

        $elements_found    = $this->_getEnabledMenuItems($Menu);
        $expected_elements = $this->_getExpectedMailMenuIds();

        /**
         * Done because when the first parameter of in_array was an array (which.
         *
         * according to PHP Docs is supported) then the function fails.
         */
        $test_result = true;
        foreach ($expected_elements as $element) {
            $test_result = $test_result && in_array($element, $elements_found);
        }

        $error_msg = 'Expected menu items were not found when Local Mail Delivery' .
        'was enabled';
        $this->assertTrue($test_result, $error_msg);
    }

    /**
     * Returns an array of all the menu ids in mail which should be variable.
     *
     * based on the status of local mail delivery.
     *
     * @return array
     */
    private function _getExpectedMailMenuIds()
    {
        return array( 'iw-menu-features-email-box',
                  'iw-menu-features-email-alias',
                  'iw-menu-features-email-group',
                  'iw-menu-features-email-auto',
                  'iw-menu-admin-mailserver-filters',
                  'iw-menu-admin-mailserver-spam' );
    }

    /**
     * Recursave function.
     *
     * Looks through the given menu and all of it's children and returns an
     * array of menu ids which are presently enabled.
     *
     * @param IWorxMenu $Menu
     * @return array
     */
    private function _getEnabledMenuItems(IWorxMenu $Menu)
    {
        /*
         * Recurse through all the children.
         */
        $enabled_ids = [];
        $children    = $Menu->getChildren();
        if (!empty($children)) {
            foreach ($children as $Child) {
                $tmp_ids     = $this->_getEnabledMenuItems($Child);
                $enabled_ids = array_merge($enabled_ids, $tmp_ids);
            }
        }

        /**
         * Now we check the current menu id, and if it is enabled then we add.
         * it to the enabled element array.
         */
        $menu_array = $Menu->toArray();
        if ($menu_array['is_disabled'] == false) {
            $enabled_ids[] = $menu_array['id'];
        }
        return $enabled_ids;
    }


    /**
     * Gets the record id of an MX record.
     *
     * @param string $pref
     * @return string|false
     */
    private function _getMXRecordId($pref = self::PREF_NUMBER)
    {
        $records = $this->_getMxRecords();
        foreach ($records as $r) {
            if ($r['target'] === "{$pref} " . self::HOST) {
                return $r['record_id'];
            }
        }

        return false;
    }

    /**
     * Gets an array of the existing MX records.
     *
     * @return array
     */
    private function _getMxRecords()
    {
        $SW = $this->getSWAccount();

        return $SW->getWorkingZone()->getRecords(
            'type = "MX"',
            'CONVERT( target, UNSIGNED )'
        );
    }

    /**
     * Deletes an MX Record.
     *
     * @param string $pref Preference Number to delete
     * @return boolean
     */
    private function _deleteMxRecord($pref = self::PREF_NUMBER)
    {
        $record_id = $this->_getMXRecordId($pref);

        if ($record_id === false) {
            return true;
        }

        $SW = $this->getSWAccount();

        $input = array( 'record_id' => $record_id );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);

        if ($Reply->wasSuccessful() !== true) {
            echo 'Unable to delete existing MX Record!';
            return false;
        }
        return true;
    }
}
