<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Brandon Locke <blocke@liquidweb.com>
 * @date    Jan 16, 2023
 */

namespace tests\integration\Ctrl\Siteworx;

use Ctrl_Siteworx_Logs;
use Exception;
use Ini;
use IW;
use IWorx_Framework_TestCase;
use IWorxFileSys;
use Spyc;

/**
 * Tests the SiteWorx Logs Controller.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class LogsTest extends IWorx_Framework_TestCase
{
    private const CTRL        = Ctrl_Siteworx_Logs::class;
    private const LOG_GROUP   = 'php-fpm';

    /**
     * Tear down after tests
     * @return void
     */
    public function tearDown(): void
    {
        IWorxFileSys::rm($this->_getLogsDYamlFilePath(), true, true);
        self::deleteSWAccountIfNecessary();
    }

    /**
     * Ensure that if a group is not declared, return http logs
     * This was the previous behavior of the call, so we can't break it
     *
     * @return void
     * @throws Exception
     */
    public function testListLogsWithoutGroupReturnsHttp(): void
    {
        $SW    = self::getSWAccount();
        $input = [];
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listLogs',
            $input,
            $SW
        );
        $this->assertReplySuccessful($Reply);
        $httpd_log_dir = $SW->getLocalStateDir('logs');
        foreach ($Reply->getPayloadData() as $item) {
            $this->assertStringStartsWith($httpd_log_dir, $item->log_file);
            $basename = basename($item->log_file);
            $this->assertRegexp('/(error|transfer)-\d{4}-\d{2}-\d{2}.log/', $basename);
        }
    }

    /**
     * Ensure that declaring a group that doesn't exist causes an error
     *
     * @return void
     * @throws Exception
     */
    public function testListLogsWithNonExistentGroupFails(): void
    {
        $SW    = self::getSWAccount();
        $input = ['group' => 'idontexist'];
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listLogs',
            $input,
            $SW
        );
        $this->assertReplyNotSuccessful(
            $Reply,
            'Supplying non-existent group should have failed.'
        );
    }

    /**
     * Setup a custom log group and ensure the logs appear in listLogs
     *
     * @return void
     * @throws Exception
     */
    public function testListLogsCustomLogReturnsWithGroup(): void
    {
        $this->_createLogsDFile();
        $this->_createTestLogDir();
        $this->_createTestLogFile();

        $SW    = self::getSWAccount();
        $input = ['group' => self::LOG_GROUP];
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listLogs',
            $input,
            $SW
        );
        $this->assertReplySuccessful($Reply);
        $data = $Reply->getPayloadData();
        $this->assertCount(1, $data);
        $this->assertEquals($this->_getCustomLogFilePath(), $data[0]->log_file);
    }

    /**
     * Ensure that zip files in the logs dir show up in listLogs
     *
     * @return void
     * @throws Exception
     */
    public function testListLogsZipFilesShowInHttpGroup(): void
    {
        $this->_createZipInLogDir();

        $SW    = self::getSWAccount();
        $input = ['group' => 'Http'];
        $Reply = IW::FC()->routeFromPHP(
            self::CTRL,
            'listLogs',
            $input,
            $SW
        );
        $this->assertReplySuccessful($Reply);
        $data = $Reply->getPayloadData();
        $zip_file_found = false;
        foreach ($data as $log) {
            if ($log->log_file == $this->_getZipLogFilePath()) {
                $zip_file_found = true;
                break;
            }
        }
        $this->assertTrue($zip_file_found);
    }

    /**
     * Create a custom log group
     *
     * @return void
     */
    private function _createLogsDFile(): void
    {
        $logs_d_file = $this->_getLogsDYamlFilePath();

        $content = [
            'test' => [
                'name' => 'Php-Fpm Logs',
                'path' => '/home/{$user}/var/php-fpm/*.log',
                'group' => self::LOG_GROUP
            ]
        ];
        $yaml = Spyc::YAMLDump($content, 2, 0, true);
        IWorxFileSys::barf($logs_d_file, $yaml);
    }

    /**
     * Create a custom log dir in our test account
     *
     * @return void
     */
    private function _createTestLogDir(): void
    {
        IWorxFileSys::mkdir($this->_getCustomLogDir());
    }

    /**
     * Create a custom log in our custom log directory
     *
     * @return void
     */
    private function _createTestLogFile(): void
    {
        IWorxFileSys::smartBarf($this->_getCustomLogFilePath(), '');
    }

    /**
     * Get the location of a custom log definition in ~iworx/etc/logs.d/siteworx
     *
     * @return string
     */
    private function _getLogsDYamlFilePath(): string
    {
        $sw_logs_d = Ini::get(Ini::IWORX_DIR, 'siteworx_custom_logs');
        return "$sw_logs_d/test.yaml";
    }

    /**
     * Get the path of the custom log file in our test account
     *
     * @return string
     */
    private function _getCustomLogFilePath(): string
    {
        return "{$this->_getCustomLogDir()}/test.log";
    }

    /**
     * Get the path of the custom log directory in /home/unixuser/var
     * This mimics the location used by Nexcess for php-fpm logs
     *
     * @return string
     */
    private function _getCustomLogDir(): string
    {
        $SW = self::getSWAccount();
        $root       = $SW->getPackageRoot();
        $localstate = Ini::get(Ini::IWORX_LOCALSTATE_DIR, 'base');
        return "$root/$localstate/php-fpm";
    }

    /**
     * Create a zip file in the log directory of our test account
     *
     * @return void
     */
    private function _createZipInLogDir(): void
    {
        IWorxFileSys::smartBarf($this->_getZipLogFilePath(), '');
    }

    /**
     * Get the path to the zip file based on our test account
     *
     * @return string
     */
    private function _getZipLogFilePath(): string
    {
        $SW = self::getSWAccount();
        $dir = $SW->getLocalStateDir('logs');
        return "$dir/test.zip";
    }
}
