<?php

use Zend\Stdlib\Message;

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2023 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Apr 28, 2023
 */

class AutoresponseTest extends IWorx_Framework_TestCase
{
    private const TO_BOX_NAME = 'to_testbox';
    private const FROM_BOX_NAME = 'from_testbox';
    private const AUTORESPONDER_BOX_NAME = 'autoresponder';

    private const DOMAIN_NAME = 'somedomain';
    private const POINTER_HOSTNAME = 'somepointer.com';

    private const PASSWORD = 'somePassWd456';

    private $_SW;

    /**
     * add sw acct
     */
    public static function setUpBeforeClass(): void
    {
        $master_domain = self::_getFullMasterDomainName();
        $SW = new SiteWorx($master_domain);

        if (!$SW->exists()) {
            $input = self::_getSWAccountInput();

            $input['master_domain'] = $master_domain;
            $input['uniqname']      = self::DOMAIN_NAME;
            $input['nickname']      = self::DOMAIN_NAME;

            self::addSWAccountFromInput($input);
        }

        parent::setUpBeforeClass();
    }

    /**
     * delete sw acct
     */
    public static function tearDownAfterClass()
    {
        $master_domain = self::_getFullMasterDomainName();
        self::deleteSWAccountIfNecessaryStatic($master_domain);
    }

    /**
     * @return string
     */
    private static function _getFullMasterDomainName()
    {
        return implode('.', [self::DOMAIN_NAME, IWorxSysInfo::hostname()]);
    }

    /**
     * test vacation responder for pointer domain
     *
     * expects "from" to be from pointer
     */
    public function testVacationResponder()
    {
        $this->_addMailBox(self::FROM_BOX_NAME);
        $this->_addMailBox(self::TO_BOX_NAME);
        $this->_enableVacationMessage(self::TO_BOX_NAME);

        $hostname = $this->_getMasterDomainName();
        $before_count = $this->_getFromBoxMessageCount();

        $this->_sendEmail(self::TO_BOX_NAME, self::FROM_BOX_NAME, $hostname);

        sleep(10);

        $expected_from = $this->_getEmailAddress(self::TO_BOX_NAME, $hostname);
        $this->_checkIncomingMailContainsResponseFrom($expected_from, $before_count);
    }

    /**
     * test vacation responder for pointer domain
     *
     * expects "from" to be from pointer
     */
    public function testVacationResponderPointer()
    {
        $hostname = self::POINTER_HOSTNAME;
        $before_count = $this->_getFromBoxMessageCount();
        $this->_addPointerDomain(self::POINTER_HOSTNAME, Domain::PTR_ALIAS, $this->_getMasterDomainName(), $this->_getSWAccount());

        $this->_sendEmail(self::TO_BOX_NAME, self::FROM_BOX_NAME, $hostname);

        sleep(10);

        $expected_from = $this->_getEmailAddress(self::TO_BOX_NAME, $hostname);
        $this->_checkIncomingMailContainsResponseFrom($expected_from, $before_count);
    }

    /**
     * test autoresponder returns autoresponse
     */
    public function testAutoResponder()
    {
        // will receive both vacation message and autoresponse
        // if we don't disable it here
        $this->_disabledVacationMessage(self::TO_BOX_NAME);

        $hostname = self::_getMasterDomainName();
        $this->_testAutoresponder($hostname);
    }

    /**
     * test autoresponder "from" header says pointer domain
     */
    public function testAutoResponderPointer()
    {
        $this->_testAutoresponder(self::POINTER_HOSTNAME);
    }

    /**
     * @param string $hostname
     */
    private function _testAutoresponder($hostname)
    {
        $this->_activateAutoresponder();
        $before_count = $this->_getFromBoxMessageCount();

        $this->_sendEmail(self::AUTORESPONDER_BOX_NAME, self::FROM_BOX_NAME, $hostname);
        sleep(10);

        $expected_from = $this->_getEmailAddress(self::AUTORESPONDER_BOX_NAME, $hostname);
        $this->_checkIncomingMailContainsResponseFrom($expected_from, $before_count);
    }

    /**
     * @param string $box_name
     * @param string $host
     */
    private function _getEmailAddress($box_name, $host)
    {
        return "$box_name@$host";
    }

    /**
     * @return string
     */
    private function _getMasterDomainName()
    {
        return $this->_getSWAccount()->getMasterDomainName();
    }

    /**
     * @param string $box_name
     */
    private function _addMailBox($box_name)
    {
        $SW = $this->_getSWAccount();
        $Driver = $SW->getIncomingMailDriver();

        if (!$Driver->boxExists($box_name)) {
            $success = $Driver->addBox($box_name, self::PASSWORD);
            $this->assertTrue($success);
        }
    }

    /**
     * @return SiteWorx
     */
    private function _getSWAccount()
    {
        if (!isset($this->_SW)) {
            $this->_SW = new SiteWorx($this->_getFullMasterDomainName());
        }
        return $this->_SW;
    }

    /**
     * @param string $box_name
     */
    private function _enableVacationMessage($box_name)
    {
        $input = $this->_getInputVacationResponder($box_name, true);
        $this->_requestVacationResponder($input);
    }

    /**
     * @param string $box_name
     */
    private function _disabledVacationMessage($box_name)
    {
        $input = $this->_getInputVacationResponder($box_name, false);
        $this->_requestVacationResponder($input);
    }

    /**
     * @param string  $box_name
     * @param boolean $enable
     */
    private function _getInputVacationResponder($box_name, $enable)
    {
        return [
            'username' => $box_name,
            'onvacation' => $enable ? '1' : '0',
            'vacation_message' => 'some vacation message'
        ];
    }

    /**
     * @param array $input
     */
    private function _requestVacationResponder(array $input)
    {
        $Reply = IW::FC()->routeFromPHP(
            Ctrl_Siteworx_EmailBox::class,
            'editCommit',
            $input,
            $this->_getSWAccount()
        );

        $this->assertTrue($Reply->wasSuccessful(), 'vacation responder request failed: ' . $Reply->getErrorsAsString());
    }

    /**
     * activate autoresponder
     */
    private function _activateAutoresponder()
    {
        $SW = $this->_getSWAccount();
        $Driver = $SW->getIncomingMailDriver();

        if ($Driver->autoresponderExists(self::AUTORESPONDER_BOX_NAME)) {
            return;
        }

        $input = [
            'username' => self::AUTORESPONDER_BOX_NAME,
            'copyto' => $this->_getEmailAddress(self::TO_BOX_NAME, $this->_getMasterDomainName()),
            'autorespondmessage' => 'some autoresponse message'
        ];

        $Reply = IW::FC()->routeFromPHP(Ctrl_Siteworx_EmailAutorespond::class, 'add', $input, $SW);
        $this->assertReplySuccessful($Reply);
    }


    /**
     * @param string $to
     * @param string $from
     * @param string $hostname
     */
    private function _sendEmail($to, $from, $hostname)
    {
        $email_to = $this->_getEmailAddress($to, $hostname);
        $email_from = $this->_getEmailAddress($from, $hostname);

        $Mail = new IWorx_Mail($email_from);
        $Mail->addTo($email_to);
        $Mail->setReplyTo($email_from);
        $Mail->setFrom($email_from);
        $Mail->setBodyHtml('<div>some message</div>');

        $config_host = $this->_getMasterDomainName();
        $config = [
            'name' => $config_host,
            'host'=> $config_host,
            'auth' => 'login',
            'username' => $email_from,
            'password' => self::PASSWORD
        ];


        $Smtp = new Zend_Mail_Transport_Smtp($config_host, $config);
        $Mail->send($Smtp);
    }

    /**
     * @param string  $expected_from
     * @param integer $before_count
     */
    private function _checkIncomingMailContainsResponseFrom($expected_from, $before_count)
    {
        $actual_message_count = $this->_getFromBoxMessageCount();
        $expected_message_count = ++$before_count;
        $box_name = self::FROM_BOX_NAME;

        $this->assertSame($expected_message_count, $actual_message_count, "expected mailbox $box_name to have received $expected_message_count messages!");

        $Mail = new IWorx_Mail();
        $Message = $Mail->getMessageById($this->_getMaildirPath($box_name), $expected_message_count);
        $headers = $Message->getHeaders();
        $from = $headers['from'];

        $this->assertSame($expected_from, $from, "Unexpected 'from' header!");
    }

    /**
     * @param string $user
     * @return string
     */
    private function _getMaildirPath($user)
    {
        $SW = $this->_getSWAccount();
        $hostname = $SW->getMasterDomainName();
        return "/home/vpopmail/domains/$hostname/$user/Maildir";
    }

    /**
     * @return integer
     */
    private function _getFromBoxMessageCount()
    {
        $Mail = new IWorx_Mail();
        $maildir = $this->_getMaildirPath(self::FROM_BOX_NAME);
        return $Mail->getMessageCount($maildir);
    }
}
