<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Tim College <tcollege@interworx.com>
 * @date       Jun 30, 2011
 */

/**
 * Tests for missing translations.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class MissingTranslationsTest extends IWorx_Framework_TestCase
{
    /**
     * Checks if a translation exists in English.
     *
     * @param string $lg_key
     * @param string $class
     * @param string $source
     * @dataProvider dataProviderFormKeys
     */
    public function testMissingFormLanguageKeys($lg_key, $class, $source)
    {
        $this->assertTrue(IWorxTranslate::hasTranslation($lg_key));
    }

    /**
     * Checks if a form is instantiable without arguments.
     *
     * @param string $class
     * @return boolean
     */
    public static function isStandardFormInit($class)
    {
        $RC = new ReflectionClass($class);

        if ($RC->isSubclassOf('Form') === false) {
            return false;
        }

        if ($RC->isAbstract()) {
            return false;
        }

        $Function = $RC->getMethod('_initialize');

        $Parameters = $Function->getParameters();

        if (count($Parameters) > 0) {
            return false;
        }

        return true;
    }

    /**
     * Extracts multiple language keys from a single string.
     *
     * @param string $string
     * @param string $class
     * @param string $source
     * @param array  $output
     */
    private static function _extractMultKeys(
        $string,
        $class,
        $source,
        array &$output
    ) {
        $matches = [];
        preg_match_all('/((LG_[A-Z0-9_]+)(\(?))/', $string, $matches, PREG_SET_ORDER);

        if (count($matches) === 2 && $matches[0][1] === $matches[1][1]) {
            if (strpos($string, 'const') !== false) {
                return;
            }
        }

        foreach ($matches as $match) {
            $lg_key = trim($match[1], '#');

            if (substr($match[1], -1) === '_') {
                continue;
            }

            if (substr($match[1], -1) === '(') {
                continue;
            }

            if ($lg_key === 'LG_OPT' || $lg_key === 'LG_CHARSET') {
                continue;
            }

            $output[] = array( 'lg_key' => "##$lg_key##",
                         'class'  => $class,
                         'source' => $source );
        }
    }

    /**
     * Extracts lang keys from a Form.
     *
     * @param Form  $Form
     * @param array $output
     */
    private static function _extractFormLang(Form $Form, array &$output)
    {
        self::_extractMultKeys(
            $Form->getLabel(),
            get_class($Form),
            'Form label',
            $output
        );

        foreach ($Form as $Group) {
            foreach ($Group as $Input) {
                /* @var $Input Input_Input */
                if ($Input instanceof Input_Hidden) {
                    // These labels are never displayed, fuckem.
                    continue;
                }

                self::_extractMultKeys(
                    $Input->getLabel(),
                    get_class($Form),
                    'Input label of ' . $Input->getName(),
                    $output
                );

                if ($Input->getQuickHelp()) {
                    $output[] = array( 'lg_key' => $Input->getQuickHelp(true),
                             'class'  => get_class($Form),
                             'source' => 'Input quickhelp of ' . $Input->getName() );
                }

                if ($Input instanceof Input_Set) {
                    $DS = $Input->getDataSource();

                    foreach ($DS->getDisplayValues() as $value) {
                        $qh = $Input->getQuickhelpForDisplayValue($value);

                        if ($qh) {
                            $output[] = array( 'lg_key' => $qh,
                                 'class'  => get_class($Form),
                                 'source' => 'Display Value quickhelp of ' .
                                 $Input->getName() . ' value ' .
                                 $value );
                        }
                    }
                }
            }//end foreach
        }//end foreach
    }

    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderFormKeys()
    {
        $files = IWorxFileSys::rglob(
            '*.php',
            0,
            '/usr/local/interworx/include/Form'
        );

        $output = [];

        foreach ($files as $file) {
            try {
                $parts = explode('/', $file);
                array_shift($parts); // remove /
                array_shift($parts); // remove home/
                array_shift($parts); // remove interworx/
                array_shift($parts); // remove include/
                array_shift($parts); // remove include/
                $class = implode('_', $parts);
                $class = substr($class, 0, -4);

                if (class_exists($class) === false) {
                    continue;
                }

                if (self::isStandardFormInit($class)) {
                    Form::resetInstances();
                    IW::Env()->resetSession();

                    // Clear the input values
                    $input = IW::Env()->getAllInput();

                    foreach ($input as $name => $value) {
                        IW::Env()->addInput($name, null);
                    }

                    $Form = Form::getInstance($class);

                    self::_extractFormLang($Form, $output);

                    if ($Form instanceof Form_Input) {
                        foreach ($Form as $Group) {
                            foreach ($Group as $Input) {
                                if ($Input->getDataSource()) {
                                    $DS = $Input->getDataSource();
                                    $values = $DS->getValues();
                                    $value = $values[0];
                                    IW::Env()->addInput($Input->getName(), $value);
                                }
                            }
                        }

                        Form::resetInstances();

                        $Form = Form::getInstance($class);

                        if ($Form instanceof Form_Input === false) {
                            self::_extractFormLang($Form, $output);
                        } else {
                            if (!is_subclass_of($class, 'Form_Confirm')) {
                                // meh.
                            }
                        }
                    }//end if
                } else {
                    //echo "Non standard constructor problem\n";
                }//end if
            } catch (Exception $e) {
                // fuckit.
            }//end try
        }//end foreach

        return $output;
    }
}
