<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Apr 17, 2009
 */

/**
 * Tests to make sure the correct data carried over on import.
 *
 * @package    InterWorx
 * @subpackage Tests
 * @group      slow-tests
 */
class MySQL_UserTest extends IWorx_Framework_TestCase
{
    const EMPTY_USER_NAME = '';
    const EMPTY_PASSWORD = '';
    const EMPTY_HOST = null;
    const NON_PREFIXED_USER = 'testusr';
    const PREFIXED_USER = 'addedbyt_testusr';
    const NON_PREFIXED_DB = 'testdb';
    const PASSWORD = 'abc123';

    /**
     * Tests the MySQL users are reimported properly on import.
     */
    public function testMySQLUsersPresistOnImport()
    {
        self::addSWAccountIfNecessary();
        $expected_users = $this->_makeDummyUsers();

        $this->doBackupAndImport();

        $users_same = $this->_expectedUsersMatchActual($expected_users);
        $this->assertTrue($users_same === true);
    }

    /**
     * Make a number of dummy MySQL users.
     *
     * @return array
     */
    private function _makeDummyUsers()
    {
        $SiteWorx = new SiteWorx(self::SW_DOMAIN);
        $Driver   = $SiteWorx->getMySQLDriver();

        for ($i = 1; $i <= 10; $i++) {
            if ($Driver->userExists("test{$i}")) {
                $Driver->deleteUser("test{$i}");
            }//end if
            $Driver->addUser("test{$i}", 'test');
        }//end for

        return $Driver->getUsers();
    }

    /**
     * Gets the expected users parameter and then checks the actual users to make.
     * sure they are the same.
     *
     * @param array $expected
     * @return boolean
     */
    private function _expectedUsersMatchActual(array $expected)
    {
        $SiteWorx = new SiteWorx(self::SW_DOMAIN);
        $Driver   = $SiteWorx->getMySQLDriver();
        $actual   = $Driver->getUsers();

        $all_good = true;
        $all_good = $all_good && count($expected) == count($actual);

        foreach ($expected as $e_user) {
            $a_user = $this->_getUserFromArray($e_user, $actual);

            if (!is_null($a_user) &&
            ($e_user->fqun == $a_user->fqun ) &&
            ($e_user->crypt_password == $a_user->crypt_password ) ) {
                $all_good = $all_good && true;
            } else {
                $all_good = false;
                break;
            }
        }
        return $all_good;
    }

    /**
     * Search over the array of user objects and return the one which matches the.
     * host and fully qualified user name.
     *
     * @param string $user
     * @param array  $users_array
     * @return object|null
     */
    private function _getUserFromArray($user, array $users_array)
    {
        $fqun = $user->fqun;

        foreach ($users_array as $key => $search_user) {
            if (( $search_user->fqun == $fqun )) {
                return $search_user;
            }//end if
        }//end foreach
        return null;
    }

    /**
     * Test all addUser/addUserFQ exceptions
     */
    public function testAddUserExceptions()
    {
        self::deleteSWAccountIfNecessary();

        if ($this->getSWAccountExists('addedbytest2.com')) {
            self::deleteSWAccountStatic('addedbytest2.com');
        }


        $Driver = $this->_getDriver();

        //empty user name
        $caught = false;
        try {
            $Driver->addUser(self::EMPTY_USER_NAME, self::PASSWORD);
        } catch (IWorx_Exception $e) {
            $caught = true;
            $this->assertEquals($e->getMessage(), 'MySQL user name is empty.');
        }
        $this->assertTrue($caught);

        //empty password
        $caught = false;
        try {
            $Driver->addUser(self::PREFIXED_USER, self::EMPTY_PASSWORD);
        } catch (IWorx_Exception $e) {
            $caught = true;
            $this->assertEquals($e->getMessage(), 'MySQL user password is empty.');
        }
        $this->assertTrue($caught);

        //userExists
        $caught = false;
        $Driver->addUser(self::NON_PREFIXED_USER, self::PASSWORD);
        try {
            $Driver->addUser(self::NON_PREFIXED_USER, self::PASSWORD);
        } catch (IWorx_Exception $e) {
            $caught = true;
            $this->assertEquals($e->getMessage(), self::NON_PREFIXED_USER . ' MySQL user already exists.');
        }
        $this->assertTrue($caught);

        //cleanup
        $Driver->deleteUser(self::NON_PREFIXED_USER);
        $this->assertFalse($Driver->userExists(self::NON_PREFIXED_USER));

        //empty host
        $caught = false;
        try {
            $Driver->addUser(self::PREFIXED_USER, self::PASSWORD, self::EMPTY_HOST);
        } catch (IWorx_Exception $e) {
            $caught = true;
            $this->assertEquals($e->getMessage(), 'MySQL host is empty.');
        }
        $this->assertTrue($caught);

        //addUserFQ
        $caught = false;
        try {
            $Driver->addUserFQ(self::EMPTY_USER_NAME, self::PASSWORD);
        } catch (IWorx_Exception $e) {
            $caught = true;
            $this->assertEquals($e->getMessage(), 'MySQL user name is empty.');
        }
        $this->assertTrue($caught);

        //empty password
        $caught = false;
        try {
            $Driver->addUserFQ(self::NON_PREFIXED_USER, self::EMPTY_PASSWORD);
        } catch (IWorx_Exception $e) {
            $caught = true;
            $this->assertEquals($e->getMessage(), 'MySQL user password is empty.');
        }
        $this->assertTrue($caught);

        //empty host
        $caught = false;
        try {
            $Driver->addUserFQ(self::NON_PREFIXED_USER, self::PASSWORD, self::EMPTY_HOST);
        } catch (IWorx_Exception $e) {
            $caught = true;
            $this->assertEquals($e->getMessage(), 'MySQL host is empty.');
        }
        $this->assertTrue($caught);


        //userExists same username
        $caught = false;
        $Driver->addUserFQ(self::NON_PREFIXED_USER, self::PASSWORD);
        try {
            $Driver->addUserFQ(self::NON_PREFIXED_USER, self::PASSWORD);
        } catch (IWorx_Exception $e) {
            $caught = true;
            $this->assertEquals($e->getMessage(), self::NON_PREFIXED_USER . ' MySQL user already exists.');
        }
        $this->assertTrue($caught);

        //cleanup
        $Driver->deleteUserFQ(self::NON_PREFIXED_USER);

        //userExists same username
        $caught = false;
        $Driver->addUserFQ(self::PREFIXED_USER, self::PASSWORD);
        try {
            $Driver->addUserFQ(self::NON_PREFIXED_USER, self::PASSWORD);
        } catch (IWorx_Exception $e) {
            $caught = true;
            $this->assertEquals($e->getMessage(), self::NON_PREFIXED_USER . ' MySQL user exists.');
        }
        $this->assertTrue($caught);

        //cleanup
        $Driver->deleteUserFQ(self::PREFIXED_USER);

        //userExists same username other way
        $caught = false;
        $Driver->addUserFQ(self::NON_PREFIXED_USER, self::PASSWORD);
        try {
            $Driver->addUserFQ(self::PREFIXED_USER, self::PASSWORD);
        } catch (IWorx_Exception $e) {
            $caught = true;
            $this->assertEquals($e->getMessage(), self::NON_PREFIXED_USER . ' MySQL user exists.');
        }
        $this->assertTrue($caught);

        //cleanup
        $Driver->deleteUserFQ(self::NON_PREFIXED_USER);


        // test systemDatabaseUserExistsFQ throw in addUserFQ
        $caught = false;
        $Driver2 = $this->_getDriver2();
        $Driver2->addUserFQ(self::NON_PREFIXED_USER, self::PASSWORD);
        $Driver2->addDatabaseFQ(self::NON_PREFIXED_DB);
        $Driver2->editPermissionsFQ(self::NON_PREFIXED_USER, self::NON_PREFIXED_DB, ['INSERT']);
        try {
            $Driver->addUserFQ(self::NON_PREFIXED_USER, self::PASSWORD);
        } catch (IWorx_Exception $e) {
            $caught = true;
            $this->assertEquals($e->getMessage(), self::NON_PREFIXED_USER . ' is related to another database.');
        }
        $this->assertTrue($caught);

        //cleanup
        self::deleteSWAccountStatic('addedbytest2.com');
    }

    /**
     * Gets the Database Driver.
     *
     * @return MySQL
     */
    private function _getDriver()
    {
        self::addSWAccountIfNecessary();

        $SiteWorx = new SiteWorx(self::SW_DOMAIN);
        $Driver   = $SiteWorx->getMySQLDriver();

        return $Driver;
    }

    /**
     * Gets the Database Driver.
     *
     * @return MySQL
     */
    private function _getDriver2()
    {
        if ($this->getSWAccountExists('addedbytest2.com')) {
            self::deleteSWAccountStatic('addedbytest2.com');
        }
        $input = ['master_domain' => 'addedbytest2.com',
                    'uniqname' => 'addbyts2',
                    'nick' => 'abt2'];

        self::addSWAccount(false, $input);

        $SiteWorx = new SiteWorx('addedbytest2.com');
        $Driver2   = $SiteWorx->getMySQLDriver();

        return $Driver2;
    }
}
