<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2017 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Aug 28, 2017
 */

require_once 'Abstract.php';

/**
 * Class Php_PerDomainTest
 *
 * @package    InterWorx
 * @subpackage TEST
 */
class Php_PerDomainTest extends Php_Abstract
{
    const PHP_VERSIONS = [ '/opt/remi/php70', '/opt/remi/php71' ];
    const CTRL         = 'Ctrl_Siteworx_DomainsSlave';

    /**
     * Setup.
     */
    public static function setUpBeforeClass()
    {
        if (PHP::isMultiplePhpInstalled() === false) {
            PHP::enableMultiplePhp();
        }

        foreach (self::PHP_VERSIONS as $version) {
            $Php = new PhpFpm($version);
            if ($Php->isInstalled() === false) {
                PHP::install($version);
            }

            if ($Php->isEnabled() === false) {
                PHP::setEnabledVersions(array_merge(
                    PHP::getEnabledVersions(),
                    [$version]
                ));
            }
        }
    }

    /**
     * Add the siteworx account.
     */
    public function testAddingSwAccount()
    {
        $this->deleteSWAccount();
        self::addSWAccountFromInput(self::_getSWInput());
        $this->assertTrue($this->getSWAccountExists());
    }

    /**
     * Test that the default PHP version is what we want.
     */
    public function testDefaultPhpVersionIsExpected()
    {
        if ($this->getSWAccountExists() === false) {
            self::addSWAccountFromInput(self::_getSWInput());
        }

        $SW = $this->getSWAccount();
        $this->assertEquals(self::PHP_VERSIONS[0], $SW->getDefaultPhpVersion());
    }

    /**
     * Test that the versions were marked as available to the SiteWorx account.
     */
    public function testAvailablePhpVersionsIsExpected()
    {
        if ($this->getSWAccountExists() === false) {
            self::addSWAccountFromInput(self::_getSWInput());
        }

        $SW = $this->getSWAccount();

        $expected = self::PHP_VERSIONS;
        sort($expected);
        $actual = $SW->getPhpAvailable();
        sort($actual);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Verify expected count of "in use" php.
     */
    public function testCountOfInUsePhp()
    {
        if ($this->getSWAccountExists() === false) {
            self::addSWAccountFromInput(self::_getSWInput());
        }

        $SW = $this->getSWAccount();
        $this->assertCount(1, $SW->getPhpVersionsInUse());
    }

    /**
     * Test that the slave domain uses the default PHP in the absence of other input.
     */
    public function testSlaveUsesDefaultModProxyOff()
    {
        $NW = $this->getMasterNodeWorx();
        $Driver = $NW->getHTTPDriver();
        if ($Driver->getVersion() >= '2.4') {
            IW::Env()->setConfig('FPM_USE_MOD_PROXY_FCGI', 0);
            $this->deleteSWAccountIfNecessary();
        }

        if ($this->getSWAccountExists() === false) {
            self::addSWAccountFromInput(self::_getSWInput());
        }

        $SW = $this->getSWAccount();

        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);

        $this->assertReplySuccessful($Reply);

        $Domain = new Domain(self::SW_SECONDARY_DOMAIN);

        $this->assertEquals($SW->getDefaultPhpVersion(), $Domain->getPhpVersion());

        $this->_verifyPhpFpmStuff($SW, $Domain);

        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test that the slave domain uses the default PHP in the absence of other input.
     */
    public function testSlaveUsesDefaultModProxyOn()
    {
        $NW = $this->getMasterNodeWorx();
        $Driver = $NW->getHTTPDriver();
        if ($Driver->getVersion() >= '2.4') {
            IW::Env()->setConfig('FPM_USE_MOD_PROXY_FCGI', 1);
            $this->deleteSWAccountIfNecessary();
        } else {
            $this->markTestSkipped("Doesn't apply to httpd 2.2");
        }

        if ($this->getSWAccountExists() === false) {
            self::addSWAccountFromInput(self::_getSWInput());
        }

        $SW = $this->getSWAccount();

        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);

        $this->assertReplySuccessful($Reply);

        $Domain = new Domain(self::SW_SECONDARY_DOMAIN);

        $this->assertEquals($SW->getDefaultPhpVersion(), $Domain->getPhpVersion());

        $this->_verifyPhpFpmStuff($SW, $Domain);

        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test creating slave domain with different PHP.
     *
     * @depends testSlaveUsesDefaultModProxyOff
     */
    public function testSlaveWithAlternatePhp()
    {
        if ($this->getSWAccountExists() === false) {
            self::addSWAccountFromInput(self::_getSWInput());
        }

        $SW = $this->getSWAccount();

        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );

        IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);

        $expected_php = self::PHP_VERSIONS[1];

        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN,
                    Form_SW_DomainsSlave_Add::INPUT_PHP_VERSION => $expected_php );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);

        $this->assertReplySuccessful($Reply);

        $Domain = new Domain(self::SW_SECONDARY_DOMAIN);

        $this->assertEquals($expected_php, $Domain->getPhpVersion());

        $this->_verifyPhpFpmStuff($SW, $Domain);

        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Test editing a slave domain.
     *
     * @depends testSlaveUsesDefaultModProxyOff
     */
    public function testEditSlaveToAlternatePhp()
    {
        if ($this->getSWAccountExists() === false) {
            self::addSWAccountFromInput(self::_getSWInput());
        }

        $SW = $this->getSWAccount();

        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );

        IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);

        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'add', $input, $SW);

        $this->assertReplySuccessful($Reply);

        $new_php = self::PHP_VERSIONS[1];
        $input   = array( 'domain' => self::SW_SECONDARY_DOMAIN,
                      Form_SW_DomainsSlave_Add::INPUT_PHP_VERSION => $new_php );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'edit', $input, $SW);

        $this->assertReplySuccessful($Reply);

        $Domain = new Domain(self::SW_SECONDARY_DOMAIN);

        $this->assertEquals($new_php, $Domain->getPhpVersion());

        $this->_verifyPhpFpmStuff($SW, $Domain);

        $input = array( 'domain' => self::SW_SECONDARY_DOMAIN );

        $Reply = IW::FC()->routeFromPHP(self::CTRL, 'delete', $input, $SW);

        $this->assertReplySuccessful($Reply);
    }

    /**
     * Get input needed to create SiteWorx account for test.
     *
     * @return array
     */
    private static function _getSWInput()
    {
        $input = self::_getSWAccountInput();
        $input[ Form_NW_Siteworx_Add::INPUT_PHP_VERSION ]  = self::PHP_VERSIONS[0];
        $input[ Form_NW_Siteworx_Add::INPUT_PHP_AVAILABLE] = self::PHP_VERSIONS;
        return $input;
    }

    /**
     * Run through a series of tests on the domain to verify PHP behavior.
     *
     * @param SiteWorx $SW
     * @param Domain   $Domain
     */
    protected function _verifyPhpFpmStuff(SiteWorx $SW, Domain $Domain)
    {
        $this->assertFileExists($Domain->getPhpFpmPoolPath());
        $this->assertFileExists($Domain->getPhpFpmSocketPath());

        $confdir = Ini::get(Ini::APACHE_DIR, 'conf.d');
        $fcgi_defs = "{$confdir}/z01-fastcgi-servers.conf";

        if (IW::Env()->getConfig('FPM_USE_MOD_PROXY_FCGI') == 0) {
            $this->assertFileContains($fcgi_defs, $Domain->getPhpFpmFcgiServerPath());
        }

        $vhost_file = Ini::get(Ini::APACHE_DIR, 'conf.d') .
        "/vhost_{$Domain->getName()}.conf";

        $this->assertFileExists($vhost_file);

        if (IW::Env()->getConfig('FPM_USE_MOD_PROXY_FCGI') == 0) {
            $this->assertFileContains($vhost_file, $Domain->getPhpFpmFcgiServerPath());
        }

        $this->_setUpWebroot($SW, $Domain);

        $this->_verifyHelloWorld($Domain);
        $this->_verifyExpectedUser($Domain, $SW->getUniqname());
        $this->_verifyPhpSapi($Domain, 'fpm-fcgi');

        $this->_verifyVersion($Domain, $Domain->getPhp()->getVersion());
    }
}
