<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2017 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Mar 22, 2017
 */

require_once 'Abstract.php';

/**
 * Class Php_Scl70Test
 *
 * @package    InterWorx
 * @subpackage Test
 */
class Php_Scl70Test extends Php_Abstract
{
    const TEST_DOMAIN      = 'iwtestphpscl70.com';
    const TEST_SCL_VERSION = '/opt/remi/php70';
    const TEST_UNIQNAME    = 'php70tst';
    const TEST_SYSTEM_DEFAULT = 'system-php';

    /**
     * Setup.
     */
    public static function setUpBeforeClass()
    {
        self::deleteSWAccountIfNecessaryStatic(static::TEST_DOMAIN);
        if (PHP::isMultiplePhpInstalled() === false) {
            PHP::enableMultiplePhp();
        }

        $Php = new PhpFpm(static::TEST_SCL_VERSION);
        if ($Php->isInstalled() === false) {
            PHP::install(static::TEST_SCL_VERSION);
        }

        if ($Php->isEnabled() === false) {
            PHP::setEnabledVersions(array_merge(
                PHP::getEnabledVersions(),
                [static::TEST_SCL_VERSION]
            ));
        }


        self::_addSWAccount();
        $SW = new SiteWorx(static::TEST_DOMAIN);
        self::_setUpWebroot($SW);
    }

    /**
     * Loose verification of the setup function.
     */
    public function testSetupFiles()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $SW = new SiteWorx(static::TEST_DOMAIN);

        $filename = $SW->getWebRoot() . '/php-mode-test/hello-world.php';
        $this->assertFileExists($filename);
        $this->assertTrue(is_readable($filename));
    }

    /**
     * Do a simple hello world test.
     */
    public function testPhpHelloWorldWorks()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $this->_verifyHelloWorld(new Domain(static::TEST_DOMAIN));
    }

    /**
     * Verify that php-fpm is in use.
     */
    public function testPhpSapiName()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $this->_verifyPhpSapi(new Domain(static::TEST_DOMAIN), 'fpm-fcgi');
    }

    /**
     * Verify that we're running as the unix user of the account.
     */
    public function testWhoami()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $this->_verifyExpectedUser(
            new Domain(static::TEST_DOMAIN),
            static::TEST_UNIQNAME
        );
    }

    /**
     * Verify that we're running the right version.
     */
    public function testVersion()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $Php = new PhpFpm(static::TEST_SCL_VERSION);
        $this->_verifyVersion(new Domain(static::TEST_DOMAIN), $Php->getVersion());
    }

    /**
     * Test backup and restore with the expectation it'll work.
     */
    public function testBackupRestoreWithCorrectVersionInstalled()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);

        $this->doBackupAndImport(null, static::TEST_DOMAIN);
        $SW  = new SiteWorx(static::TEST_DOMAIN);
        $Php = $SW->getMasterDomain()->getPhp();

        $this->assertEquals(static::TEST_SCL_VERSION, $Php->getKey());
        $this->_verifyPhpSapi($SW->getMasterDomain(), 'fpm-fcgi');
        $this->_verifyVersion($SW->getMasterDomain(), $Php->getVersion());
        $this->_verifyExpectedUser($SW->getMasterDomain(), static::TEST_UNIQNAME);
    }

    /**
     * Test what happens when the correct version is disabled.
     */
    public function testBackupRestoreWithCorrectVersionMissing()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);

        $this->doBackupAndImport('_disablePhpVersion', static::TEST_DOMAIN);
        $SW  = new SiteWorx(static::TEST_DOMAIN);

        $Php = $SW->getMasterDomain()->getPhp();

        $this->assertNotEquals($Php->getVersion(), static::TEST_SCL_VERSION);
        $this->_verifyExpectedUser($SW->getMasterDomain(), static::TEST_UNIQNAME);
    }

    /**
     * Disable the version under test.
     */
    protected function _disablePhpVersion()
    {
        $enabled = PHP::getEnabledVersions();
        $current = [ static::TEST_SCL_VERSION ];

        $new_enabled = array_diff($enabled, $current);

        PHP::setEnabledVersions($new_enabled);
    }

    /**
     * Add the test siteworx account.
     */
    private static function _addSWAccount()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);

        $input                                            = self::_getSWAccountInput();
        $input['master_domain']                           = static::TEST_DOMAIN;
        $input['uniqname']                                = static::TEST_UNIQNAME;
        $input[ Form_NW_Siteworx_Add::INPUT_PHP_VERSION ] = static::TEST_SCL_VERSION;
        $input[ Form_NW_Siteworx_Add::INPUT_PHP_AVAILABLE ] = [static::TEST_SYSTEM_DEFAULT, static::TEST_SCL_VERSION];
        self::addSWAccountFromInput($input);
    }
}
