<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2016 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Test
 * @author     Tim College <tcollege@interworx.com>
 * @date       Dec 15, 2016
 */

/**
 * Class PhpModeTest
 *
 * @package    InterWorx
 * @subpackage Test
 */
class PhpModeTest extends IWorx_Framework_TestCase
{
    /**
     * Set up the siteworx account for the test.
     */
    public static function setUpBeforeClass()
    {
        self::deleteSWAccountIfNecessaryStatic();
        self::_setUpPhpMode();
        self::addSWAccountIfNecessary();
        self::_setUpWebroot();
    }

    /**
     * Prepare the php mode to a known state.
     */
    private function _setUpPhpMode()
    {
        IW::getLog()->info('Test class setting up PHP mode');
        self::_setPhpModeStatic(PHP::MODE_MOD_PHP);
        PHP::restartFpmServices();
        $version = PHP::getVersion(PHP::SYSTEM_PHP);
        Yum::remove([$version->fpm_package_name, 'mod_fastcgi']);
        PHP::refreshAvailableVersions();
        IW::getLog()->info('Test class finished setting up PHP mode');
    }

    /**
     * Prepare the webroot with test files.
     */
    private static function _setUpWebroot()
    {
        $SW = new SiteWorx(self::SW_DOMAIN);
        IWorxFileSys::cp(
            __DIR__ . '/../_resources/php-mode-test',
            $SW->getWebRoot(),
            true
        );
    }

    /**
     * Loose verification of the setup function.
     */
    public function testSetupFiles()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $SW = $this->getSWAccount();

        $filename = $SW->getWebRoot() . '/php-mode-test/hello-world.php';
        $this->assertFileExists($filename);
        $this->assertTrue(is_readable($filename));
    }

    /**
     * Verify that we start with a blank slate here.
     */
    public function testSystemFpmNotInstalled()
    {

        $version = PHP::getVersion(PHP::SYSTEM_PHP);
        $this->assertFalse($version->fpm_installed);
    }

    /**
     * Test the basic replacement in the file.
     *
     * This is not a great test - it's really just making sure that the replacement
     * doesn't mangle the vhost completely - because it was, at one point.
     */
    public function testVhostFileReplacement()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $file = __DIR__ . '/../_resources/php-mode-vhost.conf';
        PHP::setVhostPhpMode(self::SW_DOMAIN, $file);

        $contents = file_get_contents($file);

        $this->assertContains('<VirtualHost', $contents);
    }

    /**
     * Do a simple hello world test for mod_php.
     */
    public function testPhpHelloWorldWorksModPhp()
    {
        Ini::set(Ini::APACHE, 'allow_mod_php', '1');
        Ini::writeAndResetInstance();

        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $this->_setPhpMode(PHP::MODE_MOD_PHP);
        $this->_setOwner('apache');
        $this->_verifyHelloWorld();
    }

    /**
     * Verify that mod_php is in use.
     */
    public function testPhpSapiNameModPhp()
    {
        Ini::set(Ini::APACHE, 'allow_mod_php', '1');
        Ini::writeAndResetInstance();

        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $this->_setPhpMode(PHP::MODE_MOD_PHP);
        $this->_setOwner('apache');
        $this->_verifyPhpSapi('apache2handler');
    }

    /**
     * Verify that we're running as apache.
     */
    public function testWhoamiModPhp()
    {
        Ini::set(Ini::APACHE, 'allow_mod_php', '1');
        Ini::writeAndResetInstance();

        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $this->_setPhpMode(PHP::MODE_MOD_PHP);
        $this->_setOwner('apache');
        $this->_verifyExpectedUser('apache');
    }

    /**
     * Do a simple hello world test for mod_suphp.
     */
    public function testPhpHelloWorldWorksModSuhp()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $this->_setPhpMode(PHP::MODE_MOD_SUPHP);
        $this->_setOwner($this->getSWAccount()->getUniqname());
        $this->_verifyHelloWorld();
    }

    /**
     * Verify that mod_suphp is in use.
     */
    public function testPhpSapiNameModSuphp()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $this->_setPhpMode(PHP::MODE_MOD_SUPHP);
        $this->_setOwner($this->getSWAccount()->getUniqname());
        $this->_verifyPhpSapi('cgi-fcgi');
    }

    /**
     * Verify that we're running as the siteworx user.
     */
    public function testWhoamiModSuphp()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $this->_setPhpMode(PHP::MODE_MOD_SUPHP);
        $this->_setOwner($this->getSWAccount()->getUniqname());
        $this->_verifyExpectedUser($this->getSWAccount()->getUniqname());
    }

    /**
     * Verify that installing PHP-FPM updates the cache.
     */
    public function testPhpFpmInstalledVersionsCache()
    {
        PHP::install(PHP::SYSTEM_PHP);
        $installed = PHP::getInstalledVersions();
        $this->assertNotEmpty($installed);
    }

    /**
     * Do a simple hello world test for php-fpm.
     */
    public function testPhpHelloWorldWorksPhpFpm()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $this->_setPhpMode(PHP::MODE_PHP_FPM);
        $this->_setOwner($this->getSWAccount()->getUniqname());
        $this->_verifyHelloWorld();
    }

    /**
     * Verify that php-fpm is in use.
     */
    public function testPhpSapiNameFpm()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $this->_setPhpMode(PHP::MODE_PHP_FPM);
        $this->_setOwner($this->getSWAccount()->getUniqname());
        $this->_verifyPhpSapi('fpm-fcgi');
    }

    /**
     * Verify that we're running as the siteworx user.
     */
    public function testWhoamiPhpFpm()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $this->_setPhpMode(PHP::MODE_PHP_FPM);
        $this->_setOwner($this->getSWAccount()->getUniqname());
        $this->_verifyExpectedUser($this->getSWAccount()->getUniqname());
    }

    /**
     * Verify pool removal.
     */
    public function testDeletingAccountRemovesPool()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        $SW   = $this->getSWAccount();
        $pool = $SW->getMasterDomain()->getPhp()->getPoolPath($SW);
        self::deleteSWAccountIfNecessaryStatic();
        $this->assertFileNotExists($pool);
    }

    /**
     * Verify that deleting and re-adding the account still works.
     */
    public function testReaddingAccountStillWorks()
    {
        IW::getLog()->info(__CLASS__ . '::' . __METHOD__);
        self::deleteSWAccountIfNecessaryStatic();
        $this->getSWAccount();
        self::_setUpWebroot();
        $this->_setOwner($this->getSWAccount()->getUniqname());
        $this->_verifyHelloWorld();
    }

    /**
     * Recursively set ownership of the web files for the php-mode test.
     *
     * @param string $owner
     */
    private function _setOwner($owner)
    {
        $SW = $this->getSWAccount();
        IWorxFileSys::chown(
            $owner,
            $SW->getWebRoot(),
            $SW->getUniqname(),
            true
        );

        if ($owner === 'apache') {
            $mode = 0755;
        } else {
            $mode = 0711;
        }

        IWorxFileSys::chmod($mode, $SW->getWebRoot() . '/php-mode-test');
    }

    /**
     * Set the php mode.
     *
     * @param string $mode
     */
    private function _setPhpMode($mode)
    {
        $this->assertReplySuccessful(self::_setPhpModeStatic($mode));
    }

    /**
     * Static method to set PHP mode.
     *
     * @param string $mode
     * @return Reply
     */
    private static function _setPhpModeStatic($mode)
    {
        return IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Http',
            'updatePhpModeCommit',
            ['php_mode' => $mode],
            self::getMasterNodeWorx()
        );
    }

    /**
     * Verify that hello world script works.
     */
    private function _verifyHelloWorld()
    {
        $response = $this->_getUrlContent('hello-world.php');
        $this->assertEquals('Hello, world', trim($response));
    }

    /**
     * Check the php sapi name.
     *
     * @param string $expected
     */
    private function _verifyPhpSapi($expected)
    {
        $actual = trim($this->_getUrlContent('php-sapi-name.php'));
        $this->assertEquals($expected, $actual);
    }

    /**
     * Verify that the script executes as the expected user.
     *
     * @param string $expected
     */
    private function _verifyExpectedUser($expected)
    {
        $actual = trim($this->_getUrlContent('whoami.php'));
        $this->assertEquals($expected, $actual);
    }

    /**
     * Gets the content served through apache.
     *
     * @param string $file
     * @return array
     */
    private function _getUrlContent($file)
    {
        $Domain   = new Domain(self::SW_DOMAIN);
        $response = IWorxHTTP::httpGet(
            $Domain->getIPv4Address(),
            $Domain->getName(),
            "/php-mode-test/{$file}",
            false
        );
        return $response;
    }
}
