<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Service
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 6, 2014
 */

/**
 * Base class for client service tests.
 *
 * @package    InterWorx
 * @subpackage Service
 */
abstract class Service_ClientTestCase extends IWorx_Framework_TestCase
{
    /**
     * Service interface object.
     *
     * @var Service_Interface
     */
    protected $_Service;

    /**
     * Basic test to see if the service responds on the port it uses.
     */
    public function testSocketConnection()
    {
        $Service = $this->_getService();

        foreach ($Service->getPortNumbers() as $port) {
            $fp = @fsockopen('127.0.0.1', $port, $errno, $errstr, 5);

            $msg = "Service {$Service->getGeneralName()} is not responding on :{$port}";

            $this->assertNotEquals(false, $fp, $msg);
        }
    }

    /**
     * Test that the service isRunning function thinks it's running.
     */
    public function testIsRunning()
    {
        $this->assertTrue($this->_getService()->isRunning());
    }

    /**
     * Run service-specific tests to verify running status one more way.
     */
    public function testConfirmIsRunning()
    {
        $Service = $this->_getService();
        $msg     = "Unable to confirm that {$Service->getGeneralName()} is running";
        $this->assertTrue($this->_confirmIsRunning(), $msg);
    }

    /**
     * Extend to confirm running status.
     *
     * @return boolean
     */
    abstract protected function _confirmIsRunning();

    /**
     * Tests that stopping the function returns true.
     *
     * @depends testIsRunning
     */
    public function testStop()
    {
        $this->assertTrue($this->_getService()->stop());
        $this->assertFalse($this->_getService()->isRunning());
    }

    /**
     * Pretty stupid test, checks whether the service is in ps auxf.
     *
     * @depends testStop
     */
    public function testStoppedWithGrepPsauxf()
    {
        $cmd = "ps auxf | grep '{$this->_getGrepProcessString()}' | grep -v grep";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval, IWorxExec::ALLOW_PIPE);

        $this->assertEmpty($result);
    }

    /**
     * Confirms that the service is actually stopped.
     *
     * @depends testStop
     */
    public function testConfirmStopped()
    {
        $Service = $this->_getService();
        $msg     = "Unable to confirm that {$Service->getGeneralName()} is stopped";
        $this->assertTrue($this->_confirmStopped(), $msg);
    }

    /**
     * Confirm that the service is stopped.
     *
     * @return boolean
     */
    abstract protected function _confirmStopped();

    /**
     * Tests that stopping the function returns true.
     *
     * @depends testIsRunning
     * @depends testStop
     */
    public function testStart()
    {
        $this->assertTrue($this->_getService()->start());
        $this->assertTrue($this->_getService()->isRunning());
    }

    /**
     * Pretty stupid test, checks whether the service is in ps auxf.
     *
     * @depends testStart
     */
    public function testStartedWithGrepPsauxf()
    {
        $cmd = "ps auxf | grep '{$this->_getGrepProcessString()}' | grep -v grep";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval, IWorxExec::ALLOW_PIPE);

        $this->assertNotEmpty($result);
    }

    /**
     * Confirms the service was started.
     *
     * @depends testStart
     */
    public function testConfirmStarted()
    {
        $Service = $this->_getService();
        $msg     = "Unable to confirm that {$Service->getGeneralName()} is started";
        $this->assertTrue($this->_confirmStarted(), $msg);
    }

    /**
     * Confirm that the service was restarted.
     *
     * @return boolean
     */
    abstract protected function _confirmStarted();

    /**
     * Sets the service.
     *
     * @param Service_Interface $S
     */
    protected function _setService(Service_Interface $S)
    {
        $this->_Service = $S;
    }

    /**
     * Gets the service object.
     *
     * @return Service_Interface
     */
    protected function _getService()
    {
        return $this->_Service;
    }

    /**
     * Get the string to look for in the ps output.
     *
     * @return string
     */
    abstract protected function _getGrepProcessString();
}
