<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Jan 26, 2021
 */

class Upgrade_Vhost_TemplateTest extends IWorx_Framework_TestCase
{
    const FILE_VHOST_BACKUP = '/tmp/vhost_template_test_backup.conf';

    /**
     * Make sure we're in the right php mode at the start.
     */
    public static function setUpBeforeClass()
    {
        $Nodeworx = new NodeWorx(NodeWorx::MASTER_ID);
        $Nodeworx->getHTTPDriver()->switchPHPMode(PHP::MODE_PHP_FPM);
        if (is_file(self::FILE_VHOST_BACKUP)) {
            unlink(self::FILE_VHOST_BACKUP);
        }
    }

    /**
     * Test the default vhost.
     */
    public function testDefaultVhostIsClear()
    {
        $this->deleteSWAccountIfNecessary();
        $this->addSWAccount();

        $domain = self::SW_DOMAIN;

        $vhost  = Ini::get(Ini::APACHE_DIR, 'conf.d') . "/vhost_{$domain}.conf";

        $this->assertFileExists($vhost);

        copy($vhost, self::FILE_VHOST_BACKUP);

        $data = $this->_getInspectDomain($domain);

        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_OK], $domain);
    }

    /**
     * Copy the clean backup of the vhost into place.
     */
    private function _restoreCleanVhost()
    {
        if (!is_file(self::FILE_VHOST_BACKUP)) {
            $this->fail('Clean vhost file could not be found!');
        }

        $domain = self::SW_DOMAIN;
        $vhost  = Ini::get(Ini::APACHE_DIR, 'conf.d') . "/vhost_{$domain}.conf";

        if (is_file($vhost)) {
            unlink($vhost);
        }

        copy(self::FILE_VHOST_BACKUP, $vhost);
    }

    /**
     * Test an expected failure.
     */
    public function testCustomizedVhostFails()
    {
        $this->_restoreCleanVhost();
        $domain = self::SW_DOMAIN;

        $this->_customizeVhost($domain, '/RewriteEngine On/', 'RewriteEngine Off');

        $data = $this->_getInspectDomain($domain);

        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_CUSTOM_SINGLE], $domain);
        $this->assertTrue(isset($data['diffs'][$domain]));
    }

    /**
     * Test that a vhost with a custom domain passes inspection.
     */
    public function testCustomDocrootDetection()
    {
        $domain = self::SW_DOMAIN;

        $this->_restoreCleanVhost();

        if (!is_dir('/tmp/docroot')) {
            mkdir('/tmp/docroot');
        }

        $this->_customizeVhost($domain, '#DocumentRoot .*#', 'DocumentRoot /tmp/docroot');

        $data = $this->_getInspectDomain($domain);

        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_OK], $domain);
        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_CUSTOM_DOCROOT], $domain);
    }

    /**
     * Test that inspection found the custom docroot.
     *
     * @depends testCustomDocrootDetection
     */
    public function testCustomDocrootExtraction()
    {
        $extractions = $this->_getExtractions(self::SW_DOMAIN);
        $this->assertEquals('/tmp/docroot', $extractions['custom_docroot'][self::SW_DOMAIN]);
    }

    /**
     * Test that the custom docroot is migrated.
     *
     * @depends testCustomDocrootExtraction
     */
    public function testCustomDocrootMigration()
    {
        $domain = self::SW_DOMAIN;
        $this->_doMigration($domain);

        $Domain = new Domain($domain);

        $this->assertEquals('/tmp/docroot', $Domain->getConfig('HTTPD_DOCUMENT_ROOT'));
    }

    /**
     * Trigger the code to import to the config system.
     *
     * @param string $domain
     */
    private function _doMigration($domain)
    {
        $cmd = "/home/interworx/bin/template-adopt.pex --migrate-to-config --domain {$domain}";
        $Cmd = new Command($cmd);
        $Cmd->exec();

        if (!$Cmd->wasSuccessful()) {
            IWorxConsole::echoLine($Cmd->getResultMessage(), '%r');
            IWorxConsole::echoLine($Cmd->getOutput());
            $this->fail('Migration failed');
        }
    }

    /**
     * Test that a custom ip:port will pass inspection
     */
    public function testExtraIpPort()
    {
        $domain = self::SW_DOMAIN;

        $this->_restoreCleanVhost();
        $this->_customizeVhost($domain, '/(<VirtualHost .*)>/', '$1 1.1.1.1:80>');
        $data = $this->_getInspectDomain($domain);

        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_OK], $domain);
        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_EXTRA_IPPORT], $domain);
    }

    /**
     * Test that inspection found the custom ip.
     *
     * @depends testExtraIpPort
     */
    public function testExtraIpsExtraction()
    {
        $extractions = $this->_getExtractions(self::SW_DOMAIN);
        $this->assertEquals('1.1.1.1:{$PORT_HTTP}', $extractions['extra_http_ips'][self::SW_DOMAIN][0]);
    }

    /**
     * Test that the extra IP is migrated.
     *
     * @depends testExtraIpsExtraction
     */
    public function testExtraIpsMigration()
    {
        $domain = self::SW_DOMAIN;
        $this->_doMigration($domain);

        $Domain = new Domain($domain);

        $this->assertEquals('1.1.1.1:{$PORT_HTTP}', $Domain->getConfig('HTTPD_EXTRA_IPS'));
    }

    /**
     * Test detection of extra ports.
     */
    public function testExtraPort()
    {
        $domain = self::SW_DOMAIN;

        $Domain = new Domain($domain);
        $ip     = $Domain->getIPv4Address();

        $this->_restoreCleanVhost();
        $this->_customizeVhost($domain, '/(<VirtualHost .*)>/', "\$1 {$ip}:81>");
        $data = $this->_getInspectDomain($domain);

        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_OK], $domain);
        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_EXTRA_IPPORT], $domain);
    }

    /**
     * Test that inspection found the custom port.
     *
     * @depends testExtraPort
     */
    public function testExtraPortExtraction()
    {
        $domain = self::SW_DOMAIN;

        $extractions = $this->_getExtractions($domain);
        $this->assertEquals('{$IP_ADDR}:81', $extractions['extra_http_ips'][$domain][0]);
    }

    /**
     * Test that a custom alias will pass inspection
     */
    public function testExtraAlias()
    {
        $domain = self::SW_DOMAIN;

        $this->_restoreCleanVhost();
        $this->_customizeVhost($domain, '/(ServerAlias .*)/', '$1 extra.aliases.com');

        $data = $this->_getInspectDomain($domain);

        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_OK], $domain);
        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_ALIASES], $domain);
    }

    /**
     * Test that inspection found the custom ip.
     *
     * @depends testExtraIpPort
     */
    public function testExtraAliasesExtraction()
    {
        $extractions = $this->_getExtractions(self::SW_DOMAIN);
        $this->assertEquals('extra.aliases.com', $extractions['extra_aliases'][self::SW_DOMAIN][0]);
    }

    /**
     * Test that the extra alias is migrated.
     *
     * @depends testExtraAliasesExtraction
     */
    public function testExtraAliasesMigration()
    {
        $domain = self::SW_DOMAIN;
        $this->_doMigration($domain);

        $Domain = new Domain($domain);

        $this->assertEquals('extra.aliases.com', $Domain->getConfig('HTTPD_EXTRA_SERVER_ALIAS'));
    }

    /**
     * Test that the adoption works when system php is in suphp mode.
     */
    public function testPhpModeSuphp()
    {
        $this->_restoreCleanVhost();
        $domain = self::SW_DOMAIN;

        $SW = new SiteWorx($domain);
        $SW->getHTTPDriver()->switchPhpMode(PHP::MODE_MOD_SUPHP);

        $data = $this->_getInspectDomain($domain);

        $SW->getHTTPDriver()->switchPhpMode(PHP::MODE_PHP_FPM);

        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_OK], $domain);
    }

    /**
     * JPurkis found this example on a customer server during a manual upgrade.
     */
    public function testPhpModeSuphpAlternateTemplate()
    {
        $this->_restoreCleanVhost();

        $domain = self::SW_DOMAIN;
        $SW = new SiteWorx($domain);
        $SW->getHTTPDriver()->switchPhpMode(PHP::MODE_MOD_SUPHP);

        $regex  = CLI_TemplateAdopter::REGEX_PHP_SECTION;
        $replace = '# php: default  don\'t edit between this and the "end php" comment below
  # iw-php-mode: system-mod-suphp
  <IfModule mod_suphp.c>
    <FilesMatch \.php$>
      AddHandler x-httpd-php .php
      suPHP_AddHandler x-httpd-php .php
    </FilesMatch>
    suPHP_Engine On
    suPHP_UserGroup addedbyt addedbyt
    suPHP_ConfigPath /home/addedbyt/etc
  </IfModule>

  # end php';

        $this->_customizeVhost($domain, $regex, $replace);
        $data = $this->_getInspectDomain($domain);
        $SW->getHTTPDriver()->switchPhpMode(PHP::MODE_PHP_FPM);

        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_OK], $domain);
    }

    /**
     * Test that disabled docroots don't break the import.
     */
    public function testDisabledDocroot()
    {
        $this->_restoreCleanVhost();
        $domain = self::SW_DOMAIN;

        $Reply = IW::FC()->routeFromPHP(
            Ctrl_Nodeworx_Siteworx::class,
            'suspendCommit',
            ['domain' => $domain],
            $this->getMasterNodeWorx()
        );

        $this->assertReplySuccessful($Reply);

        $data = $this->_getInspectDomain($domain);

        $Reply = IW::FC()->routeFromPHP(
            Ctrl_Nodeworx_Siteworx::class,
            'unsuspendCommit',
            ['domain' => $domain],
            $this->getMasterNodeWorx()
        );

        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_OK], $domain);
        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_DISABLED], $domain);

        // At one point, the disabled docroot was being seen as custom
        $this->assertFalse(isset($data['domains'][CLI_TemplateAdopter::STATE_CUSTOM_DOCROOT]));
    }

    /**
     * Test that the mod_security blocks are not being allowed.
     */
    public function testModSecurityFails()
    {
        $this->_restoreCleanVhost();
        $domain = self::SW_DOMAIN;
        $this->_customizeVhost($domain, '/(# end cgi)/', "\$1\n   <IfModule mod_security2.c>
      #Ticket ID: ASXU-3314
      <LocationMatch /administrator/index.php>
       SecRuleRemoveById 960915
       SecRuleRemoveById 950005
     </LocationMatch>
    </IfModule>");

        $data = $this->_getInspectDomain($domain);

        $this->assertFalse(isset($data['domains'][CLI_TemplateAdopter::STATE_OK]));

        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_CUSTOM_SINGLE], $domain);
    }

    /**
     * Test that the Nexcess abuse blocks are not being permitted.
     */
    public function testNexcessAbuseFails()
    {
        $this->_restoreCleanVhost();
        $domain = self::SW_DOMAIN;
        touch('/tmp/test-abuse.block');
        $this->_customizeVhost($domain, '/^(<VirtualHost.*)/', "\$1\nInclude /tmp/test-abuse.block");

        $data = $this->_getInspectDomain($domain);

        $this->assertFalse(isset($data['domains'][CLI_TemplateAdopter::STATE_OK]));

        $this->assertArrayContains($data['domains'][CLI_TemplateAdopter::STATE_CUSTOM_SINGLE], $domain);
    }

    /**
     * Customize the vhost via regex.
     *
     * @param string $domain
     * @param string $regex
     * @param string $replace
     */
    private function _customizeVhost($domain, $regex, $replace)
    {
        $vhost   = $this->_getVhostPath($domain);

        $this->assertFileExists($vhost);

        $content = file_get_contents($vhost);
        $content = preg_replace(
            $regex,
            $replace,
            $content
        );
        file_put_contents($vhost, $content);
    }

    /**
     * Get the vhost file path.
     *
     * @param string $domain
     * @return string
     */
    private function _getVhostPath($domain)
    {
        return Ini::get(Ini::APACHE_DIR, 'conf.d') . "/vhost_{$domain}.conf";
    }

    /**
     * Inspect a domain.
     *
     * @param string $domain
     * @return array
     */
    private function _getInspectDomain($domain)
    {
        return Payload_Factory_NW_Vhost::getTemplateInspect($domain);
    }

    /**
     * Get the extractions from the inspection tool.
     *
     * @param string $domain
     * @return array
     */
    private function _getExtractions($domain)
    {
        return Payload_Factory_NW_Vhost::getTemplateExtractions($domain);
    }
}
