<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2013 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       May 22, 2013
 */

/**
 * Tests the upgrade step classes.
 *
 * @package    InterWorx
 * @subpackage Upgrade
 */
class UpgradeTest extends IWorx_Framework_TestCase
{
    /**
     * Clears any existing state.
     */
    public function setup()
    {
        $cmd = 'rm -f /usr/local/interworx/etc/.upgrade/*/2731';
        exec($cmd);

        IW::Env()->clearEnvFlag('test-2731-ran');
        IW::Env()->clearEnvFlag('test-2731-should-fail');

        // Disable verbosity, it's annoying me.
        IWorxConsole::$is_verbose = false;
    }

    /**
     * Get the base command.
     *
     * @return string
     */
    protected function _cmd()
    {
        return Ini::get(Ini::IWORX_BIN, 'iworx-upgrade');
    }

    /**
     * Touches a state file.
     *
     * @param string $flag
     */
    protected function _touchFile($flag)
    {
        $file = "/home/interworx/etc/.upgrade/{$flag}/2731";
        touch($file);
        IWorxFileSys::chown('iworx', $file, 'iworx');
    }

    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderFlagCodesCLI()
    {
        return
        array(
            array( Upgrade_Abstract::FLAG_COMPLETE,
                   CLI_IWorxUpgrade::STATUS_COMPLETE ),
            array( Upgrade_Abstract::FLAG_PRE_INSTALLED,
                   CLI_IWorxUpgrade::STATUS_COMPLETE ),
            array( Upgrade_Abstract::FLAG_FAILURE,
                   CLI_IWorxUpgrade::STATUS_FAILED ),
            array( Upgrade_Abstract::FLAG_LATER,
                   CLI_IWorxUpgrade::STATUS_LATER ),
            array( Upgrade_Abstract::FLAG_IN_PROGRESS,
                   CLI_IWorxUpgrade::STATUS_IN_PROGRESS ),
            );
    }

    /**
     * Tests that the initial state is "not run".
     */
    public function testHasntRunYet()
    {
        $cmd = $this->_cmd() . ' --status --step 2731';

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(CLI_IWorxUpgrade::STATUS_NOT_RUN, $retval);
    }

    /**
     * Tests CLI retvals of various states.
     *
     * @param string  $flag
     * @param integer $expected_retval
     * @dataProvider dataProviderFlagCodesCLI
     */
    public function testCLIResponseCodes($flag, $expected_retval)
    {
        $this->_touchFile($flag);

        $cmd = $this->_cmd() . ' --status --step 2731';

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals($expected_retval, $retval);
    }

    /**
     * Checks that the state is false before anything has happened.
     */
    public function testPreInstalledFalse()
    {
        $Step = Upgrade_Abstract::factoryFromStepNum(2731);
        $this->assertFalse($Step->isPreInstalled());
    }

    /**
     * Checks that the state is true once flagged.
     */
    public function testPreInstalled()
    {
        $this->_touchFile(Upgrade_Abstract::FLAG_PRE_INSTALLED);

        $Step = Upgrade_Abstract::factoryFromStepNum(2731);

        $this->assertTrue($Step->isPreInstalled());
    }

    /**
     * Checks that the state is true once flagged.
     */
    public function testPreInstalledComplete()
    {
        $this->_touchFile(Upgrade_Abstract::FLAG_PRE_INSTALLED);

        $Step = Upgrade_Abstract::factoryFromStepNum(2731);

        $this->assertTrue($Step->isComplete());
    }

    /**
     * Checks that the state is false before anything has happened.
     */
    public function testCompleteFalse()
    {
        $Step = Upgrade_Abstract::factoryFromStepNum(2731);
        $this->assertFalse($Step->isComplete());
    }

    /**
     * Checks that the state is true once flagged.
     */
    public function testComplete()
    {
        $this->_touchFile(Upgrade_Abstract::FLAG_COMPLETE);

        $Step = Upgrade_Abstract::factoryFromStepNum(2731);

        $this->assertTrue($Step->isComplete());
    }

    /**
     * Checks that the state is false before anything has happened.
     */
    public function testRunningFalse()
    {
        $Step = Upgrade_Abstract::factoryFromStepNum(2731);
        $this->assertFalse($Step->isRunning());
    }

    /**
     * Checks that the state is true once flagged.
     */
    public function testRunning()
    {
        $this->_touchFile(Upgrade_Abstract::FLAG_IN_PROGRESS);

        $Step = Upgrade_Abstract::factoryFromStepNum(2731);

        $this->assertTrue($Step->isRunning());
    }

    /**
     * Checks that the state is false before anything has happened.
     */
    public function testPendingFalse()
    {
        $Step = Upgrade_Abstract::factoryFromStepNum(2731);
        $this->assertFalse($Step->isPending());
    }

    /**
     * Checks that the state is true once flagged.
     */
    public function testPending()
    {
        $this->_touchFile(Upgrade_Abstract::FLAG_LATER);

        $Step = Upgrade_Abstract::factoryFromStepNum(2731);

        $this->assertTrue($Step->isPending());
    }

    /**
     * Checks that the state is false before anything has happened.
     */
    public function testFailedFalse()
    {
        $Step = Upgrade_Abstract::factoryFromStepNum(2731);
        $this->assertFalse($Step->isFailed());
    }

    /**
     * Checks that the state is true once flagged.
     */
    public function testFailed()
    {
        $this->_touchFile(Upgrade_Abstract::FLAG_FAILURE);

        $Step = Upgrade_Abstract::factoryFromStepNum(2731);

        $this->assertTrue($Step->isFailed());
    }

    /**
     * Runs the step and verifies it's marked complete.
     */
    public function testRunningSuccess()
    {
        $Step = Upgrade_Abstract::factoryFromStepNum(2731);
        $Step->run();

        $this->assertTrue($Step->isComplete());
        $this->assertTrue(IW::Env()->getEnvFlag('test-2731-ran'));
    }

    /**
     * Tests that re-running the upgrade doesn't do anything without force.
     */
    public function testRunningTwiceExitsWithoutDoingAnything()
    {
        $Step = Upgrade_Abstract::factoryFromStepNum(2731);
        $Step->run();

        $this->assertTrue($Step->isComplete());
        $this->assertTrue(IW::Env()->getEnvFlag('test-2731-ran'));

        IW::Env()->clearEnvFlag('test-2731-ran');

        $Step = Upgrade_Abstract::factoryFromStepNum(2731);
        $Step->run();

        $this->assertTrue($Step->isComplete());
        $this->assertFalse(IW::Env()->getEnvFlag('test-2731-ran'));
    }

    /**
     * Tests that re-running the upgrade with force does it.
     */
    public function testRunningTwiceWithForce()
    {
        $Step = Upgrade_Abstract::factoryFromStepNum(2731);
        $Step->run();

        $this->assertTrue($Step->isComplete());
        $this->assertTrue(IW::Env()->getEnvFlag('test-2731-ran'));

        IW::Env()->clearEnvFlag('test-2731-ran');

        $Step = Upgrade_Abstract::factoryFromStepNum(2731, Upgrade_Abstract::FORCE);
        $Step->run();

        $this->assertTrue($Step->isComplete());
        $this->assertTrue(IW::Env()->getEnvFlag('test-2731-ran'));
    }

    /**
     * Runs the step triggers a failure.
     */
    public function testRunningFailure()
    {
        $Step = Upgrade_Abstract::factoryFromStepNum(2731);

        IW::Env()->setEnvFlag('test-2731-should-fail');

        $retval = $Step->run();

        $this->assertEquals(Upgrade_2731::ERROR_SHOULD_FAIL, $retval);
        $this->assertTrue($Step->isFailed());
        $this->assertFalse(IW::Env()->getEnvFlag('test-2731-ran'));
    }

    /**
     * Tests that failing allows it to re-run.
     */
    public function testRunningFailureAgain()
    {
        $Step = Upgrade_Abstract::factoryFromStepNum(2731);

        IW::Env()->setEnvFlag('test-2731-should-fail');

        $retval = $Step->run();

        $this->assertEquals(Upgrade_2731::ERROR_SHOULD_FAIL, $retval);

        $Step = Upgrade_Abstract::factoryFromStepNum(2731);
        IW::Env()->clearEnvFlag('test-2731-should-fail');

        $Step->run();

        $this->assertTrue($Step->isComplete());
    }

    /**
     * Tests that unknown bug numbers are crap.
     */
    public function testUnknownBugNumber()
    {
        $Step = Upgrade_Abstract::factoryFromStepNum(2);

        $this->assertFalse($Step);
    }

    /**
     * Tests that unknown bug numbers are crap.
     */
    public function testUnknownBugNumberCLI()
    {
        $cmd = "{$this->_cmd()} --run --step 2";

        $result = [];
        $retval = -1;

        IWorxExec::exec($cmd, $result, $retval);

        $this->assertEquals(CLI_IWorxUpgrade::ERROR_UNKNOWN_STEP, $retval);
    }
}
