<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2019 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Sep 09, 2019
 */

/**
 * Class CacheTest
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class CacheTest extends IWorx_Framework_TestCase
{
    /**
     * Verify that the callback will be invoked when appropriate.
     */
    public function testCallbackIsCalledWithCacheMiss()
    {
        $is_called = false;
        $key       = __FUNCTION__;

        IW::Cache()->get(
            $key,
            function () use (&$is_called) {
                $is_called = true;
                return new \IWorx\Cache\Cachable(1, IW::Cache()::TTL_PROCESS);
            }
        );
        IW::Cache()->delete($key);
        $this->assertTrue($is_called);
    }

    /**
     * Test that the callback is not invoked when the cached value is present.
     */
    public function testCallbackIsNotCalledWithCacheHit()
    {
        $is_called = false;
        $key = __FUNCTION__;

        IW::Cache()->set($key, 1);
        IW::Cache()->get(
            $key,
            function () use (&$is_called) {
                $is_called = true;
                return new \IWorx\Cache\Cachable(1, IW::Cache()::TTL_PROCESS);
            }
        );
        IW::Cache()->delete($key);
        $this->assertFalse($is_called);
    }

    /**
     * Test that an expired ttl invokes the callback.
     */
    public function testCallbackIsCalledWhenCacheTtlExpired()
    {
        $is_called = false;
        $key       = __FUNCTION__;

        // One second ttl
        IW::Cache()->set($key, 1, 1);

        sleep(2);

        IW::Cache()->get(
            $key,
            function () use (&$is_called) {
                $is_called = true;
                return new \IWorx\Cache\Cachable(1, IW::Cache()::TTL_PROCESS);
            }
        );
        IW::Cache()->delete($key);
        $this->assertTrue($is_called);
    }

    /**
     * Test that setting a new value is represented in calls to get.
     */
    public function testSettingNewCacheValue()
    {
        $key = __FUNCTION__;

        IW::Cache()->set($key, 'value');
        $this->assertEquals('value', IW::Cache()->get($key));

        IW::Cache()->set($key, 'value2');
        $this->assertEquals('value2', IW::Cache()->get($key));
    }

    /**
     * Test that once a session cache value is set, the global is not returned.
     */
    public function testSessionCacheOverride()
    {
        $key = __FUNCTION__;

        // One second ttl
        IW::Cache()->set($key, 1, 30);
        IW::Cache()->set($key, 2, IW::Cache()::TTL_SESSION);

        $this->assertEquals(2, IW::Cache()->get($key));

        IW::Cache()->delete($key);
        IW::Cache()->deleteForSession($key);
    }

    /**
     * Test that once a request cache value is set, the global is not returned.
     */
    public function testRequestCacheOverride()
    {
        $key = __FUNCTION__;

        // One second ttl
        IW::Cache()->set($key, 1, 30);
        IW::Cache()->set($key, 2, IW::Cache()::TTL_REQUEST);

        $this->assertEquals(2, IW::Cache()->get($key));

        IW::Cache()->delete($key);
        IW::Cache()->deleteForRequest($key);
    }

    /**
     * Test that once a process cache value is set, the global is not returned.
     */
    public function testProcessCacheOverride()
    {
        $key = __FUNCTION__;

        // One second ttl
        IW::Cache()->set($key, 1, 30);
        IW::Cache()->set($key, 2, IW::Cache()::TTL_PROCESS);

        $this->assertEquals(2, IW::Cache()->get($key));

        IW::Cache()->delete($key);
        IW::Cache()->deleteForProcess($key);
    }

    /**
     * Test clearing a set process cache value.
     */
    public function testProcClearing()
    {
        $key = __FUNCTION__;

        IW::Cache()->set($key, 1, IW::Cache()::TTL_PROCESS);

        $this->assertEquals(1, IW::Cache()->get($key));

        IW::Cache()->deleteForProcess($key);

        $this->assertNull(IW::Cache()->get($key));
    }

    /**
     * Test clearing a set request cache value.
     */
    public function testRequestClearing()
    {
        $key = __FUNCTION__;

        IW::Cache()->set($key, 1, IW::Cache()::TTL_REQUEST);

        $this->assertEquals(1, IW::Cache()->get($key));

        IW::Cache()->deleteForRequest($key);

        $this->assertNull(IW::Cache()->get($key));
    }

    /**
     * Test clearing a set session cache value.
     */
    public function testSessionClearing()
    {
        $key = __FUNCTION__;

        IW::Cache()->set($key, 1, IW::Cache()::TTL_SESSION);

        $this->assertEquals(1, IW::Cache()->get($key));

        IW::Cache()->deleteForSession($key);

        $this->assertNull(IW::Cache()->get($key));
    }

    /**
     * Test clearing all process cache values.
     */
    public function testClearingAllProcess()
    {
        $key = __FUNCTION__;

        IW::Cache()->set($key, 1, IW::Cache()::TTL_PROCESS);

        $this->assertEquals(1, IW::Cache()->get($key));

        IW::Cache()->deleteAllProcess();

        $this->assertNull(IW::Cache()->get($key));
    }

    /**
     * Test clearing all request cache values.
     */
    public function testClearingAllRequest()
    {
        $key = __FUNCTION__;

        IW::Cache()->set($key, 1, IW::Cache()::TTL_REQUEST);

        $this->assertEquals(1, IW::Cache()->get($key));

        IW::Cache()->deleteAllRequest();

        $this->assertNull(IW::Cache()->get($key));
    }

    /**
     * Test clearing all session cache values.
     */
    public function testClearingAllSession()
    {
        $key = __FUNCTION__;

        IW::Cache()->set($key, 1, IW::Cache()::TTL_SESSION);

        $this->assertEquals(1, IW::Cache()->get($key));

        IW::Cache()->deleteAllSession();

        $this->assertNull(IW::Cache()->get($key));
    }
}
