<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Tim College <tcollege@interworx.com>
 * @date       Apr 17, 2014
 */

/**
 * Tests for the command class.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class CommandTest extends IWorx_Framework_TestCase
{
    /**
     * A sample shell command used for testing.
     *
     * @return string
     */
    private function _getValidCommand()
    {
        return '/bin/echo TEST';
    }

    /**
     * Sleep shell command used for testing.
     *
     * @param integer $seconds
     * @return string
     */
    private function _getSleepCommand($seconds)
    {
        return "/bin/sleep $seconds";
    }

    /**
     * A sample shell command that does not exist.
     *
     * @return string
     */
    private function _getInValidCommand()
    {
        return '/this/does/not/exist/bin/echo TEST';
    }

    /**
     * The valid output of the sample valid command.
     *
     * @return array
     */
    private function _getValidCommandResult()
    {
        return array( 'TEST' );
    }

    /**
     * Data provider of data and appropriately escaped output.
     *
     * @return array
     */
    public static function dataProviderEscapeData()
    {
        return( array(
                  array( '/bin/echo TEST',
                         '/bin/echo TEST' ),
                  array( '`',  // escape backtick
                         '\`' ),
                  array( '>', // escape bracket
                         '\>' ),
                  array( '<', // escape bracket
                         '\<' ),
                  array( '|', // escape pipe
                         '\|' ),
                  array( '&', // escape ampersand
                         '\&' ),
                  array( '\\', // escape backslash
                         '\\\\' ),
                  array( '[', // escape left bracket
                         '\[' ),
                  array( '^', // escape caret
                         '\^' ),
                  array( ']', // escape right bracket
                         '\]' ),
                  array( '{', // escape left curly
                         '\{' ),
                  array( '}', // escape right curly
                         '\}' ),
                  array( '#', // escape pound
                         '\#' ),
                  array( '(', // escape left paren
                         '\(' ),
                  array( ')', // escape right paren
                         '\)' ),
                  array( '$', // escape dollar
                         '\$' ),
                  )
            );
    }

    /**
     * Tests escape function.
     *
     * @param string $unescaped
     * @param string $expected
     * @dataProvider dataProviderEscapeData()
     */
    public function testEscapeshellcmd($unescaped, $expected)
    {
        $Cmd = new Command();

        $this->assertEquals($expected, $Cmd->escape($unescaped));
    }

    /**
     * Test exception for >.
     */
    public function testRedirOutException()
    {
        $Cmd = new Command();
        $Cmd->allowRedirectOut();

        $this->assertEquals('>', $Cmd->escape('>'));
    }

    /**
     * Test exception for <.
     */
    public function testRedirInException()
    {
        $Cmd = new Command();
        $Cmd->allowRedirectIn();

        $this->assertEquals('<', $Cmd->escape('<'));
    }

    /**
     * Test exception for *.
     */
    public function testAsteriskException()
    {
        $Cmd = new Command();
        $Cmd->allowAsterisk();

        $this->assertEquals('*', $Cmd->escape('*'));
    }

    /**
     * Test exception for |.
     */
    public function testPipeException()
    {
        $Cmd = new Command();
        $Cmd->allowPipe();

        $this->assertEquals('|', $Cmd->escape('|'));
    }

    /**
     * Test exception for &.
     */
    public function testAmpersandException()
    {
        $Cmd = new Command();
        $Cmd->allowAmpersand();

        $this->assertEquals('&', $Cmd->escape('&'));
    }

    /**
     * Test exception for [.
     */
    public function testBracketLeftException()
    {
        $Cmd = new Command();
        $Cmd->allowBracketLeft();

        $this->assertEquals('[', $Cmd->escape('['));
    }

    /**
     * Test exception for ].
     */
    public function testBracketRightException()
    {
        $Cmd = new Command();
        $Cmd->allowBracketRight();

        $this->assertEquals(']', $Cmd->escape(']'));
    }

    /**
     * Test exception for {.
     */
    public function testCurlyLeftException()
    {
        $Cmd = new Command();
        $Cmd->allowCurlyLeft();

        $this->assertEquals('{', $Cmd->escape('{'));
    }

    /**
     * Test exception for }.
     */
    public function testCurlyRightException()
    {
        $Cmd = new Command();
        $Cmd->allowCurlyRight();

        $this->assertEquals('}', $Cmd->escape('}'));
    }

    /**
     * Test exception for #.
     */
    public function testPoundException()
    {
        $Cmd = new Command();
        $Cmd->allowPound();

        $this->assertEquals('#', $Cmd->escape('#'));
    }

    /**
     * Try a legit command.
     */
    public function testExecReturnsTrueForValidCommand()
    {
        $Cmd = new Command();
        $Cmd->setCmd($this->_getValidCommand());

        $this->assertTrue($Cmd->exec());
    }

    /**
     * Test exception for $.
     */
    public function testDollarSignException()
    {
        $Cmd = new Command();
        $Cmd->allowDollarSign();

        $this->assertEquals('$', $Cmd->escape('$'));
    }

    /**
     * Test exception for ^.
     */
    public function testCaretException()
    {
        $Cmd = new Command();
        $Cmd->allowCaret();

        $this->assertEquals('^', $Cmd->escape('^'));
    }

    /**
     * Test exception for (.
     */
    public function testParenthesisLeftException()
    {
        $Cmd = new Command();
        $Cmd->allowParenthesisLeft();

        $this->assertEquals('(', $Cmd->escape('('));
    }

    /**
     * Test exception for ).
     */
    public function testParenthesisRightException()
    {
        $Cmd = new Command();
        $Cmd->allowParenthesisRight();

        $this->assertEquals(')', $Cmd->escape(')'));
    }

    /**
     * Test exception for ;.
     */
    public function testSemicolonException()
    {
        $Cmd = new Command();
        $Cmd->allowSemicolon();

        $this->assertEquals(';', $Cmd->escape(';'));
    }

    /**
     * Test exception for :.
     */
    public function testColonException()
    {
        $Cmd = new Command();
        $Cmd->allowColon();

        $this->assertEquals(':', $Cmd->escape(':'));
    }

    /**
     * Test the output of a valid command.
     */
    public function testValidCommandOutput()
    {
        $Cmd = new Command();
        $Cmd->setCmd($this->_getValidCommand());
        $Cmd->exec();

        $this->assertEquals($this->_getValidCommandResult(), $Cmd->getOutput());
    }

    /**
     * Try a bad command.
     */
    public function testExecReturnsFalseForBadCommand()
    {
        $Cmd = new Command();
        $Cmd->noOutput(); // Suppresses a stderr that pops up in test output.
        $Cmd->setCmd($this->_getInValidCommand());

        $this->assertFalse($Cmd->exec());
    }

    /**
     * Test touching a file.
     */
    public function testTouch()
    {
        $Cmd = new Command();
        $Cmd->setCmd('touch /tmp/testTouch')->exec();

        $stats = stat('/tmp/testTouch');

        IWorxFileSys::rm('/tmp/testTouch');

        $this->assertEquals(posix_getuid(), $stats['uid']);
    }

    /**
     * Test touching a file as the iworx user.
     */
    public function testTouchAsIworx()
    {
        $file = '/tmp/testTouch2';

        $Cmd = new Command();
        $Cmd->setUser('iworx');
        $Cmd->setCmd("touch {$file}")->exec();

        $stats = stat($file);

        $this->assertFileExists($file);
        IWorxFileSys::rm($file);

        $this->assertEquals(ShellAccount::getUID('iworx'), $stats['uid']);
    }

    /**
     * Test that an exception is thrown if you forget to set the command to exec.
     *
     * @expectedException IWorx_Exception_ExecNoCommand
     */
    public function testExceptionWhenMissingCommand()
    {
        $Cmd = new Command();
        $Cmd->exec();
    }

    /**
     * Test that reusing the Command object fails by default.
     *
     * @expectedException IWorx_Exception_ExecMultiple
     */
    public function testMultipleExecuteFails()
    {
        $Cmd = new Command();
        $Cmd->setCmd($this->_getValidCommand())->exec();
        $Cmd->exec();
    }

    /**
     * Test that reusing the Command object works.
     */
    public function testMultipleExecuteWorks()
    {
        $Cmd = new Command();
        $Cmd->allowMultipleExecution();
        $Cmd->setCmd($this->_getValidCommand())->exec();
        $Cmd->exec();

        // If we get here sans exception, we're good.
        $this->assertTrue(true);
    }

    /**
     * Test output using the command multiple times.
     */
    public function testMultipleExecuteOutput()
    {
        $Cmd = new Command();
        $Cmd->allowMultipleExecution();
        $Cmd->setCmd($this->_getValidCommand())->exec();
        $Cmd->exec();

        $this->assertEquals(['TEST'], $Cmd->getOutput());
    }

    /**
     * Make sure escaping and exceptions work even when running as another user.
     */
    public function testEscapingAsIworx()
    {
        $Cmd = new Command();
        $Cmd->setUser('iworx');
        $Cmd->allowPipe();

        $Cmd->setCmd('echo "boo boo" | wc -w')->exec();

        $this->assertEquals(array( '2' ), $Cmd->getOutput());
    }

    /**
     * Verify that a character is escaped by default.
     */
    public function testEscaping()
    {
        $Cmd = new Command();
        $cmd = 'echo "boo boo" | wc -w';
        $Cmd->setCmd($cmd);

        $expected = str_replace('|', '\|', $cmd);

        $this->assertEquals($expected, $Cmd->getScrubbedCmd());
    }

    /**
     * Verify that allowing the character works.
     */
    public function testEscapingAllowed()
    {
        $Cmd = new Command();
        $cmd = 'echo "boo boo" | wc -w';
        $Cmd->setCmd($cmd)
            ->allowPipe();

        $this->assertEquals($cmd, $Cmd->getScrubbedCmd());
    }

    /**
     * Test simple piping.
     */
    public function testPipedCommandNoWeirdChars()
    {
        $One = new Command('one');
        $Two = new Command('two');

        $One->pipe($Two);

        $this->assertTrue($One->isPiped());
        $this->assertEquals('one | two', $One->getScrubbedCmd());
    }

    /**
     * Test simple piping.
     */
    public function testDoublePipedCommandNoWeirdChars()
    {
        $One   = new Command('one');
        $Two   = new Command('two');
        $Three = new Command('three');

        $One->pipe($Two->pipe($Three));

        $this->assertTrue($One->isPiped());
        $this->assertTrue($Two->isPiped());
        $this->assertEquals('one | two | three', $One->getScrubbedCmd());
    }

    /**
     * Test that each command is escaped per its own allowances.
     */
    public function testPipingWithDifferentEscaping()
    {
        $One = new Command('one &');
        $Two = new Command('two &');
        $Two->allowAmpersand();

        $One->pipe($Two);

        $this->assertEquals('one \& | two &', $One->getScrubbedCmd());
    }

    /**
     * Test that piped commands work.
     */
    public function testPipedCommandResults()
    {
        $One = new Command('echo "boo boo"');
        $One->allowQuoteDouble();

        $Two = new Command('wc -w');

        $One->pipe($Two)->exec();

        $this->assertCommandSuccessful($One);
        $this->assertEquals('2', $One->getOutput()[0]);
    }

    /**
     * Test that piped commands still escape non-allowed characters.
     */
    public function testPipedCommandResultsWithWeirdChars()
    {
        $One = new Command('echo $(ls)');
        $Two = new Command('cat');

        $One->pipe($Two)->exec();

        $this->assertCommandSuccessful($One);
        $this->assertEquals('$(ls)', $One->getOutput()[0]);
    }

    /**
     * Test that piped commands allow characters.
     */
    public function testPipedCommandResultsWithWeirdCharsAllowed()
    {
        $One = new Command('echo $(ls)');
        $One->allowDollarSign()
            ->allowParenthesisBoth();
        $Two = new Command('cat');

        $One->pipe($Two)->exec();

        $this->assertCommandSuccessful($One);
        $this->assertNotEquals('$(ls)', $One->getOutput()[0]);
    }

    /**
     * Test a simple escalation to another user.
     */
    public function testSingleCommandResultAsIworx()
    {
        $Cmd = new Command('ls /tmp');
        $Cmd->setUser('iworx')->exec();

        $this->assertCommandSuccessful($Cmd);
    }

    /**
     * Test that piped commands work.
     */
    public function testPipedCommandResultsAsIworx()
    {
        $One = new Command('echo "boo boo"');
        $One->allowQuoteDouble();
        $One->setUser('iworx');

        $Two = new Command('wc -w');

        $One->pipe($Two)->exec();

        $this->assertCommandSuccessful($One);
        $this->assertEquals('2', $One->getOutput()[0]);
    }

    /**
     * Test that piped commands still escape non-allowed characters.
     */
    public function testPipedCommandResultsWithWeirdCharsAsIworx()
    {
        $One = new Command('echo $(ls /tmp)');
        $One->setUser('iworx');
        $Two = new Command('cat');

        $One->pipe($Two)->exec();

        $this->assertCommandSuccessful($One);
        $this->assertEquals('$(ls /tmp)', $One->getOutput()[0]);
    }

    /**
     * Test that piped commands allow characters.
     */
    public function testPipedCommandResultsWithWeirdCharsAllowedAsIworx()
    {
        $One = new Command('echo $(ls /tmp)');
        $One->allowDollarSign()
            ->allowParenthesisBoth();
        $One->setUser('iworx');
        $Two = new Command('cat');

        $One->pipe($Two)->exec();

        $this->assertCommandSuccessful($One);
        $this->assertNotEquals('$(ls /tmp)', $One->getOutput()[0]);
    }
}
