<?php
/**
 * InterWorx Hosting Control Panel
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2014 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Aug 18, 2014
 */

/**
 * Class IWorx_EventTest.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class IWorx_EventTest extends IWorx_Framework_TestCase
{
    const EVENT_NAME             = 'test:event1';
    const EVENT_NAME_WILD        = 'test:*';
    const EVENT_THAT_FAILS       = 'test2:event:failure';
    const EVENT_WITH_SUCCESS_MSG = 'test2:event:success';
    const EVENT_WIN_INVOKE       = 'invoke:post:nw:Overview:winCommit';
    const ENV_FLAG               = 'test-event-fired';
    const MSG_SUCCESS            = 'A success message, yay!';
    const MSG_FAILURE            = 'A failure message, boo!';

    /**
     * Register the listener.
     */
    public function setUp()
    {
        // This is duplicated in the tear-down, but just for safety
        IW::Env()->clearEnvFlagFromProcess(self::ENV_FLAG);
    }

    /**
     * Tests that onFire gets called for a listener.
     */
    public function testEventManagerCallsListenerOnFireOnce()
    {
        $Event = new IWorx_EventType_Simple();
        $Event->setName(self::EVENT_NAME);

        $SpyListener = $this->getMockBuilder('IWorx_Event_Listener')->getMock();
        $SpyListener->expects($this->once())->method('onFire');

        $FakeManager = new FakeEventManager($SpyListener);

        $Reply = $FakeManager->fire($Event);
        $this->assertReplySuccessful($Reply);
    }

    /**
     * Fire the event and verify wild card listener caught it.
     */
    public function testFireWild()
    {
        $Listener = new TestListener();
        IW::Event()->register($Listener);

        $Event = new TestEvent();
        $Event->setName('test:wildcard');
        $Event->event_fired = $Event->getName();
        $Reply = IW::Event()->fire($Event);
        $this->assertTrue($Reply->wasSuccessful(), $Reply->getErrorsAsString());
        $this->_assertListenerWasCalled($Event);
    }

    /**
     * Test what happens when events fail.
     */
    public function testFireFailure()
    {
        $Listener = new TestListenerFail();
        IW::Event()->register($Listener);

        $Event = new TestEvent();
        $Event->setName(self::EVENT_THAT_FAILS);
        $Reply = IW::Event()->fire($Event);

        $this->assertFalse($Event->wasSuccessful());
        $this->assertTrue($Event->hasErrors());

        $this->assertFalse($Reply->wasSuccessful());
    }

    /**
     * Tests that the error message makes it back in the reply.
     */
    public function testFailureMessage()
    {
        $Listener = new TestListenerFail();
        IW::Event()->register($Listener);

        $Event = new TestEvent();
        $Event->setName(self::EVENT_THAT_FAILS);

        $Reply = IW::Event()->fire($Event);

        $found_message = false;

        foreach ($Reply->getErrors() as $error) {
            // Gotta look the hard way, because the error code is prepended to the message.
            if (strpos($error, self::MSG_FAILURE)  !== false) {
                $found_message = true;
            }
        }

        $this->assertTrue($found_message);
    }

    /**
     * Tests that the success message makes it back in the reply.
     */
    public function testSuccessMessage()
    {
        $Listener = new TestListener();
        IW::Event()->register($Listener);

        $Event = new TestEvent();
        $Event->setName(self::EVENT_WITH_SUCCESS_MSG);
        $Reply = IW::Event()->fire($Event);

        $expected = 'EventTests: ' . self::MSG_SUCCESS;
        $this->assertContains($expected, $Reply->getData());
    }

    /**
     * Test subscribing to an actual controller event.
     */
    public function testInvokedWinCommit()
    {
        $Listener = new TestListenerWinInvoke();
        IW::Event()->register($Listener);

        $NW = new NodeWorx(NodeWorx::MASTER_ID);

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Overview',
            'winCommit',
            [],
            $NW
        );

        $this->assertReplySuccessful($Reply);
        $this->assertEquals(
            self::EVENT_WIN_INVOKE,
            IW::Env()->getEnvFlag(self::ENV_FLAG)
        );
    }

    /**
     * Test the order of events fired.
     */
    public function testInvokedWinEventList()
    {
        $Listener = new TestListenerEventList();
        IW::Event()->register($Listener);

        $this->assertEquals([], TestListenerEventList::$events);

        $NW = new NodeWorx(NodeWorx::MASTER_ID);

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Overview',
            'winCommit',
            [],
            $NW
        );

        $this->assertReplySuccessful($Reply);

        $expected = ['form:validate:Form_Input',
                 'form:construct:Form_SubmitOnly',
                 'commit:pre:nw:Overview:winCommit',
                 'form:validate:Form_SubmitOnly',
                 'commit:post:nw:Overview:winCommit',
                 'invoke:post:nw:Overview:winCommit'];

        $this->assertEquals($expected, TestListenerEventList::$events);
    }

    /**
     * Test the order of events fired for a failing action.
     */
    public function testInvokedFailEventList()
    {
        $Listener = new TestListenerEventList();
        IW::Event()->register($Listener);

        $this->assertEquals([], TestListenerEventList::$events);

        $NW = new NodeWorx(NodeWorx::MASTER_ID);

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Overview',
            'failCommit',
            [],
            $NW
        );

        $this->assertReplyNotSuccessful($Reply, 'Expected fail succeeded');

        $expected = ['form:validate:Form_Input',
                 'form:construct:Form_SubmitOnly',
                 'commit:pre:nw:Overview:failCommit',
                 'form:validate:Form_SubmitOnly',
                 'invoke:post:nw:Overview:failCommit'];

        $this->assertEquals($expected, TestListenerEventList::$events);
    }

    /**
     * Unregister the listener.
     */
    public function tearDown()
    {
        foreach ($this->_getListeners() as $Listener) {
            IW::Event()->unregister($Listener);
        }
        IW::Env()->clearEnvFlagFromProcess(self::ENV_FLAG);
        IW::Env()->clearEnvFlagFromRequest(self::ENV_FLAG);

        TestListenerEventList::$events = [];
    }

    /**
     * Get the listeners to unregister.
     *
     * @return IWorx_Event_Listener[]
     */
    private function _getListeners()
    {
        $Listeners   = [];
        $Listeners[] = new TestListener();
        $Listeners[] = new TestListenerFail();
        $Listeners[] = new TestListenerWinInvoke();
        $Listeners[] = new TestListenerEventList();
        return $Listeners;
    }

    /**
     * Checks that the work done in TestListener::onFire was done the way we expect.
     *
     * @param IWorx_EventType_Abstract $Event
     */
    private function _assertListenerWasCalled(IWorx_EventType_Abstract $Event)
    {
        $this->assertEquals(
            IW::Env()->getEnvFlag(self::ENV_FLAG),
            $Event->getName()
        );
    }
}

/**
 * Class TestEvent.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class TestEvent extends IWorx_EventType_Abstract
{
    public $event_fired;
}

/**
 * Class FakeEventManager
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class FakeEventManager extends IWorx_Event_Manager
{
    /**
     * Constructor.
     *
     * @param IWorx_Event_Listener $Listener
     */
    public function __construct(IWorx_Event_Listener $Listener)
    {
        $this->_Listener = $Listener;
    }

    /**
     * Gets the array of registered listeners.
     *
     * @param IWorx_EventType_Abstract $E
     * @return IWorx_Event_Listener[]
     */
    protected function _getRegisteredListeners(IWorx_EventType_Abstract $E)
    {
        return [$this->_Listener];
    }
}

/**
 * Class TestListener.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class TestListener implements IWorx_Event_Listener
{
    use IWorx_Event_Listener_Trait;

    /**
     * Function called when a registered event is fired.
     *
     * @param IWorx_EventType_Abstract $Event
     */
    public function onFire(IWorx_EventType_Abstract $Event)
    {
        /* @var TestEvent $Event */
        IW::Env()->setEnvFlagForProcess(
            IWorx_EventTest::ENV_FLAG,
            $Event->event_fired
        );
        $this->addMessage(IWorx_EventTest::MSG_SUCCESS);
    }

    /**
     * Return an array of events to listen to.
     *
     * @return string[]
     */
    public function getEventList()
    {
        return [ IWorx_EventTest::EVENT_NAME,
             IWorx_EventTest::EVENT_NAME_WILD,
             IWorx_EventTest::EVENT_WITH_SUCCESS_MSG ];
    }

    /**
     * Gets the group id.
     *
     * @return string
     */
    public function getGroupId()
    {
        return 'EventTests';
    }
}

/**
 * Class TestListenerFail.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class TestListenerFail implements IWorx_Event_Listener
{
    use IWorx_Event_Listener_Trait;

    /**
     * Function called when a registered event is fired.
     *
     * @param IWorx_EventType_Abstract $Event
     */
    public function onFire(IWorx_EventType_Abstract $Event)
    {
        /* @var TestEvent $Event */
        IW::Env()->setEnvFlagForProcess(
            IWorx_EventTest::ENV_FLAG,
            $Event->event_fired
        );
        $this->addError('fail', IWorx_EventTest::MSG_FAILURE);
    }

    /**
     * Return an array of events to listen to.
     *
     * @return string[]
     */
    public function getEventList()
    {
        return [ IWorx_EventTest::EVENT_THAT_FAILS ];
    }

    /**
     * Gets the group id.
     *
     * @return string
     */
    public function getGroupId()
    {
        return 'EventTests';
    }
}

/**
 * Class TestListenerWinInvoke.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class TestListenerWinInvoke implements IWorx_Event_Listener
{
    use IWorx_Event_Listener_Trait;
    /**
     * Function called when a registered event is fired.
     *
     * @param IWorx_EventType_Abstract $Event
     */
    public function onFire(IWorx_EventType_Abstract $Event)
    {
        IW::Env()->setEnvFlagForRequest(
            IWorx_EventTest::ENV_FLAG,
            $Event->getName()
        );
        $this->addMessage(IWorx_EventTest::MSG_SUCCESS);
    }

    /**
     * Return an array of events to listen to.
     *
     * @return string[]
     */
    public function getEventList()
    {
        return [ IWorx_EventTest::EVENT_WIN_INVOKE ];
    }

    /**
     * Gets the group id.
     *
     * @return string
     */
    public function getGroupId()
    {
        return 'EventTests';
    }
}

/**
 * Class TestListenerWinEventList.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class TestListenerEventList implements IWorx_Event_Listener
{
    use IWorx_Event_Listener_Trait;

    static public $events = [];

    /**
     * Function called when a registered event is fired.
     *
     * @param IWorx_EventType_Abstract $Event
     */
    public function onFire(IWorx_EventType_Abstract $Event)
    {
        self::$events[] = $Event->getName();
        IW::Env()->setEnvFlagForRequest(
            IWorx_EventTest::ENV_FLAG,
            $Event->getName()
        );
        $this->addMessage(IWorx_EventTest::MSG_SUCCESS);
    }

    /**
     * Return an array of events to listen to.
     *
     * @return string[]
     */
    public function getEventList()
    {
        // There's no global '*' wildcard, only sub-wildcards.
        return [ 'commit:*', 'form:*', 'invoke:*' ];
    }

    /**
     * Gets the group id.
     *
     * @return string
     */
    public function getGroupId()
    {
        return 'EventTests';
    }
}
