<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2013 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage TESTS
 * @author     Tim College <tcollege@interworx.com>
 * @date       Feb 19, 2013
 */

/**
 * Tests for the IWorx_Ini class.
 *
 * @package    InterWorx
 * @subpackage TESTS
 */
class IWorx_IniTest extends \PHPUnit\Framework\TestCase
{
    /**
     * Data provider.
     *
     * @return array
     */
    public static function dataProviderGoodMatches()
    {
        return
        [
         ['name="value"', 'name', 'value', ''],
         ['name=value', 'name', 'value', ''],
         ['#name="value"', 'name', 'value', ''],
         ['#name=value', 'name', 'value', ''],
         ['name="value" #comment', 'name', 'value', '#comment'],
         ['#name="value" #comment', 'name', 'value', '#comment'],
         ['name=value #comment', 'name', 'value', '#comment'],
         ['name=value#comment', 'name', 'value', '#comment'],
         ['name=value ;comment', 'name', 'value', ';comment'],
         ['#name=value #comment', 'name', 'value', '#comment'],
         ['name=value #tim=sandwich #matt=banjo',
        'name',
        'value',
        '#tim=sandwich #matt=banjo'],
         ['name="value;"', 'name', 'value;', ''],
         ['name="value#"', 'name', 'value#', ''],
         ['name="value;" #comment', 'name', 'value;', '#comment'],
         ['name="value;" ;comment', 'name', 'value;', ';comment'],
         ['name="val\"ue"', 'name', 'val\"ue', ''],
         ['t="http://updates.interworx.com?value=something&amp;t=d"',
          't',
          'http://updates.interworx.com?value=something&amp;t=d',
          ''],
         ['name_2="boo"', 'name_2', 'boo', ''],
         ['name_name=boo','name_name','boo',''],
         ['flag-value', 'flag-value', true, ''],
         ['flag_value', 'flag_value', true, ''],
         ['!includedir /etc/my.cnf.d', '!includedir /etc/my.cnf.d', true, ''],
         ['!includedir /etc/my.cnf.d #lol',
          '!includedir /etc/my.cnf.d ',
          true,
          '#lol'],
         ['flag-value # comment', 'flag-value ', true, '# comment'],
         ['flag-value# comment', 'flag-value', true, '# comment'],
         ['flag multi test x', 'flag multi test x', true, ''],

         // This case detects the spaces in "name", and reverts to Flag mode.
         ['name with space=testing 123',
          'name with space=testing 123',
          true,
          ''],

         // This case detects the weird characters in "name" and reverts to Flag mode.
         ['!@%!@', '!@%!@', true, ''],
        ];
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public static function dataProviderGet()
    {
        return array( array( 'section1', 'index1', 'value1' ),
                  array( 'section1', 'index2', 'value2' ),
                  array( 'section2', 'index1', 'value1' ),
                  array( 'section2', 'index2', 'value2' ),
                  );
    }

    /**
     * Gets an instance of the ini.
     *
     * @return IWorx_Ini
     */
    public static function getIni()
    {
        return new IWorx_Ini('/home/interworx/TESTS/unit/IWorx/test.ini');
    }

    /**
     * Get a copy of the imapd-ssl.dist file for testing.
     *
     * @return string
     */
    public static function getCourierConfig()
    {
        return '/home/interworx/TESTS/unit/IWorx/imapd-ssl.dist';
    }

    /**
     * Tests creation of IWorx_Ini_Line objects.
     *
     * @param string $line
     * @param string $name
     * @param string $value
     * @param string $comment
     * @dataProvider dataProviderGoodMatches
     */
    public function testGoodMatches($line, $name, $value, $comment)
    {
        $Line = IWorx_Ini_Line::factory($line);

        /* @var $Line IWorx_Ini_Line_Value */
        $this->assertNotEquals(false, $Line);
        $this->assertEquals($name, $Line->getName(), 'Name failed');
        $this->assertEquals($value, $Line->getValue(), 'Value failed');
        $this->assertEquals($comment, $Line->getInlineComment(), 'Comment failed');
        $this->assertEquals($line, trim($Line->getLine()), 'Whole Line failed');
    }

    /**
     * Tests the getting of values.
     *
     * @param string $section
     * @param string $index
     * @param string $expected
     * @dataProvider dataProviderGet
     */
    public function testGet($section, $index, $expected)
    {
        $Ini = $this->getIni();
        $Ini->read();

        $actual = $Ini->getSection($section)->getValue($index);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Tests getting an unknown value.
     */
    public function testGetUnknownValue()
    {
        $Ini = $this->getIni();

        $actual = $Ini->getSection('section1')->getValue('index3');

        $this->assertNull($actual);
    }

    /**
     * Tests behavior when getting a value from an unknown section.
     */
    public function testGetUnknownSection()
    {
        $Ini = $this->getIni();

        $actual = $Ini->getSection('section3')->getValue('index3');

        $this->assertNull($actual);
    }

    /**
     * Tests setting an existing index to a new value.
     */
    public function testSetValueInSection()
    {
        $Ini = $this->getIni();

        $Ini->read();
        $Ini->getSection('section1')->setValue('index1', 'bob');

        $actual = $Ini->getSection('section1')->getValue('index1');

        $this->assertEquals('bob', $actual);
    }

    /**
     * Tests setting a new index.
     */
    public function testSetNewIndexInSection()
    {
        $Ini = $this->getIni();

        $Ini->read();
        $Ini->getSection('section1')->setValue('index3', 'bob');

        $actual = $Ini->getSection('section1')->getValue('index3');

        $this->assertEquals('bob', $actual);
    }

    /**
     * Tests setting a whole new section.
     */
    public function testSetNewSection()
    {
        $Ini = $this->getIni();

        $Ini->read();
        $Ini->getSection('section3')->setValue('index3', 'bob');

        $actual = $Ini->getSection('section3')->getValue('index3');

        $this->assertEquals('bob', $actual);
    }

    /**
     * Tests getting a flag.
     */
    public function testGetFlagSetting()
    {
        $Ini = $this->getIni();

        $Ini->read();
        $this->assertTrue($Ini->getSection('section2')->getValue('single-value'));
    }

    /**
     * Tests building contents.
     */
    public function testBuildContents()
    {
        $Ini = $this->getIni();
        $Ini->read();

        $expected = file_get_contents('/home/interworx/TESTS/unit/IWorx/test.ini');

        $this->assertEquals($expected, $Ini->buildContents());
    }

    /**
     * Tests courier style contents.
     */
    public function testCourierStyleBuildContents()
    {
        $Ini = new IWorx_Ini(self::getCourierConfig());
        $Ini->read();
        $expected = file_get_contents(self::getCourierConfig());

        $this->assertEquals($expected, $Ini->buildContents());
    }
}
