<?php

use IWorx\Query;

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2023 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Aug 17, 2023
 */

class IWorx_QueryTest extends IWorx_Framework_TestCase
{
    public function testHasParamsTrue()
    {
        $Q = new Query('SELECT who cares', ['yes']);

        $this->assertTrue($Q->hasParams());
    }

    public function testHasParamsFalse()
    {
        $Q = new Query('SELECT who cares');

        $this->assertFalse($Q->hasParams());
    }

    public function testNamedVar()
    {
        $sql      = 'SELECT * FROM whatever WHERE column1 = :value1';
        $expected = 'SELECT * FROM whatever WHERE column1 = ?';
        $params   = [':value1' => 24];

        $Q = new Query($sql, $params);

        $this->assertEquals($expected, $Q->getSql());
        $this->assertEquals([24], $Q->getParams());
    }

    public function testNamedVarUnprefixedVar()
    {
        $sql      = 'SELECT * FROM whatever WHERE column1 = :value1';
        $expected = 'SELECT * FROM whatever WHERE column1 = ?';
        $params   = ['value1' => 24];

        $Q = new Query($sql, $params);

        $this->assertEquals($expected, $Q->getSql());
        $this->assertEquals([24], $Q->getParams());
    }

    public function testUnnamedVar()
    {
        $sql      = 'SELECT * FROM whatever WHERE column1 = ?';
        $expected = 'SELECT * FROM whatever WHERE column1 = ?';
        $params   = [24];

        $Q = new Query($sql, $params);

        $this->assertEquals($expected, $Q->getSql());
        $this->assertEquals([24], $Q->getParams());
    }

    public function testNamedVarMultiple()
    {
        $sql      = 'SELECT * FROM whatever WHERE column1 = :value1 AND column2 = :value2';
        $expected = 'SELECT * FROM whatever WHERE column1 = ? AND column2 = ?';

        $params = [
            ':value1' => 24,
            ':value2' => 12,
        ];

        $Q = new Query($sql, $params);

        $this->assertEquals($expected, $Q->getSql());
        $this->assertEquals([24,12], $Q->getParams());
    }

    public function testNamedVarUsedTwice()
    {
        $sql      = 'SELECT * FROM whatever WHERE column1 = :value1 AND column2 = :value1';
        $expected = 'SELECT * FROM whatever WHERE column1 = ? AND column2 = ?';
        $params   = [':value1' => 24];

        $Q = new Query($sql, $params);

        $this->assertEquals($expected, $Q->getSql());
        $this->assertEquals([24,24], $Q->getParams());
    }

    public function testNamedVarMissing()
    {
        $sql = 'SELECT * FROM whatever
                WHERE column1 = :value1
                AND column2   = :value2';

        $params = [
            ':value1' => 24,
        ];

        $Q = new Query($sql, $params);

        $this->expectException(IWorx_Exception_DoingItWrong::class);
        $Q->getSql();
    }

    public function testNamedVarExtraValues()
    {
        $sql = 'SELECT * FROM whatever WHERE column1 = :value1';

        $params = [
            ':value1' => 24,
            ':value2' => 12,
        ];

        $Q = new Query($sql, $params);

        // The value2 should NOT be in the processed params
        $expected = [24];
        $this->assertEquals($expected, $Q->getParams());
    }

    public function testMixingNamedAndUnnamedNotAllowed()
    {
        $sql = 'SELECT * FROM whatever WHERE column1 = :value1 AND column2 = ?';

        $params = [
            ':value1' => 24,
        ];

        $Q = new Query($sql, $params);

        $this->expectException(IWorx_Exception_DoingItWrong::class);
        $Q->getSql();
    }

    public function testArrayAsVar()
    {
        $sql      = 'SELECT * FROM whatever WHERE column1 IN( :values )';
        $expected = 'SELECT * FROM whatever WHERE column1 IN( ?, ?, ? )';
        $params   = [':values' => [1, 2, 3]];

        $Q = new Query($sql, $params);

        $this->assertEquals($expected, $Q->getSql());

        // This is not super obvious, but instead of the array containing the
        // original array, it's been extracted to the top level.
        $this->assertEquals([1, 2, 3], $Q->getParams());
    }

    public function testArrayAsVarComplex()
    {
        $sql      = 'SELECT * FROM whatever WHERE column1 = :value1 AND column2 IN( :values )';
        $expected = 'SELECT * FROM whatever WHERE column1 = ? AND column2 IN( ?, ?, ? )';
        $params   = [
            ':value1' => 'Fred',
            ':values' => [1, 2, 3],
        ];

        $Q = new Query($sql, $params);

        $this->assertEquals($expected, $Q->getSql());
        $this->assertEquals(['Fred', 1, 2, 3], $Q->getParams());
    }
}
