<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Test
 * @author     Tim College <tcollege@interworx.com>
 * @date       Dec 29, 2009
 */

use IWorx\Process\System;

/**
 * Tests for the Search_Engines.
 *
 * @package    InterWorx
 * @subpackage Test
 */
class IWorx_Search_EngineTest extends IWorx_Framework_TestCase
{
    /**
     * Sets up the database.
     *
     * @throws \IWorx\Process\Exception\CmdValidationException
     * @throws \IWorx\Process\Exception\LogicException
     */
    public static function setUpBeforeClass()
    {
        parent::setUpBeforeClass();
        self::setUpWorldDatabase();
    }

    /**
     * Tests that an unrestricted search returns all rows in the table.
     */
    public function testUnrestrictedSearch()
    {
        $Engine = new Simple_Search_Engine();
        $Engine->addSelectableField('ID');
        $results = $Engine->search();

        $this->assertEquals(4079, count($results));
    }

    /**
     * Tests that the result of the foundRows() function is the same as expected.
     *
     * @depends testUnrestrictedSearch
     */
    public function testFoundRows()
    {
        $Engine = new Simple_Search_Engine();
        $Engine->addSelectableField('ID');
        $results = $Engine->search();

        $this->assertEquals(4079, $Engine->foundRows());
    }

    /**
     * Tests restriction to a single ID.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testTextRangeEquals()
    {
        $this->_simpleEngineTest(array( 'ID' => '2' ), 1, 2);
    }

    /**
     * Tests inclusion of multiple IDs.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testTextRangeEqualsMultiple()
    {
        $this->_simpleEngineTest(array( 'ID' => '2,367,15,2424' ), 4, 2);
    }

    /**
     * Tests exclusion of a single ID.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testTextRangeNotEquals()
    {
        $this->_simpleEngineTest(array( 'ID' => '!2' ), 4078, 1);
    }

    /**
     * Tests exclusion of multiple IDs.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testTextRangeNotEqualsMultiple()
    {
        $this->_simpleEngineTest(array( 'ID' => '!2,!367,!15,!2424' ), 4075, 1);
    }

    /**
     * Tests really stupid input.  If this ever works, it's bad bad bad.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testTextRangeMutuallyExclusive()
    {
        $this->_simpleEngineTest(array( 'ID' => '2,!2' ), 0);
    }

    /**
     * Tests restriction to range less than a specified value.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testTextRangeLessThan()
    {
        $this->_simpleEngineTest(array( 'ID' => '<10' ), 10, 1);
    }

    /**
     * Tests restriction to range greater than a specified value.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testTextRangeGreaterThan()
    {
        $this->_simpleEngineTest(array( 'ID' => '>4070'), 10, 4070);
    }

    /**
     * Tests restriction to a range of IDs.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testTextRangeBetween()
    {
        $this->_simpleEngineTest(array( 'ID' => '10::19'), 10, 10);
    }

    /**
     * Tests exclusion of a range of IDs.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testTextRangeNotBetween()
    {
        $this->_simpleEngineTest(array( 'ID' => '!10::19'), 4069, 1);
    }

    /**
     * Tests searching on a field not returned.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testTextRangeNonReturnedField()
    {
        $this->_simpleEngineTest(array( 'Population' => '>1000000'), 238, 1);
    }

    /**
     * Tests searching with empty.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testTextRangeEmpty()
    {
        $this->_simpleEngineTest(array( 'District' => 'empty'), 5, 1);
    }

    /**
     * Tests searching with not empty.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testTextRangeNotEmpty()
    {
        $this->_simpleEngineTest(array( 'District' => '!empty'), 4074, 2);
    }

    /**
     * Tests restriction to a single datetime.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeEquals()
    {
        $this->_simpleEngineTest(array( 'TestDate' => '2010-03-02 12:58:46'), 1, 4079);
    }

    /**
     * Tests restriction to a single datetime using unix timestamps.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeEqualsUnixTimestamp()
    {
        $date = strtotime('2010-03-02 12:58:46');
        $this->_simpleEngineTest(array( 'TestDate' => $date), 1, 4079);
    }

    /**
     * Tests restriction to multiple datetimes.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeEqualsMultiple()
    {
        $options = array( 'TestDate' => '2010-03-02 12:58:46,1990-03-07 14:44:56' );
        $this->_simpleEngineTest($options, 2, 4074);
    }

    /**
     * Tests exclusion of a single datetime.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeNotEquals()
    {
        $options = array( 'TestDate' => '!2010-03-02 12:58:46' );
        $this->_simpleEngineTest($options, 4078, 1);
    }

    /**
     * Tests exclusion of a single datetime using unix timestamps.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeNotEqualsUnixTimestamp()
    {
        $date = strtotime('2010-03-02 12:58:46');

        $options = array( 'TestDate' => "!{$date}" );
        $this->_simpleEngineTest($options, 4078, 1);
    }

    /**
     * Tests exclusion of multiple datetimes.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeNotEqualsMultiple()
    {
        $options = array( 'TestDate' => '!2010-03-02 12:58:46,!1990-03-07 14:44:56' );
        $this->_simpleEngineTest($options, 4077, 1);
    }

    /**
     * Tests inclusion of datetimes less than.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeLessThan()
    {
        $options = array( 'TestDate' => '<1995-03-02 12:58:46' );
        $this->_simpleEngineTest($options, 1432, 5);
    }

    /**
     * Tests inclusion of datetimes greater than.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeGreaterThan()
    {
        $this->_simpleEngineTest(array( 'TestDate' => '>2009-03-02 12:58' ), 1713, 2);
    }

    /**
     * Tests inclusion of datetimes between.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeBetween()
    {
        $options = array( 'TestDate' => '2000-01-01::2009-12-31' );
        $this->_simpleEngineTest($options, 613, 3915);
    }

    /**
     * Tests inclusion of datetimes between using unix timestamps.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeBetweenUnixTimestamp()
    {
        $from = strtotime('2000-01-01 00:00:00');
        $to   = strtotime('2009-12-31 23:59:59');

        $options = array( 'TestDate' => "{$from}::{$to}" );
        $this->_simpleEngineTest($options, 613, 3915);
    }

    /**
     * Tests inclusion of datetimes not between.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeNotBetween()
    {
        $options = array( 'TestDate' => '!1990-01-01::2009-12-31' );
        $this->_simpleEngineTest($options, 2870, 1);
    }

    /**
     * Tests exclusion and inclusion of multiple datetime ranges that overlap.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeNotBetweenAndBetween()
    {
        $options = array( 'TestDate' =>
                      '2010-01-01::2011-01-01,!2010-06-01::2010-07-01' );
        $this->_simpleEngineTest($options, 72, 2822);
    }

    /**
     * Tests inclusion of datetimes with empty.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeEmpty()
    {
        $this->_simpleEngineTest(array( 'TestDate' => 'empty' ), 1, 1);
    }

    /**
     * Tests inclusion of datetimes with not empty.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeNotEmpty()
    {
        $this->_simpleEngineTest(array( 'TestDate' => '!empty' ), 4078, 2);
    }

    /**
     * Tests inclusion of datetimes from this year.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeThisYear()
    {
        $this->_simpleEngineTest(array( 'TestDate' => 'this year' ), 58, 1321);
    }

    /**
     * Tests inclusion of datetimes from next year.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeNextYear()
    {
        $this->_simpleEngineTest(array( 'TestDate' => 'next year' ), 60, 1288);
    }

    /**
     * Tests inclusion of datetimes from last year.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeLastYear()
    {
        $this->_simpleEngineTest(array( 'TestDate' => 'last year' ), 64, 144);
    }

    /**
     * Tests changing the now reference point.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeChangingNow()
    {
        $Engine = $this->_getSimpleEngine();
        $Engine->setNow(strtotime('2008-05-15 14:00:00'));
        $Engine->setOptions(array( 'TestDate' => 'this year' ));
        $this->_searchAndAssert($Engine, 60, 1288);
    }

    /**
     * Tests inclusion of datetimes from this month.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeThisMonth()
    {
        $this->_simpleEngineTest(array( 'TestDate' => 'this month' ), 6, 214);
    }

    /**
     * Tests inclusion of datetimes from next month.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeNextMonth()
    {
        $this->_simpleEngineTest(array( 'TestDate' => 'next month' ), 4, 2565);
    }

    /**
     * Tests inclusion of datetimes from last month.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeLastMonth()
    {
        $this->_simpleEngineTest(array( 'TestDate' => 'last month' ), 6, 338);
    }

    /**
     * Tests inclusion of datetimes from this week.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeThisWeek()
    {
        $this->_simpleEngineTest(array( 'TestDate' => 'this week' ), 2, 214);
    }

    /**
     * Tests inclusion of datetimes from next week.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeNextWeek()
    {
        $this->_simpleEngineTest(array( 'TestDate' => 'next week' ), 2, 107);
    }

    /**
     * Tests inclusion of datetimes from last week.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeLastWeek()
    {
        $this->_simpleEngineTest(array( 'TestDate' => 'last week' ), 1, 3420);
    }

    /**
     * Tests inclusion of datetimes from last week through this week.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeLastWeekThroughThisWeek()
    {
        $options = array( 'TestDate' => 'last week::this week' );
        $this->_simpleEngineTest($options, 3, 3420);
    }

    /**
     * Tests inclusion of datetimes from today.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeToday()
    {
        $this->_simpleEngineTest(array( 'TestDate' => 'today' ), 1, 4052);
    }

    /**
     * Tests inclusion of datetimes from yesterday.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeYesterday()
    {
        $this->_simpleEngineTest(array( 'TestDate' => 'yesterday' ), 1, 214);
    }

    /**
     * Tests inclusion of datetimes from tomorrow.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeTomorrow()
    {
        $this->_simpleEngineTest(array( 'TestDate' => 'tomorrow' ), 1, 107);
    }

    /**
     * Tests searching on a date and a text at the same time.
     *
     * @depends testUnrestrictedSearch
     * @depends testFoundRows
     */
    public function testDateRangeWithTextRange()
    {
        $options = array( 'TestDate' => 'this year', 'CountryCode' => 'USA' );
        $this->_simpleEngineTest($options, 1, 4052);
    }

    /**
     * Tests default return format.
     *
     * @depends testTextRangeEquals
     */
    public function testDefaultFetchMode()
    {
        $Engine = $this->_getSimpleEngine();
        $Engine->setOptions(array( 'ID' => '2' ));
        $results = $Engine->search();

        $expected        = [];
        $expected[0]     = (object) null;
        $expected[0]->ID = '2';

        $this->assertEquals($expected, $results);
    }

    /**
     * Tests FETCH_ASSOC format.
     *
     * @depends testTextRangeEquals
     */
    public function testAssocFetchMode()
    {
        $Engine = $this->_getSimpleEngine();
        $Engine->setOptions(array( 'ID' => '2' ));
        $results = $Engine->search(IWorx_Search_Engine::FETCH_ASSOC);

        $expected = array( array( 'ID' => '2' ) );

        $this->assertEquals($expected, $results);
    }

    /**
     * Tests FETCH_NUMERIC format.
     *
     * @depends testTextRangeEquals
     */
    public function testNumericFetchMode()
    {
        $Engine = $this->_getSimpleEngine();
        $Engine->setOptions(array( 'ID' => '2' ));
        $results = $Engine->search(IWorx_Search_Engine::FETCH_NUMERIC);

        $expected = array( array( '2' ) );

        $this->assertEquals($expected, $results);
    }

    /**
     * Tests FETCH_INDEXED format.
     *
     * @depends testTextRangeEquals
     */
    public function testIndexedFetchMode()
    {
        $Engine = $this->_getSimpleEngine();
        $Engine->setOptions(array( 'ID' => '2' ));
        $results = $Engine->search(IWorx_Search_Engine::FETCH_INDEXED);

        $expected             = [];
        $expected['2'][0]     = (object) null;
        $expected['2'][0]->ID = '2';

        $this->assertEquals($expected, $results);
    }

    /**
     * Tests FETCH_COLUMN format.
     *
     * @depends testTextRangeEquals
     */
    public function testColumnFetchMode()
    {
        $Engine = $this->_getSimpleEngine();
        $Engine->setOptions(array( 'ID' => '2' ));
        $results = $Engine->search(IWorx_Search_Engine::FETCH_COLUMN);

        $expected = array( '2' );

        $this->assertEquals($expected, $results);
    }

    /**
     * Tests search output when adding another field.
     *
     * @depends testTextRangeEquals
     */
    public function testAddingSelectableField()
    {
        $Engine = $this->_getSimpleEngine();
        $Engine->setOptions(array( 'ID' => '4052' ));
        $Engine->addSelectableField('Name');
        $results = $Engine->search();

        $expected          = [];
        $expected[0]       = (object) null;
        $expected[0]->ID   = '4052';
        $expected[0]->Name = 'Compton';

        $this->assertEquals($expected, $results);
    }

    /**
     * Tests search output when adding multiple fields.
     *
     * @depends testTextRangeEquals
     */
    public function testAddingMultipleSelectableFields()
    {
        $Engine = $this->_getSimpleEngine();
        $Engine->setOptions(array( 'ID' => '4052' ));
        $Engine->addSelectableFields(array( 'Name', 'CountryCode' ));
        $results = $Engine->search();

        $expected                 = [];
        $expected[0]              = (object) null;
        $expected[0]->ID          = '4052';
        $expected[0]->Name        = 'Compton';
        $expected[0]->CountryCode = 'USA';

        $this->assertEquals($expected, $results);
    }

    /**
     * Tests search output when adding all selectable fields.
     *
     * @depends testTextRangeEquals
     */
    public function testAddingAllSelectableFields()
    {
        $Engine = $this->_getSimpleEngine();
        $Engine->setOptions(array( 'ID' => '4052' ));
        $Engine->addAllSelectableFields();
        $results = $Engine->search();

        $expected                 = [];
        $expected[0]              = (object) null;
        $expected[0]->ID          = '4052';
        $expected[0]->Name        = 'Compton';
        $expected[0]->CountryCode = 'USA';
        $expected[0]->District    = 'California';
        $expected[0]->Population  = '92864';
        $expected[0]->TestDate    = strtotime('2007-08-04 08:37:26');

        $this->assertEquals($expected, $results);
    }

    /**
     * Tests adding non-existant fields.
     */
    public function testAddingBogusField()
    {
        $this->expectException(IWorx_Exception_SearchEngine::class);
        $Engine = $this->_getSimpleEngine();
        $Engine->addSelectableField('This does not exist');
    }

    /**
     * Tests changing the order of the output.
     *
     * @depends testUnrestrictedSearch
     */
    public function testOrderBy()
    {
        $Engine = $this->_getSimpleEngine();
        $Engine->setOrderBy('ID DESC');
        $this->_searchAndAssert($Engine, 4079, 4079);
    }

    /**
     * Tests default return format with a complex engine.
     *
     * @depends testTextRangeEquals
     */
    public function testDefaultFetchModeComplex()
    {
        $Engine = $this->_getComplexEngine();
        $Engine->setOptions(array( 'ID' => '2' ));
        $results = $Engine->search();

        $expected        = [];
        $expected[0]     = (object) null;
        $expected[0]->ID = '2';

        $this->assertEquals($expected, $results);
    }

    /**
     * Tests the adding of aliased fields.
     */
    public function testAddingAliasedSelectableField()
    {
        $Engine = $this->_getComplexEngine();
        $Engine->setOptions(array( 'ID' => '4052' ));
        $Engine->addSelectableField('CountryName');

        $results = $Engine->search();

        $expected                 = [];
        $expected[0]              = (object) null;
        $expected[0]->ID          = '4052';
        $expected[0]->CountryName = 'United States';
    }

    /**
     * Tests search output when adding another field from a joined table.
     *
     * @depends testTextRangeEquals
     */
    public function testAddingSelectableFieldFromJoinedTable()
    {
        $Engine = $this->_getComplexEngine();
        $Engine->setOptions(array( 'ID' => '4052' ));
        $Engine->addSelectableField('Capital');
        $results = $Engine->search();

        $expected             = [];
        $expected[0]          = (object) null;
        $expected[0]->ID      = '4052';
        $expected[0]->Capital = '3813';

        $this->assertEquals($expected, $results);
    }

    /**
     * Tests search output when adding another field from a joined table.
     *
     * @depends testTextRangeEquals
     */
    public function testAddingSelectableFieldFromJoinedTable2()
    {
        $Engine = $this->_getComplexEngine();
        $Engine->setOptions(array( 'ID' => '4052' ));
        $Engine->addSelectableField('OfficialLanguage');
        $results = $Engine->search();

        $expected                      = [];
        $expected[0]                   = (object) null;
        $expected[0]->ID               = '4052';
        $expected[0]->OfficialLanguage = 'English';

        $this->assertEquals($expected, $results);
    }

    /**
     * Tests adding a join that will increase the number of results.
     */
    public function testAddingOneToManyJoin()
    {
        $Engine = $this->_getComplexEngine();
        $Engine->setOptions(array( 'ID' => '4052' ));
        $Engine->addSelectableField('Language');

        $this->_searchAndAssert($Engine, 12, 4052);
    }

    /**
     * Tests a search that ends with a wildcard.
     */
    public function testEndsWithWildcard()
    {
        $Engine = $this->_getComplexEngine();
        $Engine->setPartialMatches(true);
        $Engine->addSelectableField('Name');

        $Engine->setOptions(array( 'Name' => 'Washing*' ));

        $this->_searchAndAssert($Engine, 1, 3813);
    }

    /**
     * Tests a search that contains a wildcard.
     */
    public function testContainsWildcard()
    {
        $Engine = $this->_getComplexEngine();
        $Engine->setPartialMatches(true);
        $Engine->addSelectableField('Name');

        $Engine->setOptions(array( 'Name' => 'Washing*on' ));

        $this->_searchAndAssert($Engine, 1, 3813);
    }

    /**
     * Tests a search that begins with a wildcard.
     */
    public function testBeginsWithWildcard()
    {
        $Engine = $this->_getComplexEngine();
        $Engine->setPartialMatches(true);
        $Engine->addSelectableField('Name');

        $Engine->setOptions(array( 'Name' => '*ashington' ));

        $this->_searchAndAssert($Engine, 1, 3813);
    }

    /**
     * Tests a search that begins with a wildcard and ends with a wildcard.
     */
    public function testBeginsAndEndsWithWildcard()
    {
        $Engine = $this->_getComplexEngine();
        $Engine->setPartialMatches(true);
        $Engine->addSelectableField('Name');

        $Engine->setOptions(array( 'Name' => '*ashingt*' ));

        $this->_searchAndAssert($Engine, 1, 3813);
    }

    /**
     * Gets a simple search engine.
     *
     * @param array $fields
     * @return Simple_Search_Engine
     */
    private function _getSimpleEngine(array $fields = array( 'ID' ))
    {
        $Engine = new Simple_Search_Engine();
        $Engine->addSelectableFields($fields);
        return $Engine;
    }

    /**
     * Gets a more complicated search engine.
     *
     * @param array $fields
     * @return Less_Simple_Search_Engine
     */
    private function _getComplexEngine(array $fields = array( 'ID' ))
    {
        $Engine = new Less_Simple_Search_Engine();
        $Engine->addSelectableFields($fields);
        return $Engine;
    }

    /**
     * Helper function to reduce cut and pasted code.
     *
     * @param array        $options
     * @param integer      $expected_rows
     * @param integer|null $first_row_id
     * @param array        $fields
     */
    private function _simpleEngineTest(
        array $options,
        $expected_rows,
        $first_row_id = null,
        array $fields = array( 'ID' )
    ) {
        $Engine = $this->_getSimpleEngine($fields);
        $Engine->setOptions($options);
        // Set "now" to be a time we know has data
        $Engine->setNow(strtotime('2007-08-04 08:37:26'));
        $this->_searchAndAssert($Engine, $expected_rows, $first_row_id);
    }

    /**
     * Executes search and does some common asserts.
     *
     * @param IWorx_Search_Engine $Engine
     * @param integer             $expected_rows
     * @param integer|null        $first_row_id
     */
    private function _searchAndAssert(
        IWorx_Search_Engine $Engine,
        $expected_rows,
        $first_row_id = null
    ) {
        $results = $Engine->search();

        $this->assertEquals(
            $expected_rows,
            $Engine->foundRows(),
            'Wrong number of results returned'
        );

        if ($first_row_id !== null) {
            $this->assertEquals(
                $first_row_id,
                intval($results[0]->ID),
                'Unexpected ID found in results'
            );
        }
    }
}

/**
 * Simple test search engine.
 *
 * @package    InterWorx
 * @subpackage Test
 */
class Simple_Search_Engine extends IWorx_Search_Engine
{
    /**
     * Constructor.
     */
    public function __construct()
    {
        $this->setBaseTable(new IWorx_Search_Table('City', 'city'));
        parent::__construct();
    }

    /**
     * Initializes selectable fields.
     */
    protected function _initSelectableFields()
    {
        $this->_addSelectableFieldFrom('city', 'ID');
        $this->_addSelectableFieldFrom('city', 'Name');
        $this->_addSelectableFieldFrom('city', 'CountryCode');
        $this->_addSelectableFieldFrom('city', 'District');
        $this->_addSelectableFieldFrom('city', 'Population');
        $this->_addSelectableDateFieldFrom('city', 'TestDate');
    }
}

/**
 * Complex test search engine.
 *
 * @package    InterWorx
 * @subpackage Test
 */
class Less_Simple_Search_Engine extends Simple_Search_Engine
{
    /**
     * Set up the selectable fields.
     */
    protected function _initSelectableFields()
    {
        parent::_initSelectableFields();

        $this->_addSelectableFieldFrom('cn', 'Continent');
        $this->_addSelectableFieldFrom('cn', 'Region');
        $this->_addSelectableFieldFrom('cn', 'Capital');
        $this->_addAliasedSelectableFieldFrom('cn', 'Name', 'CountryName');

        $this->_addAliasedSelectableFieldFrom('ol', 'Language', 'OfficialLanguage');

        $this->_addSelectableFieldFrom('cl', 'Language');
    }

    /**
     * Creates table objects.
     */
    protected function _initTableObjects()
    {
        $this->_makeAndAddTable('Country', 'cn');
        $this->_makeAndAddTable('CountryLanguage', 'ol');
        $this->_makeAndAddTable('CountryLanguage', 'cl');
    }

    /**
     * Creates links between tables.
     */
    protected function _initTableObjectLinks()
    {
        $this->_linkTables('city', 'cn', 'CountryCode', 'Code');
        $this->_linkTables('cl', 'cn', 'CountryCode', 'Code');
        $this->_linkTables(
            'ol',
            'cn',
            '',
            '',
            'cn.Code = ol.CountryCode AND ol.IsOfficial = "T"'
        );
    }
}
