<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2018 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Upgrade
 * @author     Tim College <tcollege@interworx.com>
 * @date       Dec 11, 2018
 */

use IWorx\Template\FileTemplate;
use IWorx\Templateable;

/**
 * Class IWorx_Template_OverrideTest
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorx_Template_OverrideTest extends IWorx_Framework_TestCase
{
    /**
     * Clean up from the last run.
     */
    public static function setUpBeforeClass()
    {
        self::deleteSWAccountIfNecessaryStatic();
    }

    /**
     * Get a chain of configurable objects.
     *
     * @return Templateable[]
     */
    public function dataProviderConfigurables()
    {
        return [
            [IW::Env()],
            [NodeWorx::getDefaultNodeWorx()],
            [$this->getSWAccount()],
            [$this->getSWAccount()->getPackage()],
            [$this->getSWAccount()->getMasterDomain()]
        ];
    }

    /**
     * Test default behavior.
     */
    public function testDefaultTemplate()
    {
        $Template = new FileTemplate('test/test-override');
        $this->assertEquals('Default', trim($Template->toString()));
    }

    /**
     * Test default template with Templateable set.
     */
    public function testDefaultTemplateWithContext()
    {
        foreach ($this->dataProviderConfigurables() as $args) {
            $Templateable = $args[0];
            $Template = new FileTemplate('test/test-override');
            $Template->setContext($Templateable);
            $this->assertEquals('Default', trim($Template->toString()));
        }
    }

    /**
     * Set up the global override.
     */
    protected function _setupGlobalOverride()
    {
        $dir = '/usr/local/interworx/etc/templates/custom-global/test';
        IWorxFileSys::mkdir($dir, 0700, true);
        $file = "{$dir}/test-override.tpl";
        file_put_contents($file, 'Global');
        IWorxFileSys::chgrp('iworx', $file);
        IWorxFileSys::chmod(0660, $file);
    }

    /**
     * Test global template with Templateable set.
     */
    public function testGlobalTemplateWithContext()
    {
        $this->_setupGlobalOverride();
        foreach ($this->dataProviderConfigurables() as $args) {
            $Templateable = $args[0];
            $Template     = new FileTemplate('test/test-override');
            $Template->setContext($Templateable);
            $this->assertEquals('Global', trim($Template->toString()));
        }
    }

    /**
     * Set up the reseller override.
     */
    public function _setupResellerOverride()
    {
        $dir = '/usr/local/interworx/etc/templates/custom-reseller/1/test';
        IWorxFileSys::mkdir($dir, \IWorx\Template::PERM_FOLDER, true);
        $file = "{$dir}/test-override.tpl";
        file_put_contents($file, 'Reseller');
        IWorxFileSys::chgrp('iworx', $file);
        IWorxFileSys::chmod(\IWorx\Template::PERM_FILE, $file);
    }

    /**
     * Test behavior overriding at the reseller level.
     */
    public function testResellerOverride()
    {
        $this->_setupResellerOverride();

        $Template = new FileTemplate('test/test-override');

        $this->assertEquals('Global', $Template->toString());

        $Template->setContext(NodeWorx::getDefaultNodeWorx());

        $this->assertEquals('Reseller', $Template->toString());

        $Template->setContext($this->getSWAccount());

        $this->assertEquals('Reseller', $Template->toString());

        $Template->setContext($this->getSWAccount()->getMasterDomain());

        $this->assertEquals('Reseller', $Template->toString());
    }

    /**
     * Set up the siteworx override.
     */
    public function _setupSiteworxOverride()
    {
        $SW = $this->getSWAccount();
        $dir = "{$SW->getLocalStateDir('templates_siteworx')}/test";
        IWorxFileSys::mkdir($dir, \IWorx\Template::PERM_FOLDER, true);
        $file = "{$dir}/test-override.tpl";
        file_put_contents($file, 'Siteworx');
        IWorxFileSys::chgrp('iworx', $file);
        IWorxFileSys::chmod(\IWorx\Template::PERM_FILE, $file);
    }

    /**
     * Test behavior overriding at the siteworx level.
     */
    public function testSiteworxOverride()
    {
        $this->_setupSiteworxOverride();

        $Template = new FileTemplate('test/test-override');

        $this->assertEquals('Global', $Template->toString());

        $Template->setContext(NodeWorx::getDefaultNodeWorx());

        $this->assertEquals('Reseller', $Template->toString());

        $Template->setContext($this->getSWAccount());

        $this->assertEquals('Siteworx', $Template->toString());

        $Template->setContext($this->getSWAccount()->getMasterDomain());

        $this->assertEquals('Siteworx', $Template->toString());

        self::addSecondaryDomainIfNecessary();
        $Template->setContext(new Domain(self::SW_SECONDARY_DOMAIN));

        $this->assertEquals('Siteworx', $Template->toString());
    }

    /**
     * Set up the domain override.
     */
    public function _setupDomainOverride()
    {
        $Domain = $this->getSWAccount()->getMasterDomain();
        $dir = "{$Domain->getMyStateDir('templates_domain')}/test";
        IWorxFileSys::mkdir($dir, \IWorx\Template::PERM_FOLDER, true);
        $file = "{$dir}/test-override.tpl";
        file_put_contents($file, 'Domain');
        IWorxFileSys::chgrp('iworx', $file);
        IWorxFileSys::chmod(\IWorx\Template::PERM_FILE, $file);
    }

    /**
     * Test behavior overriding at the domain level.
     */
    public function testDomainOverride()
    {
        $this->_setupDomainOverride();

        $Template = new FileTemplate('test/test-override');

        $this->assertEquals('Global', $Template->toString());

        $Template->setContext(NodeWorx::getDefaultNodeWorx());

        $this->assertEquals('Reseller', $Template->toString());

        $Template->setContext($this->getSWAccount());

        $this->assertEquals('Siteworx', $Template->toString());

        $Template->setContext($this->getSWAccount()->getMasterDomain());

        $this->assertEquals('Domain', $Template->toString());

        self::addSecondaryDomainIfNecessary();
        $Template->setContext(new Domain(self::SW_SECONDARY_DOMAIN));

        // The secondary domain should still fall back to siteworx level
        $this->assertEquals('Siteworx', $Template->toString());
    }

    /**
     * Tests the comment.
     */
    public function testTemplateComment()
    {
        $Template = new FileTemplate('test/test-override');
        $Template->setContext($this->getSWAccount()->getMasterDomain());
        $Template->setCommentChar('#');

        // Super broad test to verify that a comment has been added.
        $this->assertContains($Template->getCommentChar(), $Template->toString());
    }
}
