<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Aug 26, 2008
 */

use \IWorx\Query;

/**
 * Unit tests for IWorxDbUtil.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorxDbUtilTest extends IWorx_Framework_TestCase
{
    private $_CurrentTestLog;

    /**
     * Provider of test data for db_quote.
     *
     * @return array
     */
    public function dbQuoteData()
    {
        return array(
                 array( "Tim's Input", "'Tim\'s Input'" ),
                 array( 0, "'0'" ),
                 array( 2000, "'2000'" ),
                 array( '0', "'0'" ),
                 array( '0123string', "'0123string'" ),
                 array( '0A34BCDEF', "'0A34BCDEF'" ),
                 array( '1e72', "'1e72'" ),
                 array( md5('paul'), "'6c63212ab48e8401eaf6b59b95d816a9'" ),
                 );
    }

    /**
     * Tests behavior of db_quote.
     *
     * @param mixed $input
     * @param mixed $expected_output
     * @dataProvider dbQuoteData
     * @group bug_972
     */
    public function testDbQuoteBehavior($input, $expected_output)
    {
        $db = IW::DB();
        $this->assertEquals($expected_output, IWorxDbUtil::db_quote($db, $input));
    }

    /**
     * Test that iworx_spam is not checked in iworxdb_tuneup.pex.
     */
    public function testGetTuneupTablesDoesNotContainIworxSpam()
    {
        $msg = 'NO NO NO.  Auto repairing of the iworx_spam database ' .
            'has caused big problems due to the apparently unbounded growth ' .
            'of the iworx_spam.awl and iworx_spam.bayes* database tables. ' .
            'Do NOT do this, unless you can guarantee that the iworx_spam ' .
            'database will not eventually contain gigabytes of data, which ' . 'is harder than you think it is.  ';
        $this->assertNotContains('iworx_spam', IWorxDbUtil::getTuneupTables(), $msg);
    }

    /**
     * Set up the world database for testing.
     *
     * @return void
     */
    public static function setUpBeforeClass() : void
    {
        parent::setUpBeforeClass();
        self::setUpWorldDatabase();
    }

    /**
     * Setup function.
     *
     * @return void
     */
    public function setUp() : void
    {
        parent::setUp();
        $LogHandler = new \Monolog\Handler\TestHandler();
        IW::getLog()->pushHandler($LogHandler);
        $this->_CurrentTestLog = $LogHandler;
    }

    /**
     * Cleanup function.
     *
     * @return void
     */
    public function tearDown() : void
    {
        IWorxConsole::echoLine('');
        IWorxConsole::echoLine('LOG OUTPUT for ' . $this->getName());
        foreach ($this->_CurrentTestLog->getRecords() as $record) {
            IWorxConsole::echoLine('LOG -> ' . $record['level_name'] . ' ' . $record['message']);
        }

        IW::getLog()->popHandler();
        $this->_CurrentTestLog = null;
        parent::tearDown();
    }

    /**
     * Dataprovider of db_* functions.
     *
     * @return array[]
     */
    public function dataProviderDbFunctions()
    {
        return [
            ['db_array'],
            ['db_string'],
            ['db_assoc'],
            ['db_object'],
            ['db_numRows'],
            ['db_getAll'],
            ['db_getCol'],
            ['db_getRow'],
            ['db_getOne'],
            ['db_query'],
        ];
    }

    /**
     * Test a good DSN.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testConnectExpectedGood()
    {
        $dsn = Ini::get(Ini::IWORX, 'dsn');
        IW::getLog()->debugL3('Test - attempting connection');
        $Db = IWorxDbUtil::db_connect($dsn);
        IW::getLog()->debugL3('Test - connected');
        IWorxDbUtil::db_disconnect($Db);

        // Defeats "risky test" detection
        $this->assertTrue(true);
    }

    /**
     * Test a bad DSN.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testConnectExpectedBad()
    {
        $this->expectException(IWorx_Exception_DbUtil::class);

        $Db = IWorxDbUtil::db_connect('garbageDSN');
    }

    /**
     * Test a bad DSN with nodie specified.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testConnectExpectedBadNoDie()
    {
        $Db = IWorxDbUtil::db_connect('garbageDSN', false);

        // Check that the logging looks like a retry happened
        $this->assertTrue(
            $this->_CurrentTestLog->hasRecordThatContains('sleeping 5 seconds', 'error')
        );
    }

    /**
     * Test connect_nodie.
     *
     * @return void
     */
    public function testConnectNoDieGood()
    {
        $dsn = Ini::get(Ini::IWORX, 'dsn');
        $Db = IWorxDbUtil::db_connect_nodie($dsn);
        IWorxDbUtil::db_disconnect($Db);

        // Defeats "risky test" detection
        $this->assertTrue(true);
    }

    /**
     * Test that nodie allows execution to continue with a bad connection.
     *
     * @return void
     */
    public function testConnectNoDieBad()
    {
        // Defeats "risky test" detection
        $this->assertTrue(true);

        $Db = IWorxDbUtil::db_connect_nodie('garbageDSN');
    }

    /**
     * Confirm that the query counts are incremented.
     *
     * @param string $function
     * @dataProvider dataProviderDbFunctions
     * @return void
     */
    public function testQueryCountIncrement($function)
    {
        IW::getLog()->debugL3("Testing $function for Increment");
        $Db  = IW::DB();
        $sql = 'SELECT Name FROM City WHERE Name = "Pittsburgh"';

        $old_query_count = IW::Env()->db_query_count;
        IWorxDbUtil::{$function}($Db, $sql);
        $new_query_count = IW::Env()->db_query_count;

        $this->assertEquals($old_query_count + 1, $new_query_count);
    }

    /**
     * Loop through functions and trigger expected exceptions.
     *
     * @param string $function
     * @dataProvider dataProviderDbFunctions
     * @return void
     */
    public function testQueryError($function)
    {
        IW::getLog()->debugL3("Testing $function for Error behavior");
        $Db  = IW::DB();
        $sql = 'SELECT Name FROM City WHERE BadColumn = "Madeup City"';

        $this->expectException(IWorx_Exception_DbUtil::class);
        $actual = IWorxDbUtil::{$function}($Db, $sql);
    }

    /**
     * Test when db_query has $die set to false.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbQueryNoDie()
    {
        $Db  = IW::DB();
        $sql = 'SELECT Name FROM City WHERE BadColumn = "Madeup City"';

        $actual = IWorxDbUtil::db_query($Db, $sql, false, false);

        // Means we made it without exploding.
        $this->assertTrue(true);
    }

    /**
     * Test db_array.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbArray()
    {
        $Db  = IW::DB();
        $sql = 'SELECT Name FROM City WHERE Name = "Pittsburgh"';

        $actual = IWorxDbUtil::db_array($Db, $sql);

        $this->assertEquals(['Pittsburgh'], $actual);
    }

    /**
     * Test db_array with a prepared statement.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbArrayPrepared()
    {
        $Db = IW::DB();
        $Q  = new Query\Select(
            'SELECT Name FROM City WHERE Name = ?',
            ['Pittsburgh']
        );

        $actual = IWorxDbUtil::db_array($Db, $Q);

        $this->assertEquals(['Pittsburgh'], $actual);
    }

    /**
     * Test db_array with no result expected.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbArrayNoResult()
    {
        $Db  = IW::DB();
        $sql = 'SELECT Name FROM City WHERE Name = "Madeup City"';

        $actual = IWorxDbUtil::db_array($Db, $sql);

        // I don't love that this is null, not an empty array, but that's
        // the existing behavior, so...here's the test.
        $this->assertEquals(null, $actual);
    }

    /**
     * Test db_string. Uses db_array internally, which is why that's first.
     *
     * @return void
     */
    public function testDbString()
    {
        $Db  = IW::DB();
        $sql = 'SELECT Name FROM City WHERE Name = "Pittsburgh"';

        $actual = IWorxDbUtil::db_string($Db, $sql);

        $this->assertEquals('Pittsburgh', $actual);
    }

    /**
     * Test db_string with a Query object.
     *
     * @return void
     */
    public function testDbString2()
    {
        $Db = IW::DB();
        $Q  = new Query\Select('SELECT Name FROM City WHERE Name = "Pittsburgh"');

        $actual = IWorxDbUtil::db_string($Db, $Q);

        $this->assertEquals('Pittsburgh', $actual);
    }

    /**
     * Test db_string with a prepared statement.
     *
     * @return void
     */
    public function testDbStringPrepared()
    {
        $Db = IW::DB();
        $Q  = new Query\Select(
            'SELECT Name FROM City WHERE Name = ?',
            ['Pittsburgh']
        );

        $actual = IWorxDbUtil::db_string($Db, $Q);

        $this->assertEquals('Pittsburgh', $actual);
    }

    /**
     * Test db_string with no result.
     *
     * @return void
     */
    public function testDbStringNoResult()
    {
        $Db  = IW::DB();
        $sql = 'SELECT Name FROM City WHERE Name = "Madeup City"';

        $actual = IWorxDbUtil::db_string($Db, $sql);

        $this->assertEquals('', $actual);
    }

    /**
     * Verify the return type of db_string.
     *
     * Matters for places that do a count(*) and then === '4', etc.
     *
     * @return void
     */
    public function testDbStringReturnsString()
    {
        $Db  = IW::DB();
        $sql = 'SELECT count(*) FROM City';

        $actual = IWorxDbUtil::db_string($Db, $sql);

        $this->assertEquals('4079', $actual);
    }

    /**
     * Test db_object.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbObject()
    {
        $Db  = IW::DB();
        $sql = 'SELECT Name FROM City WHERE Name = "Pittsburgh"';

        $expected = (object) ['Name' => 'Pittsburgh'];
        $actual   = IWorxDbUtil::db_object($Db, $sql);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test db_object with a prepared statement.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbObjectPrepared()
    {
        $Q  = new Query\Select(
            'SELECT Name FROM City WHERE Name = ?',
            ['Pittsburgh']
        );

        $expected = (object) ['Name' => 'Pittsburgh'];
        $actual   = IWorxDbUtil::db_object(IW::DB(), $Q);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test db_object with no result.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbObjectNoResult()
    {
        $Db  = IW::DB();
        $sql = 'SELECT Name FROM City WHERE Name = "Madeup City"';

        $expected = false;
        $actual   = IWorxDbUtil::db_object($Db, $sql);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test that the query count is incremented.
     *
     * @return void
     */
    public function testDbObjectQueryCount()
    {
        $Db  = IW::DB();
        $sql = 'SELECT Name FROM City WHERE Name = "Pittsburgh"';

        $old_query_count = IW::Env()->db_query_count;
        IWorxDbUtil::db_object($Db, $sql);
        $new_query_count = IW::Env()->db_query_count;

        $this->assertEquals($old_query_count + 1, $new_query_count);
    }

    /**
     * Test db_numRows.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbNumRows()
    {
        $Db  = IW::DB();
        $sql = 'SELECT Name FROM City WHERE Name = "Pittsburgh"';

        $expected = 1;
        $actual   = IWorxDbUtil::db_numRows($Db, $sql);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test db_numRows.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbNumRows2()
    {
        $Db  = IW::DB();
        $sql = 'SELECT Name FROM City LIMIT 10';

        $expected = 10;
        $actual   = IWorxDbUtil::db_numRows($Db, $sql);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test db_numRows with no result.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbNumRowsNoResult()
    {
        $Db  = IW::DB();
        $sql = 'SELECT Name FROM City WHERE Name = "Madeup City"';

        $expected = 0;
        $actual   = IWorxDbUtil::db_numRows($Db, $sql);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test db_getAll.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetAllDefaultFetchmode()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City LIMIT 10';

        $expected = (object) [
            'ID'   => '1',
            'Name' => 'Kabul',
        ];

        $actual   = IWorxDbUtil::db_getAll($Db, $sql);

        $this->assertEquals($expected, $actual[0]);
    }

    /**
     * Test db_getAll with prepared statement.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetAllPrepared()
    {
        $Db = IW::DB();
        $Q  = new Query\Select(
            'SELECT ID, Name FROM City LIMIT ?',
            [10]
        );

        $expected = (object) [
            'ID'   => '1',
            'Name' => 'Kabul',
        ];

        $actual   = IWorxDbUtil::db_getAll($Db, $Q);

        $this->assertEquals($expected, $actual[0]);
    }

    /**
     * Test db_getAll.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetAllFetchmodeObject()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City LIMIT 10';

        $expected = (object) [
            'ID'   => '1',
            'Name' => 'Kabul',
        ];

        $actual   = IWorxDbUtil::db_getAll($Db, $sql, DB_FETCHMODE_OBJECT);

        $this->assertEquals($expected, $actual[0]);
    }

    /**
     * Test db_getAll.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetAllFetchmodeAssoc()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City LIMIT 10';

        $expected = [
            'ID'   => '1',
            'Name' => 'Kabul',
        ];

        $actual   = IWorxDbUtil::db_getAll($Db, $sql, DB_FETCHMODE_ASSOC);

        $this->assertEquals($expected, $actual[0]);
    }

    /**
     * Test db_getAll.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetAllFetchmodeOrdered()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City LIMIT 10';

        $expected = [
            0 => '1',
            1 => 'Kabul',
        ];

        $actual   = IWorxDbUtil::db_getAll($Db, $sql, DB_FETCHMODE_ORDERED);

        $this->assertEquals($expected, $actual[0]);
    }

    /**
     * Test db_getCol.
     *
     * There is an argument for column offset, but we don't use it, so it's
     * not being tested.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetCol()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City LIMIT 10';

        $expected = [
            '1','2','3','4','5','6','7','8','9','10',
        ];

        $actual   = IWorxDbUtil::db_getCol($Db, $sql);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test db_getCol with prepared statements.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetColPrepared()
    {
        $Db = IW::DB();
        $Q  = new Query\Select(
            'SELECT ID, Name FROM City WHERE ID <= ?',
            [10]
        );

        $expected = [
            '1','2','3','4','5','6','7','8','9','10',
        ];

        $actual   = IWorxDbUtil::db_getCol($Db, $Q);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test db_getAll.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetRowDefaultFetchmode()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City WHERE Name = "Pittsburgh"';

        $expected = (object) [
            'ID'   => '3844',
            'Name' => 'Pittsburgh',
        ];

        $actual = IWorxDbUtil::db_getRow($Db, $sql);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test db_getRow with prepared statement.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetRowPrepared()
    {
        $Db = IW::DB();
        $Q  = new Query\Select(
            'SELECT ID, Name FROM City WHERE Name = ?',
            ['Pittsburgh']
        );

        $expected = (object) [
            'ID'   => '3844',
            'Name' => 'Pittsburgh',
        ];

        $actual = IWorxDbUtil::db_getRow($Db, $Q);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test db_getRow.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetRowFetchmodeObject()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City WHERE Name = "Pittsburgh"';

        $expected = (object) [
            'ID'   => '3844',
            'Name' => 'Pittsburgh',
        ];

        $actual = IWorxDbUtil::db_getRow($Db, $sql, DB_FETCHMODE_OBJECT);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test db_getRow.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetRowFetchmodeAssoc()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City WHERE Name = "Pittsburgh"';

        $expected = [
            'ID'   => '3844',
            'Name' => 'Pittsburgh',
        ];

        $actual   = IWorxDbUtil::db_getRow($Db, $sql, DB_FETCHMODE_ASSOC);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test db_getRow.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetRowFetchmodeOrdered()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City WHERE Name = "Pittsburgh"';

        $expected = [
            0 => '3844',
            1 => 'Pittsburgh',
        ];

        $actual   = IWorxDbUtil::db_getRow($Db, $sql, DB_FETCHMODE_ORDERED);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test db_getOne.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetOne()
    {
        $Db  = IW::DB();
        $sql = 'SELECT count(*) FROM City';

        $actual = IWorxDbUtil::db_getOne($Db, $sql);

        $this->assertEquals('4079', $actual);
    }

    /**
     * Test db_getOne with prepared statements.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbGetOnePrepared()
    {
        $Db = IW::DB();
        $Q  = new Query\Select(
            'SELECT count(*) FROM City WHERE ID <= ?',
            [10]
        );

        $actual = IWorxDbUtil::db_getOne($Db, $Q);

        $this->assertEquals('10', $actual);
    }

    /**
     * Test db_affectedRows.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbAffectedRows()
    {
        $Db  = IW::DB();
        $sql = 'UPDATE City SET Population = 250000 WHERE Name = "Pittsburgh"';

        IWorxDbUtil::db_query($Db, $sql);

        $this->assertEquals(1, IWorxDbUtil::db_affectedRows($Db));
    }

    /**
     * Test that db_lastError is false with no errors.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbLastErrorWithoutError()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City WHERE Name = "Pittsburgh"';
        IWorxDbUtil::db_query($Db, $sql);

        $this->assertFalse(IWorxDbUtil::db_lastError());
    }

    /**
     * Test that db_lastError populates.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbLastErrorWithError()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City WHERE NameBROKEN = "Pittsburgh"';
        IWorxDbUtil::db_query($Db, $sql, false, false);

        $error = IWorxDbUtil::db_lastError();

        $this->assertNotFalse($error);
        $this->assertTrue(str_contains($error, 'Unknown column'));
    }

    /**
     * Test db_lastInsertId.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbLastInsertId()
    {
        $Db  = IW::DB();
        $sql = 'INSERT INTO City SET Name = "InterWorxVille"';
        IWorxDbUtil::db_query($Db, $sql);

        $this->assertEquals(4080, IWorxDbUtil::db_lastInsertId($Db));
    }

    /**
     * Test using fetchRow on the result from db_query.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbQueryFetchRowDefault()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City WHERE Name = "Pittsburgh"';

        $expected = [
            0 => '3844',
            1 => 'Pittsburgh',
        ];

        $result = IWorxDbUtil::db_query($Db, $sql);
        $actual = $result->fetchRow();

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test using fetchRow on the result from db_query.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbQueryFetchRowOrdered()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City WHERE Name = "Pittsburgh"';

        $expected = [
            0 => '3844',
            1 => 'Pittsburgh',
        ];

        $result = IWorxDbUtil::db_query($Db, $sql);
        $actual = $result->fetchRow(DB_FETCHMODE_ORDERED);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test using fetchRow on the result from db_query.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbQueryFetchRowAssoc()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City WHERE Name = "Pittsburgh"';

        $expected = [
            'ID'   => '3844',
            'Name' => 'Pittsburgh',
        ];

        $result = IWorxDbUtil::db_query($Db, $sql);
        $actual = $result->fetchRow(DB_FETCHMODE_ASSOC);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test using fetchRow on the result from db_query.
     *
     * @return void
     * @throws IWorx_Exception_DbUtil
     */
    public function testDbQueryFetchRowObject()
    {
        $Db  = IW::DB();
        $sql = 'SELECT ID, Name FROM City WHERE Name = "Pittsburgh"';

        $expected = (object) [
            'ID'   => '3844',
            'Name' => 'Pittsburgh',
        ];

        $result = IWorxDbUtil::db_query($Db, $sql);
        $actual = $result->fetchRow(DB_FETCHMODE_OBJECT);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Test db_columnExists.
     *
     * @return void
     */
    public function testDbColumnExists()
    {
        $Db = IW::DB();
        $actual = IWorxDbUtil::db_columnExists($Db, 'iworx', 'City', 'Name');

        $this->assertTrue($actual);
    }
}
