<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Nov 7, 2008
 */

/**
 * Tests a few things with IWorxTranslate - only in english at the moment.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class IWorxTranslateTest extends \PHPUnit\Framework\TestCase
{
    /**
     * Spot checks a few translations that should always exist.
     */
    public function testHasTranslation()
    {
        $this->assertTrue(
            IWorxTranslate::hasTranslation('##LG_YES##'),
            'Translate did not exist for ##LG_YES##'
        );
        $this->assertTrue(
            IWorxTranslate::hasTranslation('LG_YES'),
            'Translate did not exist for LG_YES'
        );

        $this->assertTrue(
            IWorxTranslate::hasTranslation('##LG_NO##'),
            'Translate did not exist for ##LG_NO##'
        );
        $this->assertTrue(
            IWorxTranslate::hasTranslation('LG_NO'),
            'Translate did not exist for LG_NO'
        );
    }

    /**
     * Tests converting literal newlines to actual newlines.
     */
    public function testNewlines()
    {
        $this->assertTrue(
            IWorxTranslate::hasTranslation('LG_TEST_NEWLINES'),
            'Missing language key to test with.'
        );

        $actual   = IWorxTranslate::tr('##LG_TEST_NEWLINES##');
        $expected = "This language key has a newline.\nIt is used in the test suite.";

        $this->assertEquals($expected, $actual);
    }

    /**
     * Tests that hasTranslation works when substituting variables.
     */
    public function testHasTranslationWithVariables()
    {
        $this->assertTrue(IWorxTranslate::hasTranslation('##LG_YES|Whatever##'));
    }

    /**
     * Tests that hasTranslation() still works with the escape flag.
     */
    public function testHasTranslationEscapeFlag()
    {
        $this->assertTrue(IWorxTranslate::hasTranslation('##\LG_YES##'));
    }

    /**
     * Tests a few translations that should always exist.
     */
    public function testTr()
    {
        $this->assertEquals('yes', strtolower(IWorxTranslate::tr('##LG_YES##')));
        $this->assertEquals('no', strtolower(IWorxTranslate::tr('##LG_NO##')));
    }

    /**
     * Tests multiple translated strings in the same pass.
     */
    public function testMultipleTr()
    {
        $actual = strtolower(IWorxTranslate::tr('##LG_YES####LG_NO##'));

        $this->assertEquals('yesno', $actual);
    }

    /**
     * Tests that keys without variables continue to work if variabless are provided.
     */
    public function testTrWithoutVariables()
    {
        $this->assertEquals('no', strtolower(IWorxTranslate::tr('##LG_NO|25##')));
    }

    /**
     * Tests keyword substitution in lang entries.
     */
    public function testTrWithVariables()
    {
        $key      = '##LG_SPRINTF_TEST_KEY|bullshits|20##';
        $expected = "Test key bullshits,20 do not delete! Apostrophe's are hard.";
        $actual   = IWorxTranslate::tr($key);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Tests keyword substitution in lang entries with new allowed characters.
     */
    public function testTrWithVariables2()
    {
        $key      = '##LG_SPRINTF_TEST_KEY|bull.shits|2##';
        $expected = "Test key bull.shits,2 do not delete! Apostrophe's are hard.";
        $actual   = IWorxTranslate::tr($key);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Tests that single quotes are escaped when requested.
     */
    public function testTrEscape()
    {
        $key      = '##LG_SPRINTF_TEST_KEY|bull.shits|2##';
        $expected = 'Test key bull.shits,2 do not delete! Apostrophe\\\'s are hard.';
        $actual   = IWorxTranslate::tr($key, true);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Tests the escape flag syntax.
     */
    public function testTrEscapeTag()
    {
        $key      = '##\LG_SPRINTF_TEST_KEY|bull.shits|2##';
        $expected = 'Test key bull.shits,2 do not delete! Apostrophe\\\'s are hard.';
        $actual   = IWorxTranslate::tr($key);

        $this->assertEquals($expected, $actual);
    }

    /**
     * Simple test for translateMixed.
     */
    public function testTranslateMixedString()
    {
        $key = '##LG_YES##';

        $actual = strtolower(IWorxTranslate::translateMixed($key));

        $this->assertEquals('yes', $actual);
    }

    /**
     * Simple test for translateMixed with an array.
     */
    public function testTranslateMixedArray()
    {
        $key = '##LG_YES##';

        $data = array( $key, $key );

        $actual = IWorxTranslate::translateMixed($data);

        $this->assertEquals(array( 'Yes', 'Yes' ), $actual);
    }

    /**
     * Simple test for translateMixed with an object.
     */
    public function testTranslateMixedObject()
    {
        $key = '##LG_YES##';

        $data = (object) null;

        $data->one = $key;
        $data->two = $key;

        $actual = IWorxTranslate::translateMixed($data);

        $expected = (object) null;

        $expected->one = 'Yes';
        $expected->two = 'Yes';

        $this->assertEquals($expected, $actual);
    }

    /**
     * Simple test for translateMixed with an array of arrays.
     */
    public function testTranslateMixedArrayOfArrays()
    {
        $key = '##LG_YES##';

        $data = array( $key, $key );

        $array = array( $data, $data, $data );

        $actual = IWorxTranslate::translateMixed($array);

        $expected = array( 'Yes', 'Yes' );
        $expected_array = array( $expected, $expected, $expected );

        $this->assertEquals($expected_array, $actual);
    }

    /**
     * Simple test for translateMixed with an array of objects.
     */
    public function testTranslateMixedArrayOfObjects()
    {
        $key = '##LG_YES##';

        $data = (object) null;

        $data->one = $key;
        $data->two = $key;

        $array = array( $data, $data, $data );

        $actual = IWorxTranslate::translateMixed($array);

        $expected = (object) null;

        $expected->one = 'Yes';
        $expected->two = 'Yes';

        $expected_array = array( $expected, $expected, $expected );

        $this->assertEquals($expected_array, $actual);
    }

    /**
     * Simple test for translateMixed with an object with an array of objects.
     */
    public function testTranslateMixedObjectWithArrayOfObjects()
    {
        $key = '##LG_YES##';

        $data = (object) null;

        $data->one = $key;
        $data->two = $key;

        $array = array( $data, $data, $data );

        $object = (object) null;

        $object->data = $array;
        $object->no = '##LG_NO##';

        $actual = IWorxTranslate::translateMixed($object);

        $expected = (object) null;

        $expected->one = 'Yes';
        $expected->two = 'Yes';

        $expected_array = array( $expected, $expected, $expected );

        $expected_object = (object) null;

        $expected_object->data = $expected_array;
        $expected_object->no   = 'No';

        $this->assertEquals($expected_object, $actual);
    }
}
