<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Tim College <tcollege@interworx.com>
 * @date    Nov 3, 2008
 */

/**
 * Tests for Input_Integer.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Input_IntegerTest extends \PHPUnit\Framework\TestCase
{
    /**
     * Data provider.
     *
     * @return array
     */
    public function dataProviderGoodRanges()
    {
        return array( array( 10, 20 ),
                  array( 10, 10 ),
                  array( 0, 63535 ),
                  array( -30 ),
                  array( -900000, 0 ) );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function dataProviderBadRanges()
    {
        return array( array( 10, 0 ),
                  array( 100, -100 ),
                  array( 0, 'Tim' ),
                  array( null ),
                  array( 10.1, 10.2 ) );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function dataProviderGoodData()
    {
        $max     = Ini::get(Ini::CONSTANTS, 'unlimited');
        $monster = MySQL::MAX_CONFIG_BYTES;

        // array with three values - min, max, actual
        return array( array( 0, null, 5 ),
                  array( 0, 0, 0 ),
                  array( 10, 10, 10 ),
                  array( 10, 10, '10' ),
                  array( -5, 5, 0 ),
                  array( -5, 5, '0' ),
                  array( 0, $max, 25 ),
                  array( 1, $monster, 456789 ) );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function dataProviderBadData()
    {
        // array with three values - min, max, actual
        return array( array( 0, null, -1 ),
                  array( 0, 0, 5 ),
                  array( 0, 0, -1 ),
                  array( 10, 10, 11 ),
                  array( -5, 5, 1.1 ),
                  array( 5, -1235, '0' ) );
    }

    /**
     * Tests the default values.
     */
    public function testDefaultValues()
    {
        $I = new Input_Integer('name');

        $this->assertFalse($I->hasErrors(), 'New Input_Integer had error! WTF?');

        $this->assertEquals(0, $I->getRangeMin());
        $this->assertNull($I->getRangeMax());
    }

    /**
     * Tests setting good ranges.
     *
     * @param integer $min
     * @param integer $max
     * @dataProvider dataProviderGoodRanges
     */
    public function testSetRangeWithGoodData($min, $max = null)
    {
        $I = new Input_Integer('name');

        $I->setRange($min, $max);

        $this->assertFalse($I->hasErrors(), implode("\n", $I->getErrors()));
        $this->assertEquals($min, $I->getRangeMin());
        $this->assertEquals($max, $I->getRangeMax());
    }

    /**
     * Tests setting bad ranges.
     *
     * @param integer $min
     * @param integer $max
     * @dataProvider dataProviderBadRanges
     */
    public function testSetRangeWithBadData($min, $max = null)
    {
        $I = new Input_Integer('name');

        $I->setRange($min, $max);

        $this->assertTrue($I->hasErrors(), 'Setting an invalid range did not error');
        $this->assertEquals(0, $I->getRangeMin(), 'Min range changed');
        $this->assertNull($I->getRangeMax(), 'Max range changed');
    }

    /**
     * Tests validate behavior with good data.
     *
     * @param integer        $min
     * @param integer        $max
     * @param integer|string $actual
     * @dataProvider dataProviderGoodData
     */
    public function testValidateWithGoodData($min, $max, $actual)
    {
        $I = new Input_Integer('name');

        $I->setRange($min, $max);
        $I->setCurrentValue($actual);

        $is_valid = $I->validate();

        $this->assertTrue($is_valid);
        $this->assertFalse($I->hasErrors());
    }

    /**
     * Tests validate behavior with bad data.
     *
     * @param integer        $min
     * @param integer        $max
     * @param integer|string $actual
     * @dataProvider dataProviderBadData
     */
    public function testValidateWithBadData($min, $max, $actual)
    {
        $I = new Input_Integer('name');

        $I->setRange($min, $max);
        $I->setCurrentValue($actual);

        $this->assertFalse($I->validate());
        $this->assertTrue($I->hasErrors());
    }
}
