<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Aug 21, 2008
 */

/**
 * Test Suite for JSON Daemon.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class JSON_DaemonTest extends \PHPUnit\Framework\TestCase
{
    /**
     * Data Provider.
     *
     * @return array
     */
    public static function badResponders()
    {
        return array(
                 array( '_StartsWithUnderscore' ),
                 array( 'ContainsNumbers123' ),
                 array( 'InvalidBullshits!@#)(' ),
                 array( 'OtherwiseValidButDoesNotExist' ),
                 );
    }

    /**
     * Array of good responders.  These must actually exist.
     *
     * @return array
     */
    public static function goodResponders()
    {
        return array(
                 array( 'YumLastLog' ),
                 );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public static function badInput()
    {
        return array(
                 array( json_encode('') ),
                 array( json_encode('25') ),
                 array( json_encode(array( 1,2,3,4,5 )) ),
                 // Missing quotes around r, therefore malformed
                 array( 'r="YumLastLog"' ),
                 );
    }

    /**
     * Data provider.
     *
     * @return array
     */
    public function goodInputUnencoded()
    {
        $input_1    = ( object ) null;
        $input_1->r = 'YumLastLog';

        $input_2    = ( object ) null;
        $input_2->r = 'YumLastLog';
        $input_2->i = 12;

        $input_3    = ( object ) null;
        $input_3->r = 'SomeResponderName';
        $input_3->i = array( 1,2,3,4,5 );

        $input_4    = ( object ) null;
        $input_4->r = 'YumLastLog';
        $input_4->i = $input_2;

        $input_5    = ( object ) null;
        $input_5->r = 'YumLastLog';
        $input_5->i = 'A String';

        return array(
                 array( $input_1 ),
                 array( $input_2 ),
                 array( $input_3 ),
                 array( $input_4 ),
                 array( $input_5 ),
                 );
    }

    /**
     * Tests that bogus names aren't allowed.
     *
     * @param string $name Name of the bad responder
     * @dataProvider badResponders
     * @expectedException IWorx_Exception_JSON_Reply
     */
    public function testValidateResponderThrowsExceptionForBadNames($name)
    {
        JSON_Daemon::validateResponder($name);
    }

    /**
     * Tests that good names are allowed.
     *
     * @param string $name
     * @dataProvider goodResponders
     */
    public function testValidateResponderReturnsTrueForGoodNames($name)
    {
        $this->assertTrue(JSON_Daemon::validateResponder($name));
    }

    /**
     * Tests input behavior.
     *
     * @param string $input
     * @dataProvider badInput
     * @expectedException IWorx_Exception_JSON_Reply
     */
    public function testGetInputThrowsExceptionForInvalidInput($input)
    {
        // set the get
        $_GET['i'] = $input;
        try {
            JSON_Daemon::getInput();
        } catch (IWorx_Exception_JSON_Reply $e) {
            // Verify the reply code
            $this->assertEquals(
                JSON_Reply::CODE_CLIENT_MALFORMED_REQUEST,
                $e->getCode()
            );
            // rethrow the exception to prove we have it
            throw $e;
        }
    }

    /**
     * Tests input behavior.
     *
     * @param object $input
     * @dataProvider goodInputUnencoded
     */
    public function testGetInputReturnsObjectForValidInput($input)
    {
        $json = json_encode($input);

        // set the get
        $_GET['i'] = $json;

        $result = JSON_Daemon::getInput();

        $this->assertInternalType(
            'object',
            $result,
            'getInput() did not return object'
        );
        $this->assertEquals($input->r, $result->r, '->r was unexpected value');
        if (isset($input->i)) {
            $this->assertEquals($input->i, $result->i, 'Result i was unexpected');
        }
    }
}
