<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Aug 13, 2008
 */

/**
 * JSON_Reply class Tester.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class JSON_ReplyTest extends \PHPUnit\Framework\TestCase
{
    /**
     * Verifies output of JSON_Reply when called with no defaults.
     */
    public function testConstructorWithNoArguments()
    {
        $Reply = new JSON_Reply();
        ob_start();
        $Reply->send();
        $sent_reply = ob_get_clean();

        $decoded_reply = json_decode($sent_reply);

        $this->assertInternalType('object', $decoded_reply, 'Unable to decode reply!');
        $this->assertEquals(
            JSON_Reply::CODE_OK,
            $decoded_reply->c,
            'replyCode value was unexpected'
        );
        $this->assertEquals(
            'Ok',
            $decoded_reply->t,
            'replyText value was unexpected'
        );
    }

    /**
     * Verifies that reply output is translated.
     */
    public function testWellFormedCustomReplyTranslation()
    {
        $message = '##LG_YES##';
        $Reply   = new JSON_Reply(JSON_Reply::CODE_OK_CUSTOM, $message);
        ob_start();
        $Reply->send();
        $sent_reply = ob_get_clean();

        $decoded_reply = json_decode($sent_reply);

        $this->assertInternalType('object', $decoded_reply, 'Unable to decode reply!');
        $this->assertEquals(
            JSON_Reply::CODE_OK_CUSTOM,
            $decoded_reply->c,
            'replyCode value was unexpected'
        );
        $this->assertEquals(
            'Yes',
            $decoded_reply->t,
            'replyText value was unexpected'
        );
    }

    /**
     * Verifies that the output is decodable JSON, and that input = output.
     */
    public function testWellFormedCustomReply()
    {
        $message = 'Look, a custom message!';
        $Reply   = new JSON_Reply(JSON_Reply::CODE_OK_CUSTOM, $message);
        ob_start();
        $Reply->send();
        $sent_reply = ob_get_clean();

        $decoded_reply = json_decode($sent_reply);

        $this->assertInternalType('object', $decoded_reply, 'Unable to decode reply!');
        $this->assertEquals(
            JSON_Reply::CODE_OK_CUSTOM,
            $decoded_reply->c,
            'replyCode value was unexpected'
        );
        $this->assertEquals(
            $message,
            $decoded_reply->t,
            'replyText value was unexpected'
        );
    }

    /**
     * Tests that a code that has a built-in default is not overridden.
     */
    public function testDefaultCodeOverridesCustomText()
    {
        $Reply = new JSON_Reply(JSON_Reply::CODE_OK, 'Custom text that should not be');
        ob_start();
        $Reply->send();
        $sent_reply = ob_get_clean();

        $decoded_reply = json_decode($sent_reply);

        $this->assertInternalType('object', $decoded_reply, 'Unable to decode reply!');
        $this->assertEquals(
            JSON_Reply::CODE_OK,
            $decoded_reply->c,
            'replyCode value was unexpected'
        );
        $this->assertEquals(
            'Ok',
            $decoded_reply->t,
            'replyText value was unexpected'
        );
    }

    /**
     * Verifies that JSON_Reply's cannot be created with invalid codes.
     */
    public function testReplyWithOutOfRangeLowReplyCodeThrows501Exception()
    {
        try {
            // Out of range low
            $Reply = new JSON_Reply(100, 'Bogus Reply');
        } catch (IWorx_Exception_JSON_Reply $e) {
            $this->assertEquals(JSON_Reply::CODE_SERVER_MALFORMED_REPLY, $e->getCode());
        }
    }

    /**
     * Verifies that JSON_Reply's cannot be created with invalid codes.
     */
    public function testReplyWithOutOfRange300ReplyCodeThrows501Exception()
    {
        try {
            // Out of range low
            $Reply = new JSON_Reply(305, 'Bogus Reply');
        } catch (IWorx_Exception_JSON_Reply $e) {
            $this->assertEquals(JSON_Reply::CODE_SERVER_MALFORMED_REPLY, $e->getCode());
        }
    }

    /**
     * Verifies that JSON_Reply's cannot be created with invalid codes.
     */
    public function testReplyWithOutOfRangeHighReplyCodeThrows501Exception()
    {
        try {
            // Out of range low
            $Reply = new JSON_Reply(1000, 'Bogus Reply');
        } catch (IWorx_Exception_JSON_Reply $e) {
            $this->assertEquals(JSON_Reply::CODE_SERVER_MALFORMED_REPLY, $e->getCode());
        }
    }

    /**
     * Verifies that a 201 actually includes output data as expected.
     */
    public function test201WithDataWorks()
    {
        $Reply = new JSON_Reply(JSON_Reply::CODE_OK_WITH_DATA, '', 'Data');
        ob_start();
        $Reply->send();
        $sent_reply = ob_get_clean();

        $decoded_reply = json_decode($sent_reply);

        $this->assertInternalType('object', $decoded_reply, 'Unable to decode reply!');
        $this->assertEquals(
            JSON_Reply::CODE_OK_WITH_DATA,
            $decoded_reply->c,
            'replyCode value was unexpected'
        );
        $this->assertEquals(
            'Data Follows',
            $decoded_reply->t,
            'replyText value was unexpected'
        );
        $this->assertEquals('Data', $decoded_reply->d);
    }

    /**
     * Tests translation of data parameter.
     */
    public function test201WithDataTranslation()
    {
        $Reply = new JSON_Reply(JSON_Reply::CODE_OK_WITH_DATA, '', '##LG_YES##');
        ob_start();
        $Reply->send();
        $sent_reply = ob_get_clean();

        $decoded_reply = json_decode($sent_reply);

        $this->assertInternalType('object', $decoded_reply, 'Unable to decode reply!');
        $this->assertEquals(
            JSON_Reply::CODE_OK_WITH_DATA,
            $decoded_reply->c,
            'replyCode value was unexpected'
        );
        $this->assertEquals(
            'Data Follows',
            $decoded_reply->t,
            'replyText value was unexpected'
        );
        $this->assertEquals('Yes', $decoded_reply->d);
    }

    /**
     * Tests translation of nested data parameter.
     */
    public function test201WithComplexDataTranslation()
    {
        $object      = ( object ) null;
        $object->no  = '##LG_NO##';
        $object->yes = '##LG_YES##';
        $Reply       = new JSON_Reply(
            JSON_Reply::CODE_OK_WITH_DATA,
            '',
            array('##LG_YES##', $object )
        );
        ob_start();
        $Reply->send();
        $sent_reply = ob_get_clean();

        $decoded_reply = json_decode($sent_reply);

        $this->assertInternalType('object', $decoded_reply, 'Unable to decode reply!');
        $this->assertEquals(
            JSON_Reply::CODE_OK_WITH_DATA,
            $decoded_reply->c,
            'replyCode value was unexpected'
        );
        $this->assertEquals(
            'Data Follows',
            $decoded_reply->t,
            'replyText value was unexpected'
        );
        $this->assertEquals('Yes', $decoded_reply->d[ 0 ]);
        $this->assertEquals('No', $decoded_reply->d[ 1 ]->no);
    }

    /**
     * Verifies that 201 requires data.
     */
    public function test201WithoutDataThrows501Exception()
    {
        try {
            $Reply = new JSON_Reply(JSON_Reply::CODE_OK_WITH_DATA, 'No Data');
        } catch (IWorx_Exception_JSON_Reply $e) {
            $this->assertEquals(JSON_Reply::CODE_SERVER_MALFORMED_REPLY, $e->getCode());
        }
    }

    /**
     * Verifies that 202 requires custom message.
     */
    public function test202WithoutCustomMessageThrows501Exception()
    {
        try {
            $Reply = new JSON_Reply(JSON_Reply::CODE_OK_CUSTOM);
        } catch (IWorx_Exception_JSON_Reply $e) {
            $this->assertEquals(JSON_Reply::CODE_SERVER_MALFORMED_REPLY, $e->getCode());
        }
    }
}
