<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Jonathan Daniel <jdaniel@interworx.com>
 * @date    Mar 4, 2011
 */

/**
 * Tests the Nodeworx_User Model functionality.
 *
 * @package    IntertWorx
 * @subpackage Tests
 */
class Model_Nodeworx_UserTest extends IWorx_Framework_TestCase
{

    const TEST_EMAIL        = 'unittest@unittest.email.example.com';
    const TEST_MASTER_EMAIL = 'test.master.nw.user@unittest.email.example.com';

    /**
     * Tests that the create function returns the correct object.
     */
    public function testCreate()
    {
        $User = Model_Nodeworx_User::create();

        $this->assertTrue($User instanceof Model_Nodeworx_User);
    }

    /**
     * Delete all the test data.
     */
    private static function _deleteTestData()
    {
        $q_test_email        = q(self::TEST_EMAIL);
        $q_test_master_email = q(self::TEST_MASTER_EMAIL);

        $sql   = "email = {$q_test_email} OR email = {$q_test_master_email}";
        $users = Model_Nodeworx_User::find($sql);

        foreach ($users as $User) {
            $User->delete();
        }
    }

    /**
     * Set up before class.
     */
    public static function setUpBeforeClass()
    {
        self::_deleteTestData();
    }

    /**
     * Tear down after class.
     */
    public static function tearDownAfterClass()
    {
        self::_deleteTestData();
    }

    /**
     * Test building the user.
     */
    public function testBuildingUser()
    {
        $User = Model_Nodeworx_User::create();

        $nickname   = 'UNIT TEST USER';
        $email      = self::TEST_EMAIL;
        $password   = IWorxUtil::hashWithSalt('abc123');
        $global_uid = Model_User::PENDING;
        $theme      = Ini::get(Ini::IWORX_TEMPLATES, 'nodeworx');
        $language   = Ini::get(Ini::IWORX_LANGUAGE, 'nodeworx');
        $status     = Model_Nodeworx_User::INACTIVE;
        $type       = Model_Nodeworx_User::SLAVE;
        $menu_style = Model_Nodeworx_User::MENU_SMALL;

        $User->nickname   = $nickname;
        $User->global_uid = $global_uid;
        $User->email      = $email;
        $User->password   = $password;
        $User->theme      = $theme;
        $User->language   = $language;
        $User->status     = $status;
        $User->type       = $type;
        $User->menu_style = $menu_style;

        $User->save();

        $this->assertEquals($nickname, $User->nickname);
        $this->assertEquals($email, $User->email);
        $this->assertEquals($password, $User->password);
        $this->assertEquals($theme, $User->theme);
        $this->assertEquals($language, $User->language);
        $this->assertEquals($status, $User->status);
        $this->assertEquals($type, $User->type);
        $this->assertEquals($menu_style, $User->menu_style);

        $this->assertNotEquals($global_uid, $User->global_uid);
    }

    /**
     * Tests the get by email function.
     */
    public function testGetByEmail()
    {
        $User = Model_Nodeworx_User::getByEmail(self::TEST_EMAIL);

        $nickname   = 'UNIT TEST USER';
        $email      = self::TEST_EMAIL;
        $global_uid = Model_User::PENDING;
        $theme      = Ini::get(Ini::IWORX_TEMPLATES, 'nodeworx');
        $language   = Ini::get(Ini::IWORX_LANGUAGE, 'nodeworx');
        $status     = Model_Nodeworx_User::INACTIVE;
        $type       = Model_Nodeworx_User::SLAVE;
        $menu_style = Model_Nodeworx_User::MENU_SMALL;

        $this->assertEquals($nickname, $User->nickname);
        $this->assertEquals($email, $User->email);
        $this->assertEquals($theme, $User->theme);
        $this->assertEquals($language, $User->language);
        $this->assertEquals($status, $User->status);
        $this->assertEquals($type, $User->type);
        $this->assertEquals($menu_style, $User->menu_style);
    }

    /**
     * Tests that the user's initial permissions are empty.
     */
    public function testInitialPermissionsAreEmpty()
    {
        $User  = Model_Nodeworx_User::getByEmail(self::TEST_EMAIL);
        $perms = $User->getPermissions();

        $this->assertTrue(empty($perms));
    }

    /**
     * Test deactivate.
     */
    public function testDeactivate()
    {
        $User = Model_Nodeworx_User::getByEmail(self::TEST_EMAIL);

        $User->deactivate();

        $perms = $User->getPermissions();

        $this->assertEquals(Model_User::INACTIVE, $User->status);
        $this->assertArrayNotHasKey(Session_NodeWorx::PERM_LOGIN, $perms);
    }

    /**
     * Test activate.
     */
    public function testActivate()
    {
        $User = Model_Nodeworx_User::getByEmail(self::TEST_EMAIL);

        $User->activate();

        $perms = $User->getPermissions();

        $this->assertEquals(Model_User::ACTIVE, $User->status);
        $this->assertArrayHasKey(Session_NodeWorx::PERM_LOGIN, $perms);
    }

    /**
     * Test setting reseller permissions.
     */
    public function testSettingResellerPermissions()
    {
        $User = Model_Nodeworx_User::getByEmail(self::TEST_EMAIL);

        $User->setResellerPermissions();
        $User->save();

        $perm_names = NodeWorx::getResellerPerms();
        $perms      = $User->getPermissions();

        foreach ($perm_names as $perm_name) {
            $this->assertArrayHasKey($perm_name, $perms);
        }
    }

    /**
     * Test adding the master user.
     */
    public function testAddMasterUser()
    {
        Model_Nodeworx_User::addMasterUser(self::TEST_MASTER_EMAIL, 'abc123');

        $User = Model_Nodeworx_User::getByEmail(self::TEST_MASTER_EMAIL);

        $this->assertInstanceOf('Model_Nodeworx_User', $User);
        $this->assertEquals(self::TEST_MASTER_EMAIL, $User->email);

        $actual_permissions   = $User->getPermissions();
        $expected_permissions = Model_Nodeworx_Permission::find();

        foreach ($expected_permissions as $Perm) {
            $this->assertArrayHasKey($Perm->short_name, $actual_permissions);
        }
    }
}
