<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Apr 6, 2010
 */

/**
 * Tests for banner notices.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Notice_BannerTest extends IWorx_Framework_TestCase
{
    const TEST_CODE = 'UNITTEST_CODE';

    /**
     * Clears bullshits if prior tests failed.
     */
    public static function setUpBeforeClass()
    {
        $User = self::getMasterNodeWorx()->getWorkingUser();

        Notice_Util::clearIgnores($User->getGlobalId());
        Notice_Util::clearUnsubscribe(self::TEST_CODE, $User->getGlobalId());
        Notice_Util::dismissCode(self::TEST_CODE);
    }

    /**
     * Gets the count of notices.
     *
     * @return integer
     */
    private function _getNoticeCount()
    {
        $NW = self::getMasterNodeWorx();
        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Notice',
            'listBannerNotices',
            array(),
            $NW
        );

        return count($Reply->getPayloadData());
    }

    /**
     * Tests delivery of a banner notice.
     */
    public function testNoticeDelivery()
    {
        $original_count = $this->_getNoticeCount();

        $Notice = new Notice('Test notice', 'Test Notice!  Whoo!');
        $User   = self::getMasterNodeWorx()->getWorkingUser();

        $Recipient = new Notice_Recipient_User($User);
        $Recipient->addTransport(new Notice_Transport_Banner());

        $Notice->addRecipient($Recipient);
        $Notice->send();

        $new_count = $this->_getNoticeCount();

        $this->assertEquals(
            $original_count + 1,
            $new_count,
            'The notice was not delivered'
        );
    }

    /**
     * Tests that a notice is dismissed.
     *
     * @depends testNoticeDelivery
     */
    public function testNoticeDismissal()
    {
        $original_count = $this->_getNoticeCount();

        $User = self::getMasterNodeWorx()->getWorkingUser();

        $Search = new Notice_Search();
        $Search->addSelectableField('delivery_id');
        $Search->setOptions(array( 'global_uid'        => $User->getGlobalId(),
                                'subject'           => 'Test notice',
                                'acknowledged_date' => 'empty' ));
        $results = $Search->search(IWorx_Search_Engine::FETCH_COLUMN);

        $this->assertCount(1, $results);

        Notice_Util::dismiss($results[0]);

        $new_count = $this->_getNoticeCount();

        $this->assertEquals(
            $original_count - 1,
            $new_count,
            'The notice was not dismissed'
        );
    }

    /**
     * Tests that a notice cannot be added multiple times if noDuplicates is set.
     */
    public function testNoDuplicates()
    {
        $original_count = $this->_getNoticeCount();

        $Notice = new Notice('Test duplicate notice', 'Testing duplicates');
        $User   = self::getMasterNodeWorx()->getWorkingUser();

        $Recipient = new Notice_Recipient_User($User);
        $Recipient->addTransport(new Notice_Transport_Banner());

        $Notice->addRecipient($Recipient);
        $Notice->noDuplicates();
        $Notice->setCode(self::TEST_CODE);
        $Notice->send();

        $new_count = $this->_getNoticeCount();

        $this->assertEquals(
            $original_count + 1,
            $new_count,
            'The notice was not delivered'
        );

        $Notice->send();

        $final_count = $this->_getNoticeCount();

        $this->assertEquals(
            $original_count + 1,
            $final_count
        );
    }

    /**
     * Tests ignoring a code.
     *
     * @depends testNoDuplicates
     */
    public function testCodeIgnore()
    {
        $original_count = $this->_getNoticeCount();

        $User = self::getMasterNodeWorx()->getWorkingUser();

        Notice_Util::ignoreCodeUntil(
            $User->getGlobalId(),
            self::TEST_CODE,
            Node::getNodeID(),
            time() + 600
        );

        $new_count = $this->_getNoticeCount();

        $this->assertEquals(
            $original_count - 1,
            $new_count,
            'The notice was not ignored'
        );

        Notice_Util::clearIgnores($User->getGlobalId());

        $final_count = $this->_getNoticeCount();

        $this->assertEquals($original_count, $final_count, 'Ignore not cleared');
    }

    /**
     * Tests dismissing a code.
     *
     * @depends testNoDuplicates
     */
    public function testCodeDismissal()
    {
        $original_count = $this->_getNoticeCount();

        Notice_Util::dismissCode(self::TEST_CODE);

        $new_count = $this->_getNoticeCount();

        $this->assertEquals(
            $original_count - 1,
            $new_count,
            'The notice was not dismissed'
        );
    }

    /**
     * Tests unsubscribing a code.
     */
    public function testCodeUnsubscribe()
    {
        $original_count = $this->_getNoticeCount();

        $Notice = new Notice('Test unsubscribe', 'Testing Unsubscribe');
        $User   = self::getMasterNodeWorx()->getWorkingUser();

        $Recipient = new Notice_Recipient_User($User);
        $Recipient->addTransport(new Notice_Transport_Banner());

        $Notice->addRecipient($Recipient);
        $Notice->noDuplicates();
        $Notice->setCode(self::TEST_CODE);

        Notice_Util::unsubscribe(self::TEST_CODE, $User->getGlobalId());

        $Notice->send();

        $new_count = $this->_getNoticeCount();

        Notice_Util::dismissCode(self::TEST_CODE);

        $this->assertEquals(
            $original_count,
            $new_count,
            'The notice was delivered despite the unsubscription'
        );

        Notice_Util::clearUnsubscribe(self::TEST_CODE, $User->getGlobalId());
    }

    /**
     * Tests unsubscribing a code via the controllers.
     */
    public function testCodeUnsubscribeViaControllersNW()
    {
        $original_count = $this->_getNoticeCount();

        $Notice = new Notice('Test unsubscribe', 'Testing Unsubscribe');
        $User   = self::getMasterNodeWorx()->getWorkingUser();

        $Recipient = new Notice_Recipient_User($User);
        $Recipient->addTransport(new Notice_Transport_Banner());

        $Notice->addRecipient($Recipient);
        $Notice->noDuplicates();
        $Notice->setCode(self::TEST_CODE);

        $NW = self::getMasterNodeWorx();

        $Reply = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Notice',
            'unsubscribeCommit',
            array( 'code' => self::TEST_CODE ),
            $NW
        );

        $Notice->send();

        $new_count = $this->_getNoticeCount();

        Notice_Util::dismissCode(self::TEST_CODE);

        $this->assertEquals(
            $original_count,
            $new_count,
            'The notice was delivered despite the unsubscription'
        );

        Notice_Util::clearUnsubscribe(self::TEST_CODE, $User->getGlobalId());
    }
}
