<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Mar 30, 2010
 * @id $Id$
 */

/**
 * Tests for Notice Recipients.
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Notice_RecipientTest extends IWorx_Framework_TestCase
{
    /**
     * Data Provider.
     *
     * @return array
     */
    public static function dataProviderTargets()
    {
        return array(
                 array( '1', 'Notice_Recipient_AllUsers' ),
                 array( '2', 'Notice_Recipient_NWUsers' ),
                 array( '3', 'Notice_Recipient_Resellers' ),
                 array( '4', 'Notice_Recipient_SwUsers' ),
                 array( '5', 'Notice_Recipient_DomainOwner', self::SW_DOMAIN ),
                 array( '6', 'Notice_Recipient_SiteworxOwner', self::SW_DOMAIN ),
                 array( 'tcollege@interworx.com', 'Notice_Recipient_Email' ),
                 array( 'nw1', 'Notice_Recipient_User' ),
                 );
    }

    /**
     * Tests the Notice_Recipient_User class.
     */
    public function testNoticeRecipientUser()
    {
        $User = new NodeWorx_User(NodeWorx_User::MASTER_ID);

        $Recipient = new Notice_Recipient_User($User);

        $recipient_users = $Recipient->getUsers();

        $this->assertCount(1, $recipient_users);

        $FirstUser = array_pop($recipient_users);

        $this->assertEquals($User, $FirstUser);
    }

    /**
     * Tests that the NwUsers only include users from the master nodeworx.
     */
    public function testNoticeRecipientNwUsers()
    {
        $Recipient = new Notice_Recipient_NwUsers();

        $recipient_users = $Recipient->getUsers();

        foreach ($recipient_users as $User) {
            $this->assertEquals(NodeWorx::MASTER_ID, $User->getNodeWorxId());
        }
    }

    /**
     * Tests that the Resellers do not include users from the master nodeworx.
     */
    public function testNoticeRecipientResellers()
    {
        $this->addResellerIfNecessary();

        $Recipient = new Notice_Recipient_Resellers();

        $recipient_users = $Recipient->getUsers();

        foreach ($recipient_users as $User) {
            $this->assertNotEquals(NodeWorx::MASTER_ID, $User->getNodeWorxId());
        }
    }

    /**
     * Tests recipient NW account.
     */
    public function testNoticeRecipientNwAccount()
    {
        $this->addResellerIfNecessary();

        $reseller_id = self::getResellerId();

        $Recipient = new Notice_Recipient_NwAccount($reseller_id);

        foreach ($Recipient->getUsers() as $User) {
            $this->assertEquals($reseller_id, $User->getNodeWorxId());
        }
    }

    /**
     * Stupid test of getting all SW users.
     */
    public function testNoticeRecipientSwUsers()
    {
        $this->setMaxRunningTime(self::QUICK_TEST_TIME_LIMIT * 3);
        self::addSWAccountIfNecessary();

        $Recipient = new Notice_Recipient_SwUsers();

        $this->assertGreaterThan(0, count($Recipient->getUsers()));

        foreach ($Recipient->getUsers() as $User) {
            $this->assertGreaterThan(0, $User->getId());
        }
    }

    /**
     * Tests getting recipients of a single siteworx account.
     */
    public function testNoticeRecipientSwAccount()
    {
        $SW = $this->getSWAccount();

        $Recipient = new Notice_Recipient_SwAccount($SW->getId());

        $this->assertGreaterThan(0, count($Recipient->getUsers()));

        foreach ($Recipient->getUsers() as $User) {
            $this->assertEquals($SW->getId(), $User->getSiteWorxId());
        }
    }

    /**
     * Shitty test of reseller siteworx users.
     */
    public function testNoticeRecipientResellerSwUsers()
    {
        $this->setMaxRunningTime(self::QUICK_TEST_TIME_LIMIT * 3);
        $this->deleteResellerAccountIfNecessary();
        $this->addResellerSwAccountIfNecessary();

        $id = self::getResellerId();

        $Recipient = new Notice_Recipient_ResellerSwUsers($id);

        $this->assertGreaterThan(0, count($Recipient->getUsers()));

        foreach ($Recipient->getUsers() as $User) {
            $this->assertGreaterThan(0, $User->getId());
        }
    }

    /**
     * Tests that the NodeWorx master, the reseller, and siteworx accounts are return.
     */
    public function testNoticeRecipientAllUsers()
    {
        $this->addResellerIfNecessary();

        $reseller_id = self::getResellerId();

        $Recipient = new Notice_Recipient_AllUsers();

        $found_master   = false;
        $found_reseller = false;
        $found_siteworx = false;

        foreach ($Recipient->getUsers() as $User) {
            if ($User->getId() === NodeWorx_User::MASTER_ID &&
            $User instanceof NodeWorx_User ) {
                $found_master = true;
            }
            if ($User instanceof NodeWorx_User &&
            $User->getNodeWorxId() === (string) $reseller_id ) {
                $found_reseller = true;
            }
            if ($User instanceof SiteWorx_User) {
                $found_siteworx = true;
            }
        }

        $this->assertTrue($found_master, 'Unable to find master user in output');
        $this->assertTrue($found_reseller, 'Unable to find reseller user in output');
        $this->assertTrue($found_siteworx, 'Unable to find SiteWorx user in output');
    }

    /**
     * Tests resolution of targets.
     *
     * @param string $target
     * @param string $expected
     * @param string $domain
     *
     * @dataProvider dataProviderTargets
     */
    public function testTargetResolution($target, $expected, $domain = null)
    {
        $Recipient = Notice_Recipient::resolveTarget($target, $domain);

        $this->assertInstanceOf($expected, $Recipient);
    }
}
