<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Jan 17, 2020
 */

class SiteworxDbMapperTest extends IWorx_Framework_TestCase
{
    const EXISTING = true;
    const NONEXISTING = false;

    private $_existingDbs = ['existingDatabase1', 'existingDatabase2'];
    private $_nonexistingDbs = ['nonexistingDatabase1', 'nonexistingDatabase2'];

    private $_existingUsers = ['existingUser1', 'existingUser2'];
    private $_nonexistingUsers = ['nonexistingUser1', 'nonexistingUser2'];

    private $_serverDbs = ['mapped1', 'mapped2', 'mapped3', 'unmapped1', 'unmapped2', 'unmapped3', 'mysql', 'performance_schema'];
    private $_mappedDbs = ['mapped1', 'mapped2', 'mapped3'];
    private $_expectedUnmappedDbs = ['unmapped1', 'unmapped2', 'unmapped3'];


    private $_mappedUsers = ['mapped1', 'mapped2', 'mapped3'];
    private $_serverUsers = ['mapped1', 'mapped2', 'mapped3', 'unmapped1', 'unmapped2', 'unmapped3', 'iworx', 'root'];
    private $_expectedUnmappedUsers = ['unmapped1', 'unmapped2', 'unmapped3'];

    /**
     * Test mapDbs action.
     * Should throw exception if sw id does not exist.
     */
    public function testMapDbsShouldThrowExceptionNonexistingSW()
    {
        $sut = $this->_getSutMock();
        $SW = $this->_getSwMock(self::NONEXISTING);

        $this->_expectIworxException();
        $sut->mapDbs($SW, $this->_existingDbs);
    }

    /**
     * Test mapDbs action.
     * Should throw exception if databases do not exist.
     */
    public function testMapDbsShouldThrowExceptionNonexistingDatabases()
    {
        $SW = $this->_getSwMock(self::EXISTING);
        $sut = $this->_getSutMock();

        $this->_expectIworxException();
        $sut->mapDbs($SW, $this->_nonexistingDbs);
    }

    /**
     * Test mapDbs action.
     * Should add db if they exist.
     */
    public function testMapDbsShouldAddExistingDbs()
    {
        $SW = $this->_getSwMock(self::EXISTING);
        $sut = $this->_getSutMock();
        $dbAmount = count($this->_existingDbs);

        // expecting sut to loop through array of
        // dbs to map & call addDb with valid args
        $sut->expects($this->exactly($dbAmount))
            ->method('addDb')
            ->withConsecutive(
                array($SW, $this->_existingDbs[0]),
                array($SW, $this->_existingDbs[1])
            );

        $sut->mapDbs($SW, $this->_existingDbs);
    }

    /**
     * Tets mapDbUsers.
     * Should throw exception if sw account does not exist.
     */
    public function testMapDbUsersShouldThrowExceptionNonexistingSW()
    {
        $SW = $this->_getSwMock(self::NONEXISTING);
        $sut = $this->_getSutMock();

        $this->_expectIworxException();
        $sut->mapDbUsers($SW, $this->_existingUsers);
    }

    /**
     * Test mapDbUsers.
     * Should throw exception if db users do not exist.
     */
    public function testMapDbUsersShouldThrowExceptionNonexistingDbUsers()
    {
        $SW = $this->_getSwMock(self::EXISTING);
        $sut = $this->_getSutMock();

        $this->_expectIworxException();
        $sut->mapDbUsers($SW, $this->_nonexistingUsers);
    }

    /**
     * Test mapDbUsers.
     * Should add users to sw account if they exist.
     */
    public function testMapDbUsersShouldAddExistingDbUsers()
    {
        $SW = $this->_getSwMock(self::EXISTING);
        $sut = $this->_getSutMock();
        $dbAmount = count($this->_existingUsers);

        $sut->expects($this->exactly($dbAmount))
            ->method('addUser')
            ->withConsecutive(
                array($SW, $this->_existingUsers[0]),
                array($SW, $this->_existingUsers[1])
            );

        $sut->mapDbUsers($SW, $this->_existingUsers);
    }

    /**
     * Test getUnmappedDatabases.
     * Should return dbs not mapped to an sw account and filter out mysql & performance_schema.
     */
    public function testGetUnmappedDatabasesShouldReturnMappableDbs()
    {
        $Driver = $this->_getMysqlMock();

        // mock private method that performs sql query
        $sut = $this->_getSutMock();

        // expecting 'mysql' & 'performance_schema' to be filtered out from result.
        // expecting numerically indexed array - test will fail index is off.
        $actual = $sut->getUnmappedDatabases($Driver);
        $this->assertEquals($this->_expectedUnmappedDbs, $actual);
    }

    /**
     * Test getUnmappedUsers.
     * Should return users not mapped to an sw account and filter out iworx and root.
     */
    public function testGetUnmappedUsersShouldReturnMappableUsers()
    {
        $Driver = $this->_getMysqlMock();
        // mock private method that performs sql query

        $sut = $this->_getSutMock();

        // expecting 'mysql' & 'performance_schema' to be filtered out from result.
        // expecting numerically indexed array - test will fail index is off.
        $actual = $sut->getUnmappedDatabaseUsers($Driver);
        $this->assertEquals($this->_expectedUnmappedUsers, $actual);
    }

    /**
     * Partially mocking the sut itself to avoid any error/exception
     * handling when addDb or addUser is called in mapDbs/mapDbUsers.
     *
     * @return MockObject
     */
    private function _getSutMock()
    {
        $methodsToMock = array('addDb', 'addUser', '_getMappedDatabases', '_getMappedUsers');
        $mock = $this->getMockBuilder(SiteworxDbMapper::class)
            ->setMethods($methodsToMock)
            ->disableOriginalConstructor()
            ->getMock();

        $mock->expects($this->any())
            ->method('_getMappedUsers')
            ->willReturn($this->_mappedUsers);

        $mock->expects($this->any())
            ->method('_getMappedDatabases')
            ->willReturn($this->_mappedDbs);

        return $mock;
    }

    /**
     * Used to verify whether or not SW user exists, and to retrieve MySQL instance.
     * Mocking MySQL by controlling getMysqlDriver return value.
     *
     * @param boolean $exists
     * @return MockObject
     */
    private function _getSwMock($exists)
    {
        $mysql = $this->_getMysqlMock();

        $mock = $this->createMock(SiteWorx::class);
        $mock->expects($this->once())
            ->method('exists')
            ->willReturn($exists);

        $mock->expects($this->any())
            ->method('getMySQLDriver')
            ->willReturn($mysql);

        return $mock;
    }

    /**
     * Used internally to verify whether or not dbs/users exists.
     *
     * @return MockObject
     */
    private function _getMysqlMock()
    {
        $mock = $this->createMock(MySQL::class);

        // first param = expected argument,
        // second param = return value.
        $dbsMap = array(
            [$this->_existingDbs[0], true],
            [$this->_existingDbs[1], true],
            [$this->_nonexistingDbs[0], false],
            [$this->_nonexistingDbs[1], false]
        );

        $usersMap = array(
            [$this->_existingUsers[0], true],
            [$this->_existingUsers[1], true],
            [$this->_nonexistingUsers[0], false],
            [$this->_nonexistingUsers[1], false]
        );

        $mock->expects($this->any())
            ->method('systemDatabaseExistsFQ')
            ->will($this->returnValueMap($dbsMap));

        $mock->expects($this->any())
            ->method('systemDatabaseUserExistsFQ')
            ->will($this->returnValueMap($usersMap));

        $mock->expects($this->any())
            ->method('getSystemDatabasesLike')
            ->with('')
            ->willReturn($this->_serverDbs);

            $mock->expects($this->any())
            ->method('getSystemDatabaseUsersLike')
            ->willReturn($this->_serverUsers);

        return $mock;
    }

    /**
     * Expect exception of type IWorx_Exception.
     *
     * @return mixed
     */
    private function _expectIworxException()
    {
        return $this->expectException(IWorx_Exception::class);
    }
}
