<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2011 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @subpackage Tests
 * @author     Tim College <tcollege@interworx.com>
 * @date       Mar 29, 2011
 */

/**
 * Tests for the Tinydns encoder
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class TinydnsEncoderTest extends IWorx_Framework_TestCase
{
    /**
     * Gets a stdClass object for use with the TinydnsEncoder.
     *
     * @param Dns_Record $Record
     * @return object
     */
    private function _getStdClassDnsRec(Dns_Record $Record)
    {
        $dnsrec = (object) null;

        $dnsrec->host   = $Record->getHost();
        $dnsrec->ttl    = $Record->getTtl();
        $dnsrec->type   = $Record->getType();
        $dnsrec->target = $Record->getRdata();

        return $dnsrec;
    }

    /**
     * Helper function to encode and assert against expected.
     *
     * @param Dns_Record $Record
     * @param string     $expected
     */
    private function _checkEncode(Dns_Record $Record, $expected)
    {
        $actual = TinydnsEncoder::encode($this->_getStdClassDnsRec($Record));
        $this->assertEquals($expected, $actual);
    }

    /**
     * Tests the encoding of A records.
     */
    public function testEncodeA()
    {
        $Record = new Dns_Record_A();

        $Record->setHost('fake.com');
        $Record->setIpAddress('1.2.3.4');
        $Record->setTtl(43200);

        $expected = '+fake.com:1.2.3.4:43200';

        $this->_checkEncode($Record, $expected);
    }

    /**
     * Tests the encoding of a AAAA record.
     */
    public function testEncodeAAAA()
    {
        $Record = new Dns_Record_AAAA();

        $Record->setHost('fake.com');
        $Record->setTtl(43200);
        $Record->setIpAddress('2001:1234:5678:dead:beef:0000:0000:4321');

        $expected = ':fake.com:28:\040\001\022\064\126\170\336\255\276\357\000\000\000\000\103\041:43200';

        $this->_checkEncode($Record, $expected);
    }

    /**
     * Tests encoding of CNAME records.
     */
    public function testEncodeCNAME()
    {
        $Record = new Dns_Record_CNAME();

        $Record->setHost('ftp.fake.com');
        $Record->setTtl(43200);
        $Record->setAlias('fake.com');

        $expected = 'Cftp.fake.com:fake.com:43200';

        $this->_checkEncode($Record, $expected);
    }

    /**
     * Tests the encoding of MX records.
     */
    public function testEncodeMX()
    {
        $Record = new Dns_Record_MX();

        $Record->setHost('fake.com');
        $Record->setTtl(43200);
        $Record->setPreference(20);
        $Record->setDomain('mail.google.com');

        $expected = '@fake.com::mail.google.com:20:43200';

        $this->_checkEncode($Record, $expected);
    }

    /**
     * Tests the encoding of NS records.
     */
    public function testEncodeNS()
    {
        $Record = new Dns_Record_NS();

        $Record->setHost('fake.com');
        $Record->setTtl(43200);
        $Record->setNameserver('ns1.fake.com');

        $expected = '&fake.com::ns1.fake.com:43200';

        $this->_checkEncode($Record, $expected);
    }

    /**
     * Tests the encoding of IPv4 PTR records.
     */
    public function testEncodePTRIPv4()
    {
        $Record = new Dns_Record_PTR();

        $Record->setHost('4.3.2.1.in-addr.arpa');
        $Record->setDomain('fake.com');
        $Record->setTtl(43200);

        $expected = '^4.3.2.1.in-addr.arpa:fake.com:43200';

        $this->_checkEncode($Record, $expected);
    }

    /**
     * Tests the encoding of IPv6 PTR records.
     */
    public function testEncodePTRIPv6()
    {
        $Record = new Dns_Record_PTR();

        $Record->setHost('1.2.3.4.0.0.0.0.0.0.0.0.f.e.e.b.d.a.e.d.8.7.6.5.4.3.2.1.1.0.0.2.ip6.arpa');
        $Record->setDomain('fake.com');
        $Record->setTtl(43200);

        $expected = '^1.2.3.4.0.0.0.0.0.0.0.0.f.e.e.b.d.a.e.d.8.7.6.5.4.3.2.1.1.0.0.2.ip6.arpa:fake.com:43200';

        $this->_checkEncode($Record, $expected);
    }

    /**
     * Tests the encoding of a simple SPF record.
     */
    public function testEncodeSPF()
    {
        $Record = new Dns_Record_SPF();

        $Record->setUseMX(true);
        $Record->setHost('fake.com');
        $Record->setAll('-all');
        $Record->setTtl(43200);

        $expected = ':fake.com:16:\016v=spf1\040mx\040-all:43200';

        $this->_checkEncode($Record, $expected);
    }

    /**
     * Tests encoding of TXT records.
     */
    public function testEncodeTXT()
    {
        $Record = new Dns_Record_TXT();

        $Record->setHost('fake.com');
        $Record->setTtl(43200);
        $Record->setText('A text record of some sort');

        $expected = ':fake.com:16:\032A\040text\040record\040of\040some\040sort:43200';

        $this->_checkEncode($Record, $expected);
    }

    /**
     * Tests encoding of TXT records.
     */
    public function testEncodeTXTWithNewlines()
    {
        $Record = new Dns_Record_TXT();

        $Record->setHost('fake.com');
        $Record->setTtl(43200);
        $Record->setText('A text record
of some sort');

        $expected = ':fake.com:16:\032A\040text\040record\012of\040some\040sort:43200';

        $this->_checkEncode($Record, $expected);
    }

    /**
     * Tests encoding of SOA records.
     */
    public function testEncodeSOA()
    {
        $Record = new Dns_Record_SOA();

        $Record->setHost('fake.com');
        $Record->setTtl(43200);
        $Record->setEmail('contact@fake.com');
        $Record->setRefresh(7200);
        $Record->setRetry(2048);
        $Record->setExpiration(1048576);
        $Record->setNegativeTtl(21600);
        $Record->setNameserver('ns1.fake.com');

        $dnsrec = $this->_getStdClassDnsRec($Record);
        $dnsrec->serial = '1301422154';

        $actual   = TinydnsEncoder::encode($dnsrec);
        $expected = 'Zfake.com:ns1.fake.com:contact.fake.com:1301422154:7200:2048:1048576:21600';

        $this->assertEquals($expected, $actual);
    }

    /**
     * Tests the encoding of an SRV record.
     */
    public function testEncodeSRV()
    {
        $Record = new Dns_Record_SRV();

        $Record->setService('_derp');
        $Record->setProtocol('_asdf');
        $Record->setDomain('fake.com');
        $Record->setPort(5060);
        $Record->setPriority(10);
        $Record->setWeight(100);
        $Record->setTarget('target.com');
        $Record->setTtl(43200);

        $expected = ':_derp._asdf.fake.com:33:\000\012\000\144\023\304\006target\003com\000:43200';

        $this->_checkEncode($Record, $expected);
    }

    /**
     * Tests the encoding of an NAPTR record.
     */
    public function testEncodeNAPTR()
    {
        $Record = new Dns_Record_NAPTR();

        $Record->setHost('fake.com');
        $Record->setTtl(43200);
        $Record->setOrder(10);
        $Record->setPreference(100);
        $Record->setFlag('S');
        $Record->setService('http');
        $Record->setRegex('[a-z]');
        $Record->setReplacement('replacement');

        $expected = ':fake.com:35:\000\012\000\144\001S\004http\005[a-z]\013replacement\000:43200';

        $this->_checkEncode($Record, $expected);
    }

    /**
     * Tests the encoding of an CAA record.
     */
    public function testEncodeCAA()
    {
        $Record = new Dns_Record_CAA();

        $Record->setHost('example.com');
        $Record->setTtl(43200);
        $Record->setFlags('0');
        $Record->setTag('issue');
        $Record->setValue('ca.example.net');

        $expected = ':example.com:257:\000\005issueca.example.net:43200';

        $this->_checkEncode($Record, $expected);
    }
}
