<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Dec 01, 2020
 */

/**
 * Test Suite for Upgrade_Prep_ModifiedVhosts
 *
 * @package    InterWorx
 * @subpackage Tests
 */
class Upgrade_Prep_ModifiedVhostsTest extends IWorx_Framework_TestCase
{
    const IGNORE_FLAG = 'upgrade-prep-7-has-custom-vhosts-ignore';

    /**
     * expects getName to return reference from lang file.
     */
    public function testGetNameShouldReturnLangReference()
    {
        $SUT = new Upgrade_Prep_ModifiedVhosts();

        $expected = '##LG_VHOST_NOT_CUSTOMIZED##';
        $actual = $SUT->getName();

        $this->assertSame($expected, $actual);
    }

    /**
     * expects isIgnorable to return true.
     */
    public function testIsIgnorableShouldReturnTrue()
    {
        $SUT = new Upgrade_Prep_ModifiedVhosts();
        $isIgnorable = $SUT->isIgnorable();

        $this->assertTrue($isIgnorable);
    }

    /**
     * expects ignore to add ignore flag & make isIgnored return true
     */
    public function testIgnore()
    {

        $SUT = new Upgrade_Prep_ModifiedVhosts();

        //ensure not ignored before
        //testing ignore function
        IW::Env()->clearEnvFlag(self::IGNORE_FLAG);
        $this->assertFalse($SUT->isIgnored());

        $SUT->ignore();

        $this->assertTrue($SUT->isIgnored());
        $this->assertTrue(IW::Env()->getEnvFlag(self::IGNORE_FLAG));
    }

    /**
     * expects remove ignore to remove env flag & make isIgnored return false
     */
    public function testRemoveIgnore()
    {
        $SUT = new Upgrade_Prep_ModifiedVhosts();

        // ensure ignored before testing
        // removeIgnore function
        IW::Env()->setEnvFlag(self::IGNORE_FLAG);
        $this->assertTrue($SUT->isIgnored());

        $SUT->removeIgnore();

        $this->assertFalse($SUT->isIgnored());
        $this->assertFalse(IW::Env()->getEnvFlag(self::IGNORE_FLAG));
    }

    /**
     * expects check to return true when
     * no vhosts exists.
     */
    public function testCheckShouldSucceedWithNoVhosts()
    {
        $SUT = new Upgrade_Prep_ModifiedVhosts();
        $this->assertTrue($SUT->check());
    }

    /**
     * expects check to return true when
     * default vhost exists.
     */
    public function testCheckShouldSucceedWithDefaultVhosts()
    {
        $SUT = new Upgrade_Prep_ModifiedVhosts();

        $this->addSWAccountIfNecessary();
        $this->assertTrue($SUT->check());
    }

    /**
     * exoects check to return false when
     * vhost has been modified.
     */
    public function testCheckShouldFailWithModifiedVhosts()
    {
        $SUT = new Upgrade_Prep_ModifiedVhosts();

        $this->addSWAccountIfNecessary();
        $this->_modifySwAccountVhost();

        $this->assertFalse($SUT->check());
    }

    /**
     *
     */
    private function _modifySwAccountVhost()
    {
        $filePath = '/etc/httpd/conf.d/vhost_addedbytest.com.conf';
        $content = file_get_contents($filePath);
        $modifiedContent = str_replace('RewriteEngine On', 'RewriteEngine Off', $content);
        file_put_contents($filePath, $modifiedContent);
    }

    /**
     * Test that check returns true
     * with default vhost & mod proxy fcgi enabled
     */
    public function testCheckShouldSucceedWithFcgiEnabled()
    {

        IW::Env()->setConfig('FPM_USE_MOD_PROXY_FCGI', '1');

        $SUT = new Upgrade_Prep_ModifiedVhosts();

        $this->addSWAccountIfNecessary();
        $this->assertTrue($SUT->check());
    }

    /**
     * test check returns true when
     * mod suphp is enabled
     */
    public function testCheckShouldSucceedWithModSuphp()
    {
        $this->_setPhpMode(PHP::MODE_MOD_SUPHP);

        $SUT = new Upgrade_Prep_ModifiedVhosts();
        $this->addSWAccountIfNecessary();
        $this->assertTrue($SUT->check());
    }

    /**
     * test check returns true when
     * mod php is enabled
     */
    public function testCheckShouldSucceedWithModPhp()
    {
        $this->_setPhpMode(PHP::MODE_MOD_PHP);

        $SUT = new Upgrade_Prep_ModifiedVhosts();
        $this->addSWAccountIfNecessary();
        $this->assertTrue($SUT->check());
    }

    /**
     * method to set PHP mode.
     *
     * @param string $mode
     * @return Reply
     */
    private function _setPhpMode($mode)
    {
        return IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Http',
            'updatePhpModeCommit',
            ['php_mode' => $mode],
            self::getMasterNodeWorx()
        );
    }

    /**
     * delete sw account after each test & make sure php mode is reset.
     */
    public function tearDown()
    {
        $this->deleteSWAccountIfNecessary();

        if (PHP::getPhpMode() !== PHP::MODE_PHP_FPM) {
            $this->_setPhpMode(PHP::MODE_PHP_FPM);
        }
    }
}
